@@ -18,46 +18,47 @@ from typing import Any
 from typing import Optional
 
 import logging
+import time
 
 import nbxmpp
-from nbxmpp.client import Client as NBXMPPClient
-from nbxmpp.protocol import JID
-from nbxmpp.const import StreamError
-from nbxmpp.const import ConnectionType
-
-from gi.repository import GLib
 from gi.repository import Gio
+from gi.repository import GLib
 from gi.repository import GObject
 from gi.repository import Gtk
+from nbxmpp.client import Client as NBXMPPClient
+from nbxmpp.const import ConnectionType
+from nbxmpp.const import StreamError
+from nbxmpp.protocol import JID
 
 from gajim.common import app
 from gajim.common import helpers
 from gajim.common import modules
 from gajim.common import passwords
+from gajim.common.const import ClientState
+from gajim.common.const import SimpleClientState
 from gajim.common.events import AccountConnected
-from gajim.common.events import MessageSent
 from gajim.common.events import AccountDisconnected
+from gajim.common.events import MessageNotSent
+from gajim.common.events import MessageSent
+from gajim.common.events import Notification
 from gajim.common.events import PasswordRequired
 from gajim.common.events import PlainConnection
 from gajim.common.events import SignedIn
-from gajim.common.events import Notification
 from gajim.common.events import StanzaReceived
 from gajim.common.events import StanzaSent
-from gajim.common.const import ClientState
-from gajim.common.const import SimpleClientState
-from gajim.common.helpers import get_custom_host
 from gajim.common.helpers import get_account_proxy
-from gajim.common.helpers import warn_about_plain_connection
-from gajim.common.helpers import get_resource
+from gajim.common.helpers import get_custom_host
 from gajim.common.helpers import get_idle_status_message
+from gajim.common.helpers import get_resource
 from gajim.common.helpers import Observable
-from gajim.common.idle import Monitor
-from gajim.common.idle import IdleMonitorManager
+from gajim.common.helpers import warn_about_plain_connection
 from gajim.common.i18n import _
+from gajim.common.idle import IdleMonitorManager
+from gajim.common.idle import Monitor
 from gajim.common.structs import OutgoingMessage
+from gajim.common.util.http import create_http_session
 
-from gajim.gui.util import open_window
-
+from gajim.gtk.util import open_window
 
 log = logging.getLogger('gajim.client')
 
@@ -74,7 +75,6 @@ class Client(Observable):
         self._hostname = app.settings.get_account_setting(self._account,
                                                           'hostname')
         self._user = app.settings.get_account_setting(self._account, 'name')
-        self.password = None
 
         self._priority = 0
         self._connect_machine_calls = 0
@@ -166,16 +166,7 @@ class Client(Observable):
         self._client.set_domain(self._hostname)
         self._client.set_username(self._user)
         self._client.set_resource(get_resource(self._account))
-
-        pass_saved = app.settings.get_account_setting(self._account, 'savepass')
-        if pass_saved:
-            # Request password from keyring only if the user chose to save
-            # his password
-            self.password = passwords.get_password(self._account)
-
-        self._client.set_password(self.password)
-        self._client.set_accepted_certificates(
-            app.cert_store.get_certificates())
+        self._client.set_http_session(create_http_session())
 
         self._client.subscribe('resume-failed', self._on_resume_failed)
         self._client.subscribe('resume-successful', self._on_resume_successful)
@@ -272,9 +263,7 @@ class Client(Observable):
             self._destroy_client = True
 
             if error in ('not-authorized', 'no-password'):
-                def _on_password(password: str) -> None:
-                    self.password = password
-                    self._client.set_password(password)
+                def _on_password() -> None:
                     self._prepare_for_connect()
 
                 app.ged.raise_event(PasswordRequired(client=self,
@@ -341,6 +330,10 @@ class Client(Observable):
         app.ged.raise_event(StanzaReceived(account=self._account,
                                            stanza=stanza))
 
+    def is_own_jid(self, jid: JID | str) -> bool:
+        own_jid = self.get_own_jid()
+        return own_jid.bare_match(jid)
+
     def get_own_jid(self) -> JID:
         '''
         Return the last full JID we received on a bind event.
@@ -483,6 +476,22 @@ class Client(Observable):
             self._send_message(message)
             return
 
+        if method == 'OMEMO':
+            try:
+                self.get_module('OMEMO').encrypt_message(message)
+            except Exception:
+                log.exception('Error')
+                app.ged.raise_event(
+                    MessageNotSent(client=self._client,
+                                   jid=message.jid,
+                                   message=message.message,
+                                   error=_('Encryption error'),
+                                   time=time.time()))
+                return
+
+            self._send_message(message)
+            return
+
         # TODO: Make extension point return encrypted message
         extension = 'encrypt'
         if message.is_groupchat:
@@ -496,8 +505,9 @@ class Client(Observable):
         message.set_sent_timestamp()
         message.message_id = self.send_stanza(message.stanza)
 
+        log_line_id = None
         if not message.is_groupchat:
-            self.get_module('Message').log_message(message)
+            log_line_id = self.get_module('Message').log_message(message)
 
         app.ged.raise_event(
             MessageSent(jid=message.jid,
@@ -509,6 +519,7 @@ class Client(Observable):
                         label=message.label,
                         correct_id=message.correct_id,
                         message_id=message.message_id,
+                        msg_log_id=log_line_id,
                         play_sound=message.play_sound))
 
     def _prepare_for_connect(self) -> None:
@@ -534,6 +545,9 @@ class Client(Observable):
         if proxy is not None:
             self._client.set_proxy(proxy)
 
+        password = passwords.get_password(self._account)
+        self._client.set_password(password)
+
         self.connect()
 
     def connect(self, ignored_tls_errors: IgnoredTlsErrorsT = None) -> None:
@@ -544,6 +558,8 @@ class Client(Observable):
 
         log.info('Connect')
 
+        self._client.set_accepted_certificates(
+            app.cert_store.get_certificates())
         self._client.set_ignored_tls_errors(ignored_tls_errors)
         self._reconnect = True
         self._disable_reconnect_timer()
