@@ -15,23 +15,22 @@
 from __future__ import annotations
 
 from typing import Any
-from typing import Union
 from typing import NamedTuple
+from typing import Optional
+from typing import Union
 
-from enum import IntEnum
 from enum import Enum
+from enum import IntEnum
 from enum import unique
 from functools import total_ordering
 
 from gi.repository import Gio
-
-from nbxmpp.namespaces import Namespace
 from nbxmpp.const import PresenceShow
+from nbxmpp.namespaces import Namespace
 from nbxmpp.protocol import JID
 
 from gajim.common.i18n import _
-from gajim.common.i18n import Q_
-
+from gajim.common.i18n import p_
 
 STOP_EVENT = True
 PROPAGATE_EVENT = False
@@ -43,6 +42,18 @@ class EncryptionData(NamedTuple):
     additional_data: Any = None
 
 
+class EncryptionInfoMsg(Enum):
+    BAD_OMEMO_CONFIG = _('This chat’s configuration is unsuitable for '
+                         'encryption with OMEMO. To use OMEMO in this chat, '
+                         'it should be non-anonymous and members-only.')
+    NO_FINGERPRINTS = _('To send an encrypted message, you have to decide '
+                        'whether to trust the device of your contact.')
+    QUERY_DEVICES = _('No devices found to encypt this message to. '
+                      'Querying for devices now…')
+    UNDECIDED_FINGERPRINTS = _('There are devices for which you have not made '
+                               'a trust decision yet.')
+
+
 class Entity(NamedTuple):
     jid: JID
     node: str
@@ -215,7 +226,7 @@ class Display(Enum):
 
 
 class URIType(Enum):
-    UNKNOWN = 'unknown'
+    INVALID = 'invalid'
     XMPP = 'xmpp'
     MAIL = 'mail'
     GEO = 'geo'
@@ -223,12 +234,28 @@ class URIType(Enum):
     FILE = 'file'
     AT = 'at'
     TEL = 'tel'
+    OTHER = 'other'
 
 
-class URIAction(Enum):
+# https://xmpp.org/registrar/querytypes.html
+class XmppUriQuery(Enum):
+    NONE = ''
     MESSAGE = 'message'
     JOIN = 'join'
-    SUBSCRIBE = 'subscribe'
+
+    @staticmethod
+    def from_str(s: str) -> Optional[XmppUriQuery]:
+        try:
+            return XmppUriQuery(s)
+        except ValueError:
+            return None
+
+    @staticmethod
+    def from_str_or_none(s: str) -> XmppUriQuery:
+        qtype = XmppUriQuery.from_str(s)
+        if qtype:
+            return qtype
+        return XmppUriQuery.NONE
 
 
 class MUCJoinedState(Enum):
@@ -238,7 +265,7 @@ class MUCJoinedState(Enum):
     CREATING = 'creating'
     CAPTCHA_REQUEST = 'captcha in progress'
     CAPTCHA_FAILED = 'captcha failed'
-    PASSWORD_REQUEST = 'password request'
+    PASSWORD_REQUEST = 'password request'  # noqa: S105
 
     def __str__(self):
         return self.name
@@ -345,21 +372,21 @@ class CallType(Enum):
 
 
 MUC_CREATION_EXAMPLES = [
-    (Q_('?Group chat name:Team'),
-     Q_('?Group chat description:Project discussion'),
-     Q_('?Group chat address:team')),
-    (Q_('?Group chat name:Family'),
-     Q_('?Group chat description:Spring gathering'),
-     Q_('?Group chat address:family')),
-    (Q_('?Group chat name:Vacation'),
-     Q_('?Group chat description:Trip planning'),
-     Q_('?Group chat address:vacation')),
-    (Q_('?Group chat name:Repairs'),
-     Q_('?Group chat description:Local help group'),
-     Q_('?Group chat address:repairs')),
-    (Q_('?Group chat name:News'),
-     Q_('?Group chat description:Local news and reports'),
-     Q_('?Group chat address:news')),
+    (p_('Group chat name', 'Team'),
+     p_('Group chat description', 'Project discussion'),
+     p_('Group chat address', 'team')),
+    (p_('Group chat name', 'Family'),
+     p_('Group chat description', 'Spring gathering'),
+     p_('Group chat address', 'family')),
+    (p_('Group chat name', 'Vacation'),
+     p_('Group chat description', 'Trip planning'),
+     p_('Group chat address', 'vacation')),
+    (p_('Group chat name', 'Repairs'),
+     p_('Group chat description', 'Local help group'),
+     p_('Group chat address', 'repairs')),
+    (p_('Group chat name', 'News'),
+     p_('Group chat description', 'Local news and reports'),
+     p_('Group chat address', 'news')),
 ]
 
 
@@ -768,7 +795,7 @@ RFC5646_LANGUAGE_TAGS = {
     'uz-UZ': 'Uzbek (Latin) (Uzbekistan)',
     'uz-Cyrl-UZ': 'Uzbek (Cyrillic) (Uzbekistan)',
     'vi': 'Vietnamese',
-    'vi-VN': 'Vietnamese (Viet Nam)',
+    'vi-VN': 'Vietnamese (Vietnam)',
     'xh': 'Xhosa',
     'xh-ZA': 'Xhosa (South Africa)',
     'zh': 'Chinese',
@@ -894,11 +921,11 @@ COMMON_FEATURES = [
     Namespace.HASHES_BLAKE2B_512,
     Namespace.JINGLE,
     Namespace.JINGLE_FILE_TRANSFER_5,
-    Namespace.JINGLE_XTLS,
     Namespace.JINGLE_BYTESTREAM,
     Namespace.JINGLE_IBB,
     Namespace.AVATAR_METADATA + '+notify',
-    Namespace.MESSAGE_MODERATE
+    Namespace.MESSAGE_MODERATE,
+    Namespace.OMEMO_TEMP_DL + '+notify'
 ]
 
 
@@ -913,120 +940,35 @@ SHOW_LIST = [
 ]
 
 SHOW_STRING = {
-    'dnd': _('Busy'),
-    'xa': _('Not Available'),
-    'chat': _('Free for Chat'),
-    'online': Q_('?user status:Available'),
+    'dnd': p_('User status', 'Busy'),
+    'xa': p_('User status', 'Not Available'),
+    'chat': p_('User status', 'Free for Chat'),
+    'online': p_('User status', 'Available'),
     'connecting': _('Connecting'),
-    'away': _('Away'),
+    'away': p_('User status', 'Away'),
     'offline': _('Offline'),
 }
 
 SHOW_STRING_MNEMONIC = {
-    'dnd': _('_Busy'),
-    'xa': _('_Not Available'),
-    'chat': _('_Free for Chat'),
-    'online': Q_('?user status:_Available'),
+    'dnd': p_('User status', '_Busy'),
+    'xa': p_('User status', '_Not Available'),
+    'chat': p_('User status', '_Free for Chat'),
+    'online': p_('User status', '_Available'),
     'connecting': _('Connecting'),
-    'away': _('A_way'),
+    'away': p_('User status', 'A_way'),
     'offline': _('_Offline'),
 }
 
 GAJIM_FAQ_URI = 'https://dev.gajim.org/gajim/gajim/wikis/help/gajimfaq'
+GAJIM_PRIVACY_POLICY_URI = 'https://gajim.org/privacy/'
 GAJIM_WIKI_URI = 'https://dev.gajim.org/gajim/gajim/wikis'
 GAJIM_SUPPORT_JID = 'gajim@conference.gajim.org'
 
-URI_SCHEMES = {
-    'aaa://',
-    'aaas://',
-    'acap://',
-    'cap://',
-    'cid:',
-    'crid://',
-    'data:',
-    'dav:',
-    'dict://',
-    'dns:',
-    'fax:',
-    'file:/',
-    'ftp://',
-    'geo:',
-    'go:',
-    'gopher://',
-    'h323:',
-    'http://',
-    'https://',
-    'iax:',
-    'icap://',
-    'im:',
-    'imap://',
-    'info:',
-    'ipp://',
-    'iris:',
-    'iris.beep:',
-    'iris.xpc:',
-    'iris.xpcs:',
-    'iris.lwz:',
-    'ldap://',
-    'mailto:',
-    'mid:',
-    'modem:',
-    'msrp://',
-    'msrps://',
-    'mtqp://',
-    'mupdate://',
-    'news:',
-    'nfs://',
-    'nntp://',
-    'opaquelocktoken:',
-    'pop://',
-    'pres:',
-    'prospero://',
-    'rtsp://',
-    'service:',
-    'sip:',
-    'sips:',
-    'sms:',
-    'snmp://',
-    'soap.beep://',
-    'soap.beeps://',
-    'tag:',
-    'tel:',
-    'telnet://',
-    'tftp://',
-    'thismessage:/',
-    'tip://',
-    'tv:',
-    'urn://',
-    'vemmi://',
-    'xmlrpc.beep://',
-    'xmlrpc.beeps://',
-    'xmpp:',
-    'z39.50r://',
-    'z39.50s://',
-    'about:',
-    'apt:',
-    'cvs://',
-    'daap://',
-    'ed2k://',
-    'feed:',
-    'fish://',
-    'git://',
-    'iax2:',
-    'irc://',
-    'ircs://',
-    'ldaps://',
-    'magnet:',
-    'mms://',
-    'rsync://',
-    'ssh://',
-    'svn://',
-    'sftp://',
-    'smb://',
-    'webcal://',
-    'aesgcm://',
-}
 
+# Clickable URI schemes not (yet) registered with IANA (see ./iana.py)
+NONREGISTERED_URI_SCHEMES = {
+    'gemini',
+}
 
 # This is an excerpt of Media Types from
 # https://www.iana.org/assignments/media-types/media-types.xhtml
@@ -1080,14 +1022,22 @@ MIME_TYPES = (
     'audio/aac',
     'audio/ac3',
     'audio/flac',
+    'audio/mp3',
     'audio/mp4',
     'audio/mpeg',
     'audio/ogg',
     'audio/opus',
+    'audio/vnd.wave',
     'audio/wav',
+    'audio/wave',
+    'audio/x-aac',
     'audio/x-flac',
     'audio/x-m4a',
     'audio/x-matroska',
+    'audio/x-mp3',
+    'audio/x-mpeg',
+    'audio/x-ogg',
+    'audio/x-wav',
     # font/*
     'font/ttf',
     'font/woff',
