@@ -16,36 +16,36 @@ from __future__ import annotations
 
 import typing
 from typing import Any
-from typing import Union
-from typing import Optional
 from typing import Callable
+from typing import Optional
+from typing import Union
 
 from dataclasses import dataclass
 from dataclasses import field
 
-from nbxmpp.protocol import JID
-from nbxmpp.modules.security_labels import Catalog
-from nbxmpp.modules.security_labels import Displaymarking
-from nbxmpp.modules.security_labels import SecurityLabel
-from nbxmpp.structs import HTTPAuthData
-from nbxmpp.structs import ModerationData
-from nbxmpp.structs import LocationData
-from nbxmpp.structs import RosterItem
-from nbxmpp.structs import TuneData
 from nbxmpp.const import Affiliation
 from nbxmpp.const import InviteType
 from nbxmpp.const import Role
 from nbxmpp.const import StatusCode
+from nbxmpp.modules.security_labels import Catalog
+from nbxmpp.modules.security_labels import Displaymarking
+from nbxmpp.modules.security_labels import SecurityLabel
+from nbxmpp.protocol import JID
+from nbxmpp.structs import HTTPAuthData
+from nbxmpp.structs import LocationData
+from nbxmpp.structs import ModerationData
+from nbxmpp.structs import RosterItem
+from nbxmpp.structs import TuneData
 
-from gajim.common.file_props import FileProp
+from gajim.common.const import EncryptionInfoMsg
 from gajim.common.const import JingleState
 from gajim.common.const import KindConstant
+from gajim.common.file_props import FileProp
 from gajim.common.helpers import AdditionalDataDict
 
 if typing.TYPE_CHECKING:
     from gajim.common.client import Client
-    from gajim.plugins.pluginmanager import PluginManifest
-
+    from gajim.common.modules.httpupload import HTTPFileTransfer
 
 ChatListEventT = Union[
     'MessageReceived',
@@ -122,6 +122,13 @@ class Notification(ApplicationEvent):
     icon_name: Optional[str] = None
 
 
+@dataclass
+class ChatRead(ApplicationEvent):
+    name: str = field(init=False, default='chat-read')
+    account: str
+    jid: JID
+
+
 @dataclass
 class StanzaSent(ApplicationEvent):
     name: str = field(init=False, default='stanza-sent')
@@ -162,6 +169,7 @@ class MessageSent(ApplicationEvent):
     jid: JID
     message: str
     message_id: str
+    msg_log_id: Optional[int]
     chatstate: Optional[str]
     timestamp: float
     additional_data: AdditionalDataDict
@@ -236,6 +244,22 @@ class FileSendError(ApplicationEvent):
     error_msg: str = ''
 
 
+@dataclass
+class HTTPUploadStarted(ApplicationEvent):
+    name: str = field(init=False, default='http-upload-started')
+    account: str
+    jid: JID
+    transfer: HTTPFileTransfer
+
+
+@dataclass
+class HTTPUploadError(ApplicationEvent):
+    name: str = field(init=False, default='http-upload-error')
+    account: str
+    jid: JID
+    error_msg: str
+
+
 @dataclass
 class AccountEnabled(ApplicationEvent):
     name: str = field(init=False, default='account-enabled')
@@ -262,8 +286,16 @@ class BookmarksReceived(ApplicationEvent):
 
 
 @dataclass
-class BaseChatMarkerEvent(ApplicationEvent):
-    name: str
+class ReadStateSync(ApplicationEvent):
+    name: str = field(init=False, default='read-state-sync')
+    account: str
+    jid: JID
+    marker_id: str
+
+
+@dataclass
+class DisplayedReceived(ApplicationEvent):
+    name: str = field(init=False, default='displayed-received')
     account: str
     jid: JID
     properties: Any
@@ -272,16 +304,6 @@ class BaseChatMarkerEvent(ApplicationEvent):
     marker_id: str
 
 
-@dataclass
-class ReadStateSync(BaseChatMarkerEvent):
-    name: str = field(init=False, default='read-state-sync')
-
-
-@dataclass
-class DisplayedReceived(BaseChatMarkerEvent):
-    name: str = field(init=False, default='displayed-received')
-
-
 @dataclass
 class HttpAuth(ApplicationEvent):
     name: str = field(init=False, default='http-auth')
@@ -392,10 +414,9 @@ class MessageReceived(ApplicationEvent):
     unique_id: str
     stanza_id: str
     fjid: str
-    resource: str
-    session: Any
+    resource: str | None
     delayed: Optional[float]
-    msg_log_id: int
+    msg_log_id: Optional[int]
     displaymarking: Optional[Displaymarking]
 
 
@@ -437,18 +458,6 @@ class RosterPush(ApplicationEvent):
     item: RosterItem
 
 
-@dataclass
-class PluginAdded(ApplicationEvent):
-    name: str = field(init=False, default='plugin-added')
-    manifest: PluginManifest
-
-
-@dataclass
-class PluginRemoved(ApplicationEvent):
-    name: str = field(init=False, default='plugin-removed')
-    manifest: PluginManifest
-
-
 @dataclass
 class SearchFormReceivedEvent(ApplicationEvent):
     name: str = field(init=False, default='search-form-received')
@@ -546,6 +555,7 @@ class MucAdded(ApplicationEvent):
     name: str = field(init=False, default='muc-added')
     account: str
     jid: JID
+    select_chat: bool
 
 
 @dataclass
@@ -602,7 +612,7 @@ class SecCatalogReceived(ApplicationEvent):
     name: str = field(init=False, default='sec-catalog-received')
     account: str
     jid: str
-    catalog: dict[str, Catalog]
+    catalog: Catalog
 
 
 @dataclass
@@ -679,6 +689,7 @@ class AllowGajimUpdateCheck(ApplicationEvent):
 class GajimUpdateAvailable(ApplicationEvent):
     name: str = field(init=False, default='gajim-update-available')
     version: str
+    setup_url: str
 
 
 @dataclass
@@ -707,7 +718,7 @@ class MUCRoomConfigFinished(ApplicationEvent):
 class MUCRoomPresenceError(ApplicationEvent):
     name: str = field(init=False, default='muc-room-presence-error')
     timestamp: float
-    error: Any
+    error: str
 
 
 @dataclass
@@ -777,3 +788,11 @@ class MUCUserStatusShowChanged(ApplicationEvent):
     nick: str
     status: str
     show_value: str
+
+
+@dataclass
+class EncryptionInfo(ApplicationEvent):
+    name: str = field(init=False, default='encryption-check')
+    account: str
+    jid: JID
+    message: EncryptionInfoMsg
