@@ -30,88 +30,90 @@ from __future__ import annotations
 from typing import Any
 from typing import Callable
 from typing import Optional
-from typing import Union
 from typing import TYPE_CHECKING
+from typing import Union
 
-import sys
-import re
-import os
-import subprocess
-import hashlib
-import shlex
-import socket
-import logging
-import json
-import copy
 import collections
-import platform
+import copy
 import functools
-from collections import defaultdict
-import random
-import weakref
+import hashlib
+import importlib.metadata
 import inspect
+import json
+import logging
+import os
+import platform
+import random
+import re
+import socket
 import string
+import sys
+import unicodedata
+import weakref
 import webbrowser
-from string import Template
+from collections import defaultdict
 from datetime import datetime
 from datetime import timedelta
-from urllib.parse import unquote
-from urllib.parse import urlparse
 from functools import wraps
 from pathlib import Path
-from packaging.version import Version as V
-import unicodedata
-
-from nbxmpp.namespaces import Namespace
-from nbxmpp.const import Role
-from nbxmpp.const import Chatstate
-from nbxmpp.const import ConnectionProtocol
-from nbxmpp.const import ConnectionType
-from nbxmpp.const import Affiliation
-from nbxmpp.errors import StanzaError
-from nbxmpp.structs import CommonError
-from nbxmpp.structs import ProxyData
-from nbxmpp.protocol import JID
-from nbxmpp.protocol import InvalidJid
-from nbxmpp.protocol import Iq
-
-from OpenSSL.crypto import X509, load_certificate
-from OpenSSL.crypto import FILETYPE_PEM
+from string import Template
+from urllib.parse import unquote
+from urllib.parse import urlparse
 
+import precis_i18n.codec  # noqa: F401
+import qrcode
+from cryptography import x509
+from cryptography.hazmat.backends import default_backend
+from gi.repository import GdkPixbuf
 from gi.repository import Gio
 from gi.repository import GLib
 from gi.repository import GObject
 from gi.repository import Soup
-
-import precis_i18n.codec  # noqa: F401
+from nbxmpp.const import Affiliation
+from nbxmpp.const import Chatstate
+from nbxmpp.const import ConnectionProtocol
+from nbxmpp.const import ConnectionType
+from nbxmpp.const import Role
+from nbxmpp.errors import StanzaError
+from nbxmpp.namespaces import Namespace
+from nbxmpp.protocol import InvalidJid
+from nbxmpp.protocol import Iq
+from nbxmpp.protocol import JID
+from nbxmpp.structs import CommonError
+from nbxmpp.structs import ProxyData
+from packaging.requirements import Requirement
+from packaging.version import Version as V
+from qrcode.image.pil import PilImage as QrcPilImage
 
 from gajim.common import app
 from gajim.common import configpaths
-from gajim.common.i18n import Q_
-from gajim.common.i18n import _
-from gajim.common.i18n import ngettext
-from gajim.common.i18n import get_rfc5646_lang
+from gajim.common import iana
+from gajim.common import types
+from gajim.common.const import CONSONANTS
+from gajim.common.const import GIO_TLS_ERRORS
+from gajim.common.const import NONREGISTERED_URI_SCHEMES
+from gajim.common.const import SHOW_LIST
 from gajim.common.const import SHOW_STRING
 from gajim.common.const import SHOW_STRING_MNEMONIC
 from gajim.common.const import URIType
-from gajim.common.const import URIAction
-from gajim.common.const import GIO_TLS_ERRORS
-from gajim.common.const import SHOW_LIST
-from gajim.common.const import CONSONANTS
 from gajim.common.const import VOWELS
-from gajim.common.regex import INVALID_XML_CHARS_REGEX
-from gajim.common.regex import STH_AT_STH_DOT_STH_REGEX
+from gajim.common.const import XmppUriQuery
+from gajim.common.dbus.file_manager import DBusFileManager
+from gajim.common.i18n import _
+from gajim.common.i18n import get_rfc5646_lang
+from gajim.common.i18n import ngettext
+from gajim.common.i18n import p_
 from gajim.common.structs import URI
-from gajim.common import types
+
 if TYPE_CHECKING:
     from gajim.common.modules.util import LogAdapter
 
 HAS_PYWIN32 = False
 if os.name == 'nt':
     try:
-        import win32file
-        import win32con
         import pywintypes
+        import win32con
+        import win32file
         HAS_PYWIN32 = True
     except ImportError:
         pass
@@ -183,28 +185,28 @@ def get_uf_show(show: str, use_mnemonic: bool = False) -> str:
 
 def get_uf_sub(sub: str) -> str:
     if sub == 'none':
-        uf_sub = Q_('?Subscription we already have:None')
-    elif sub == 'to':
-        uf_sub = _('To')
-    elif sub == 'from':
-        uf_sub = _('From')
-    elif sub == 'both':
-        uf_sub = _('Both')
-    else:
-        uf_sub = _('Unknown')
+        return p_('Contact subscription', 'None')
 
-    return uf_sub
+    if sub == 'to':
+        return p_('Contact subscription', 'To')
+
+    if sub == 'from':
+        return p_('Contact subscription', 'From')
+
+    if sub == 'both':
+        return p_('Contact subscription', 'Both')
+
+    return p_('Contact subscription', 'Unknown')
 
 
 def get_uf_ask(ask: Union[str, None]) -> str:
     if ask is None:
-        uf_ask = Q_('?Ask (for Subscription):None')
-    elif ask == 'subscribe':
-        uf_ask = _('Subscribe')
-    else:
-        uf_ask = ask
+        return p_('Contact subscription', 'None')
 
-    return uf_ask
+    if ask == 'subscribe':
+        return p_('Contact subscription', 'Subscribe')
+
+    return ask
 
 
 def get_uf_role(role: Union[Role, str], plural: bool = False) -> str:
@@ -213,19 +215,19 @@ def get_uf_role(role: Union[Role, str], plural: bool = False) -> str:
         role = role.value
 
     if role == 'none':
-        return Q_('?Group Chat Contact Role:None')
+        return p_('Group chat contact role', 'None')
     if role == 'moderator':
         if plural:
-            return _('Moderators')
-        return _('Moderator')
+            return p_('Group chat contact role', 'Moderators')
+        return p_('Group chat contact role', 'Moderator')
     if role == 'participant':
         if plural:
-            return _('Participants')
-        return _('Participant')
+            return p_('Group chat contact role', 'Participants')
+        return p_('Group chat contact role', 'Participant')
     if role == 'visitor':
         if plural:
-            return _('Visitors')
-        return _('Visitor')
+            return p_('Group chat contact role', 'Visitors')
+        return p_('Group chat contact role', 'Visitor')
     return ''
 
 
@@ -237,56 +239,56 @@ def get_uf_affiliation(affiliation: Union[Affiliation, str],
         affiliation = affiliation.value
 
     if affiliation == 'none':
-        return Q_('?Group Chat Contact Affiliation:None')
+        return p_('Group chat contact affiliation', 'None')
     if affiliation == 'owner':
         if plural:
-            return _('Owners')
-        return _('Owner')
+            return p_('Group chat contact affiliation', 'Owners')
+        return p_('Group chat contact affiliation', 'Owner')
     if affiliation == 'admin':
         if plural:
-            return _('Administrators')
-        return _('Administrator')
+            return p_('Group chat contact affiliation', 'Administrators')
+        return p_('Group chat contact affiliation', 'Administrator')
     if affiliation == 'member':
         if plural:
-            return _('Members')
-        return _('Member')
+            return p_('Group chat contact affiliation', 'Members')
+        return p_('Group chat contact affiliation', 'Member')
     return ''
 
 
-def get_uf_relative_time(timestamp: float) -> str:
-    date_time = datetime.fromtimestamp(timestamp)
-    now = datetime.now()
+def get_uf_relative_time(date_time: datetime,
+                         now: Optional[datetime] = None
+                         ) -> str:
+
+    if now is None:  # used by unittest
+        now = datetime.now()
     timespan = now - date_time
 
-    if timespan > timedelta(days=365):
-        return str(date_time.year)
-    if timespan > timedelta(days=7):
-        return date_time.strftime('%b %d')
-    if timespan > timedelta(days=2):
-        return date_time.strftime('%a')
-    if date_time.strftime('%d') != now.strftime('%d'):
-        return _('Yesterday')
-    if timespan > timedelta(minutes=15):
-        return date_time.strftime('%H:%M')
-    if timespan > timedelta(minutes=1):
+    if timespan < timedelta(minutes=1):
+        return _('Just now')
+    if timespan < timedelta(minutes=15):
         minutes = int(timespan.seconds / 60)
-        return ngettext('%i min ago',
-                        '%i mins ago',
+        return ngettext('%s min ago',
+                        '%s mins ago',
                         minutes,
-                        minutes,
-                        minutes)
-    return _('Just now')
+                        str(minutes),
+                        str(minutes))
+    today = now.date()
+    if date_time.date() == today:
+        format_string = app.settings.get('time_format')
+        return date_time.strftime(format_string)
+    yesterday = now.date() - timedelta(days=1)
+    if date_time.date() == yesterday:
+        return _('Yesterday')
+    if timespan < timedelta(days=7):  # this week
+        return date_time.strftime('%a')  # weekday
+    if timespan < timedelta(days=365):  # this year
+        return date_time.strftime('%b %d')
+    return str(date_time.year)
 
 
 def to_one_line(msg: str) -> str:
     msg = msg.replace('\\', '\\\\')
-    msg = msg.replace('\n', '\\n')
-    # s1 = 'test\ntest\\ntest'
-    # s11 = s1.replace('\\', '\\\\')
-    # s12 = s11.replace('\n', '\\n')
-    # s12
-    # 'test\\ntest\\\\ntest'
-    return msg
+    return msg.replace('\n', '\\n')
 
 
 def from_one_line(msg: str) -> str:
@@ -296,13 +298,7 @@ def from_one_line(msg: str) -> str:
     # So here match '\\n' but not if you have a '\' before that
     expr = re.compile(r'(?<!\\)\\n')
     msg = expr.sub('\n', msg)
-    msg = msg.replace('\\\\', '\\')
-    # s12 = 'test\\ntest\\\\ntest'
-    # s13 = re.sub('\n', s12)
-    # s14 s13.replace('\\\\', '\\')
-    # s14
-    # 'test\ntest\\ntest'
-    return msg
+    return msg.replace('\\\\', '\\')
 
 
 def chatstate_to_string(chatstate: Optional[Chatstate]) -> str:
@@ -324,44 +320,6 @@ def chatstate_to_string(chatstate: Optional[Chatstate]) -> str:
     raise ValueError('unknown value: %s' % chatstate)
 
 
-def exec_command(command: str,
-                 use_shell: bool = False,
-                 posix: bool = True
-                 ) -> None:
-    '''
-    execute a command. if use_shell is True, we run the command as is it was
-    typed in a console. So it may be dangerous if you are not sure about what
-    is executed.
-    '''
-    if use_shell:
-        subprocess.Popen(f'{command} &', shell=True).wait()
-    else:
-        args = shlex.split(command, posix=posix)
-        process = subprocess.Popen(args)
-        app.thread_interface(process.wait)
-
-
-def build_command(executable: str, parameter: str) -> str:
-    # we add to the parameter (can hold path with spaces)
-    # "" so we have good parsing from shell
-    parameter = parameter.replace('"', '\\"')  # but first escape "
-    command = f'{executable} "{parameter}"'
-    return command
-
-
-def get_file_path_from_dnd_dropped_uri(uri: str) -> str:
-    path = unquote(uri)  # escape special chars
-    path = path.strip('\r\n\x00')  # remove \r\n and NULL
-    # get the path to file
-    if re.match('^file:///[a-zA-Z]:/', path):  # windows
-        path = path[8:]  # 8 is len('file:///')
-    elif path.startswith('file://'):  # nautilus, rox
-        path = path[7:]  # 7 is len('file://')
-    elif path.startswith('file:'):  # xffm
-        path = path[5:]  # 5 is len('file:')
-    return path
-
-
 def sanitize_filename(filename: str) -> str:
     '''
     Sanitize filename of elements not allowed on Windows
@@ -377,7 +335,7 @@ def sanitize_filename(filename: str) -> str:
         ]
         filename = ''.join(char for char in filename if char not in blacklist)
 
-        filename = ''.join(char for char in filename if 31 < ord(char))
+        filename = ''.join(char for char in filename if ord(char) > 31)
 
         filename = unicodedata.normalize('NFKD', filename)
         filename = filename.rstrip('. ')
@@ -412,6 +370,21 @@ def get_contact_dict_for_account(account: str) -> dict[str, types.BareContact]:
     return contacts_dict
 
 
+def generate_qr_code(content: str) -> GdkPixbuf.Pixbuf | None:
+    qr = qrcode.QRCode(version=None,
+                       error_correction=qrcode.constants.ERROR_CORRECT_L,
+                       box_size=6,
+                       border=4)
+    qr.add_data(content)
+    qr.make(fit=True)
+
+    img = qr.make_image(image_factory=QrcPilImage).convert('RGB')
+    return GdkPixbuf.Pixbuf.new_from_bytes(
+        GLib.Bytes.new(img.tobytes()),
+        GdkPixbuf.Colorspace.RGB, False, 8,
+        img.width, img.height, img.width*3)
+
+
 def play_sound(sound_event: str,
                account: Optional[str] = None,
                force: bool = False,
@@ -578,13 +551,7 @@ def get_auth_sha(sid: str, initiator: str, target: str) -> str:
     Return sha of sid + initiator + target used for proxy auth
     '''
     return hashlib.sha1(
-        (f'{sid}{initiator}{target}').encode('utf-8')).hexdigest()
-
-
-def remove_invalid_xml_chars(string_: str) -> str:
-    if string_:
-        string_ = re.sub(INVALID_XML_CHARS_REGEX, '', string_)
-    return string_
+        (f'{sid}{initiator}{target}').encode()).hexdigest()
 
 
 def get_random_string(count: int = 16) -> str:
@@ -614,29 +581,45 @@ def get_os_info() -> str:
 
 def message_needs_highlight(text: str, nickname: str, own_jid: str) -> bool:
     '''
-    Check text to see whether any of the words in (muc_highlight_words and
-    nick) appear
+    Check whether 'text' contains 'nickname', 'own_jid', or any string of the
+    'muc_highlight_words' setting.
     '''
-    special_words = app.settings.get('muc_highlight_words').split(';')
-    special_words.append(nickname)
-    special_words.append(own_jid)
-    # Strip empties: ''.split(';') == [''] and would highlight everything.
-    # Also lowercase everything for case insensitive compare.
-    special_words = [word.lower() for word in special_words if word]
+
+    search_strings = app.settings.get('muc_highlight_words').split(';')
+    search_strings.append(nickname)
+    search_strings.append(own_jid)
+
+    search_strings = [word.lower() for word in search_strings if word]
     text = text.lower()
 
-    for special_word in special_words:
-        found_here = text.find(special_word)
-        while found_here > -1:
-            end_here = found_here + len(special_word)
-            if ((found_here == 0 or not text[found_here - 1].isalpha()) and
-                    (end_here == len(text) or not text[end_here].isalpha())):
-                # It is beginning of text or char before is not alpha AND
-                # it is end of text or char after is not alpha
+    for search_string in search_strings:
+        match = text.find(search_string)
+
+        while match > -1:
+            search_end = match + len(search_string)
+
+            if match == 0 and search_end == len(text):
+                # Text contains search_string only (exact match)
                 return True
-            # continue searching
-            start = found_here + 1
-            found_here = text.find(special_word, start)
+
+            char_before_allowed = bool(
+                match == 0 or
+                (not text[match - 1].isalpha() and
+                text[match - 1] not in ('/', '-')))
+
+            if char_before_allowed and search_end == len(text):
+                # search_string found at the end of text and
+                # char before search_string is allowed.
+                return True
+
+            if char_before_allowed and not text[search_end].isalpha():
+                # char_before search_string is allowed and
+                # char_after search_string is not alpha.
+                return True
+
+            start = match + 1
+            match = text.find(search_string, start)
+
     return False
 
 
@@ -694,9 +677,8 @@ def jid_is_blocked(account: str, jid: str) -> bool:
 
 
 def get_subscription_request_msg(account: Optional[str] = None) -> str:
-    message = _('I would like to add you to my contact list.')
     if account is None:
-        return message
+        return _('I would like to add you to my contact list.')
 
     message = app.settings.get_account_setting(
         account, 'subscription_request_msg')
@@ -779,22 +761,6 @@ def version_condition(current_version: str, required_version: str) -> bool:
     return True
 
 
-def get_available_emoticon_themes() -> list[str]:
-    files: list[Path] = []
-    for folder in configpaths.get('EMOTICONS').iterdir():
-        if not folder.is_dir():
-            continue
-        files += [theme for theme in folder.iterdir() if theme.is_file()]
-
-    my_emots = configpaths.get('MY_EMOTS')
-    if my_emots.is_dir():
-        files += list(my_emots.iterdir())
-
-    emoticons_themes = ['font']
-    emoticons_themes += [file.stem for file in files if file.suffix == '.png']
-    return sorted(emoticons_themes)
-
-
 def call_counter(func):
     def helper(self, restart=False):
         if restart:
@@ -824,10 +790,9 @@ class AdditionalDataDict(collections.UserDict):
 
     @staticmethod
     def _get_path_childs(full_path: str) -> list[str]:
-        path_childs = [full_path]
         if ':' in full_path:
-            path_childs = full_path.split(':')
-        return path_childs
+            return full_path.split(':')
+        return [full_path]
 
     def set_value(self, full_path: str, key: str, value: Optional[str]) -> None:
         path_childs = self._get_path_childs(full_path)
@@ -874,17 +839,6 @@ class AdditionalDataDict(collections.UserDict):
         return copy.deepcopy(self)
 
 
-class Singleton(type):
-
-    _instances: dict[Any, Any] = {}
-
-    def __call__(cls, *args: Any, **kwargs: Any):
-        if cls not in cls._instances:
-            cls._instances[cls] = super(Singleton, cls).__call__(
-                *args, **kwargs)
-        return cls._instances[cls]
-
-
 def delay_execution(milliseconds):
     # Delay the first call for `milliseconds`
     # ignore all other calls while the delay is active
@@ -936,74 +890,153 @@ def catch_exceptions(func):
     return func_wrapper
 
 
-def parse_uri_actions(uri: str) -> tuple[str, dict[str, str]]:
-    uri = uri[5:]
-    if '?' not in uri:
-        return 'message', {'jid': uri}
+def is_known_uri_scheme(scheme: str) -> bool:
+    '''
+    `scheme` is lower-case
+    '''
+    if scheme in iana.URI_SCHEMES:
+        return True
+    if scheme in NONREGISTERED_URI_SCHEMES:
+        return True
+    return scheme in app.settings.get('additional_uri_schemes').split()
 
-    jid, action = uri.split('?', 1)
-    data = {'jid': jid}
-    if ';' in action:
-        action, keys = action.split(';', 1)
-        action_keys = keys.split(';')
-        for key in action_keys:
-            if key.startswith('subject='):
-                data['subject'] = unquote(key[8:])
 
-            elif key.startswith('body='):
-                data['body'] = unquote(key[5:])
+def parse_xmpp_uri_query(pct_iquerycomp: str) -> tuple[str, dict[str, str]]:
+    '''
+    Parses 'mess%61ge;b%6Fdy=Hello%20%F0%9F%8C%90%EF%B8%8F' into
+    ('message', {'body': 'Hello 🌐️'}), empty string into ('', {}).
+    '''
+    # Syntax and terminology from
+    # <https://rfc-editor.org/rfc/rfc5122#section-2.2>; the pct_ prefix means
+    # percent encoding not being undone yet.
 
-            elif key.startswith('thread='):
-                data['thread'] = key[7:]
-    return action, data
+    if not pct_iquerycomp:
+        return '', {}
+
+    pct_iquerytype, _, pct_ipairs = pct_iquerycomp.partition(';')
+    iquerytype = unquote(pct_iquerytype, errors='strict')
+    if not pct_ipairs:
+        return iquerytype, {}
+
+    pairs: dict[str, str] = {}
+    for pct_ipair in pct_ipairs.split(';'):
+        pct_ikey, _, pct_ival = pct_ipair.partition('=')
+        pairs[
+            unquote(pct_ikey, errors='strict')
+        ] = unquote(pct_ival, errors='replace')
+    return iquerytype, pairs
 
 
 def parse_uri(uri: str) -> URI:
-    if uri.startswith('xmpp:'):
-        action, data = parse_uri_actions(uri)
-        try:
-            validate_jid(data['jid'])
-            return URI(type=URIType.XMPP,
-                       action=URIAction(action),
-                       data=data)
-        except ValueError:
-            # Unknown action
-            return URI(type=URIType.UNKNOWN)
-
-    if uri.startswith('mailto:'):
-        uri = uri[7:]
-        return URI(type=URIType.MAIL, data=uri)
-
-    if uri.startswith('tel:'):
-        uri = uri[4:]
-        return URI(type=URIType.TEL, data=uri)
-
-    if STH_AT_STH_DOT_STH_REGEX.match(uri):
-        return URI(type=URIType.AT, data=uri)
-
-    if uri.startswith('geo:'):
-        location = uri[4:]
-        lat, _, lon = location.partition(',')
-        if not lon:
-            return URI(type=URIType.UNKNOWN, data=uri)
-
-        if Gio.AppInfo.get_default_for_uri_scheme('geo'):
-            return URI(type=URIType.GEO, data=uri)
-
-        uri = geo_provider_from_location(lat, lon)
-        return URI(type=URIType.GEO, data=uri)
-
-    if uri.startswith('file://'):
-        return URI(type=URIType.FILE, data=uri)
-
     try:
         urlparts = urlparse(uri)
-        if not urlparts.scheme:
-            return URI(type=URIType.WEB, data=f'http://{uri}')
-    except Exception:
-        pass
+    except Exception as err:
+        return URI(URIType.INVALID, uri, data={'error': str(err)})
 
-    return URI(type=URIType.WEB, data=uri)
+    if not urlparts.scheme:
+        return URI(URIType.INVALID, uri, data={'error': 'Relative URI'})
+
+    scheme = urlparts.scheme  # urlparse is expected to return it in lower case
+
+    if not is_known_uri_scheme(scheme):
+        return URI(URIType.INVALID, uri, data={'error': 'Unknown scheme'})
+
+    if scheme in ('https', 'http'):
+        if not urlparts.netloc:
+            err = f'No host or empty host in an {scheme} URI'
+            return URI(URIType.INVALID, uri, data={'error': err})
+        return URI(URIType.WEB, uri)
+
+    if scheme == 'xmpp':
+        if not urlparts.path.startswith('/'):
+            pct_jid = urlparts.path
+        else:
+            pct_jid = urlparts.path[1:]
+
+        data: dict[str, str] = {}
+        try:
+            data['jid'] = unquote(pct_jid, errors='strict')
+            validate_jid(data['jid'])
+            qtype, qparams = parse_xmpp_uri_query(urlparts.query)
+        except ValueError as err:
+            data['error'] = str(err)
+            return URI(URIType.INVALID, uri, data=data)
+
+        return URI(URIType.XMPP, uri,
+                   query_type=qtype,
+                   query_params=qparams,
+                   data=data)
+
+    if scheme == 'mailto':
+        data: dict[str, str] = {}
+        try:
+            data['addr'] = unquote(urlparts.path, errors='strict')
+            validate_jid(data['addr'], 'bare')  # meh, good enough
+        except ValueError as err:
+            data['error'] = str(err)
+            return URI(URIType.INVALID, uri, data=data)
+        return URI(URIType.MAIL, uri, data=data)
+
+    if scheme == 'tel':
+        # https://rfc-editor.org/rfc/rfc3966#section-3
+        # TODO: extract number
+        return URI(URIType.TEL, uri)
+
+    if scheme == 'geo':
+        # TODO: unify with .preview_helpers.split_geo_uri
+        # https://rfc-editor.org/rfc/rfc5870#section-3.3
+        lat, _, lon_alt = urlparts.path.partition(',')
+        if not lat or not lon_alt:
+            return URI(URIType.INVALID, uri, data={
+                       'error': 'No latitude or longitude'})
+        lon, _, alt = lon_alt.partition(',')
+        if not lon:
+            return URI(URIType.INVALID, uri, data={'error': 'No longitude'})
+
+        data = {'lat': lat, 'lon': lon, 'alt': alt}
+        return URI(URIType.GEO, uri, data=data)
+
+    if scheme == 'file':
+        # https://rfc-editor.org/rfc/rfc8089.html#section-2
+        data: dict[str, str] = {}
+        try:
+            data['netloc'] = urlparts.netloc
+            data['path'] = Gio.File.new_for_uri(uri).get_path()
+            assert data['path']
+        except Exception as err:
+            data['error'] = str(err)
+            return URI(URIType.INVALID, uri, data=data)
+        return URI(URIType.FILE, uri, data=data)
+
+    if scheme == 'about':
+        # https://rfc-editor.org/rfc/rfc6694#section-2.1
+        data: dict[str, str] = {}
+        try:
+            token = unquote(urlparts.path, errors='strict')
+            if token == 'ambiguous-address':  # noqa: S105
+                data['addr'] = unquote(urlparts.query, errors='strict')
+                validate_jid(data['addr'], 'bare')
+                return URI(URIType.AT, uri, data=data)
+            raise Exception(f'Unrecognized about-token "{token}"')
+        except Exception as err:
+            data['error'] = str(err)
+            return URI(URIType.INVALID, uri, data=data)
+
+    return URI(URIType.OTHER, uri)
+
+
+def _handle_message_qtype(
+        jid: str, params: dict[str, str], account: str) -> None:
+    body = params.get('body')
+    # ^ For JOIN, this is a non-standard, but nice extension
+    app.window.start_chat_from_jid(account, jid, message=body)
+
+
+_xmpp_query_type_handlers = {
+    XmppUriQuery.NONE: _handle_message_qtype,
+    XmppUriQuery.MESSAGE: _handle_message_qtype,
+    XmppUriQuery.JOIN: _handle_message_qtype,
+}
 
 
 @catch_exceptions
@@ -1012,64 +1045,108 @@ def open_uri(uri: Union[URI, str], account: Optional[str] = None) -> None:
         uri = parse_uri(uri)
 
     if uri.type == URIType.FILE:
-        open_file(uri.data)
-
-    elif uri.type == URIType.TEL:
-        if sys.platform == 'win32':
-            webbrowser.open(f'tel:{uri.data}')
+        opt_name = 'allow_open_file_uris'
+        if app.settings.get(opt_name):
+            open_file_uri(uri.source)
         else:
-            Gio.AppInfo.launch_default_for_uri(f'tel:{uri.data}')
+            log.info('Blocked opening a file URI, see %s option', opt_name)
 
-    elif uri.type == URIType.MAIL:
-        if sys.platform == 'win32':
-            webbrowser.open(f'mailto:{uri.data}')
+    elif uri.type in (URIType.MAIL, URIType.TEL, URIType.WEB, URIType.OTHER):
+        open_uri_externally(uri.source)
+
+    elif uri.type == URIType.GEO:
+        if Gio.AppInfo.get_default_for_uri_scheme('geo'):
+            open_uri_externally(uri.source)
         else:
-            Gio.AppInfo.launch_default_for_uri(f'mailto:{uri.data}')
+            open_uri(
+                geo_provider_from_location(uri.data['lat'], uri.data['lon']))
 
-    elif uri.type in (URIType.WEB, URIType.GEO):
-        if sys.platform == 'win32':
-            webbrowser.open(uri.data)
-        else:
-            Gio.AppInfo.launch_default_for_uri(uri.data)
-
-    elif uri.type == URIType.AT:
-        app.interface.start_chat_from_jid(account, uri.data)
-
-    elif uri.type == URIType.XMPP:
+    elif uri.type in (URIType.XMPP, URIType.AT):
         if account is None:
             log.warning('Account must be specified to open XMPP uri')
             return
 
-        if isinstance(uri.data, dict):
+        if uri.type == URIType.XMPP:
             jid = uri.data['jid']
-            message = uri.data.get('body')
         else:
-            log.warning('Cant open URI: %s', uri)
-            return
+            jid = uri.data['addr']
 
-        if uri.action == URIAction.JOIN:
-            app.app.activate_action(
-                'groupchat-join',
-                GLib.Variant('as', [account, jid]))
-        elif uri.action == URIAction.MESSAGE:
-            app.interface.start_chat_from_jid(account, jid, message=message)
-        else:
-            log.warning('Cant open URI: %s', uri)
+        qtype, qparams = XmppUriQuery.from_str(uri.query_type), uri.query_params
+        if not qtype:
+            log.info('open_uri: can\'t "%s": '
+                     'unsupported query type in %s', uri.query_type, uri)
+            # From <rfc5122#section-2.5>:
+            # > If the processing application does not understand [...] the
+            # > specified query type, it MUST ignore the query component and
+            # > treat the IRI/URI as consisting of, for example,
+            # > <xmpp:example-node@example.com> rather than
+            # > <xmpp:example-node@example.com?query>."
+            qtype, qparams = XmppUriQuery.NONE, {}
+        _xmpp_query_type_handlers[qtype](jid, qparams, account)
+
+    elif uri.type == URIType.INVALID:
+        log.warning('open_uri: Invalid %s', uri)
+        # TODO: UI error instead
 
     else:
-        log.warning('Cant open URI: %s', uri)
+        log.error('open_uri: No handler for %s', uri)
+        # TODO: this is a bug, so, `raise` maybe?
+
+
+def open_uri_externally(uri: str) -> None:
+    if sys.platform == 'win32':
+        webbrowser.open(uri, new=2)
+    else:
+        try:
+            Gio.AppInfo.launch_default_for_uri(uri)
+        except GLib.Error as err:
+            log.info('open_uri_externally: '
+                     "Couldn't launch default for %s: %s", uri, err)
+
+
+def open_file_uri(uri: str) -> None:
+    try:
+        if sys.platform != 'win32':
+            Gio.AppInfo.launch_default_for_uri(uri)
+        else:
+            os.startfile(uri)
+    except Exception as err:
+        log.info("Couldn't open file URI %s: %s", uri, err)
 
 
 @catch_exceptions
-def open_file(path: Union[str, Path]) -> None:
-    if os.name == 'nt':
-        os.startfile(path)
-    else:
-        # Call str() to make it work with pathlib.Path
-        path = str(path)
-        if not path.startswith('file://'):
-            path = 'file://' + path
-        Gio.AppInfo.launch_default_for_uri(path)
+def open_file(path: Path) -> None:
+    if not path.exists():
+        log.warning('Unable to open file, path %s does not exist', path)
+        return
+
+    open_file_uri(path.as_uri())
+
+
+def open_directory(path: Path) -> None:
+    return open_file(path)
+
+
+def show_in_folder(path: Path) -> None:
+    if not DBusFileManager().show_items_sync([path.as_uri()]):
+        # Fall back to just opening the containing folder
+        open_directory(path.parent)
+
+
+def filesystem_path_from_uri(uri: str) -> Optional[Path]:
+    puri = parse_uri(uri)
+    if puri.type != URIType.FILE:
+        return None
+
+    netloc = puri.data['netloc']
+    if netloc and netloc.lower() != 'localhost' and sys.platform != 'win32':
+        return None
+    return Path(puri.data['path'])
+
+
+def get_file_path_from_dnd_dropped_uri(text: str) -> Optional[Path]:
+    uri = text.strip('\r\n\x00')  # remove \r\n and NULL
+    return filesystem_path_from_uri(uri)
 
 
 def file_is_locked(path_to_file: str) -> bool:
@@ -1133,6 +1210,7 @@ def get_default_muc_config() -> dict[str, Union[bool, str]]:
         # Ejabberd options
         'allow_voice_requests': False,
         'public_list': False,
+        'mam': True,
 
         # Prosody options
         '{http://prosody.im/protocol/muc}roomconfig_allowmemberinvites': True,
@@ -1221,11 +1299,9 @@ def is_retraction_allowed(self_contact: types.GroupchatParticipant,
     return self_contact.affiliation >= contact.affiliation
 
 
-def get_tls_error_phrase(tls_error: Gio.TlsCertificateFlags) -> str:
-    phrase = GIO_TLS_ERRORS.get(tls_error)
-    if phrase is None:
-        return GIO_TLS_ERRORS[Gio.TlsCertificateFlags.GENERIC_ERROR]
-    return phrase
+def get_tls_error_phrases(tls_errors: set[Gio.TlsCertificateFlags]
+                          ) -> list[str]:
+    return [GIO_TLS_ERRORS[err] for err in tls_errors]
 
 
 class Observable:
@@ -1367,8 +1443,10 @@ def load_file_async(path: Path,
     file.load_contents_async(None, _on_load_finished)
 
 
-def convert_gio_to_openssl_cert(cert: Gio.TlsCertificate) -> X509:
-    return load_certificate(FILETYPE_PEM, cert.props.certificate_pem.encode())
+def get_x509_cert_from_gio_cert(cert: Gio.TlsCertificate) -> x509.Certificate:
+    glib_bytes = GLib.ByteArray.free_to_bytes(cert.props.certificate)
+    return x509.load_der_x509_certificate(
+        glib_bytes.get_data(), default_backend())
 
 
 def get_custom_host(account: str) -> Optional[tuple[str,
@@ -1394,37 +1472,31 @@ def warn_about_plain_connection(account: str,
                                 ) -> bool:
     warn = app.settings.get_account_setting(
         account, 'confirm_unencrypted_connection')
-    for type_ in connection_types:
-        if type_.is_plain and warn:
-            return True
-    return False
+    return any(type_.is_plain and warn for type_ in connection_types)
 
 
 def get_idle_status_message(state: str, status_message: str) -> str:
     message = app.settings.get(f'auto{state}_message')
     if not message:
-        message = status_message
-    else:
-        message = message.replace('$S', '%(status)s')
-        message = message.replace('$T', '%(time)s')
-        message = message % {
-            'status': status_message,
-            'time': app.settings.get(f'auto{state}time')
-        }
-    return message
+        return status_message
+
+    message = message.replace('$S', '%(status)s')
+    message = message.replace('$T', '%(time)s')
+    return message % {
+        'status': status_message,
+        'time': app.settings.get(f'auto{state}time')
+    }
 
 
 def get_group_chat_nick(account: str, room_jid: Union[JID, str]) -> str:
-    nick = app.nicks[account]
-
     client = app.get_client(account)
 
     bookmark = client.get_module('Bookmarks').get_bookmark(room_jid)
     if bookmark is not None:
         if bookmark.nick is not None:
-            nick = bookmark.nick
+            return bookmark.nick
 
-    return nick
+    return app.nicks[account]
 
 
 def get_random_muc_localpart() -> str:
@@ -1461,7 +1533,7 @@ def get_os_name() -> str:
 
 def get_os_version() -> str:
     if sys.platform in ('win32', 'darwin'):
-        return platform.release()
+        return platform.version()
     if os.name == 'posix':
         try:
             import distro
@@ -1472,15 +1544,30 @@ def get_os_version() -> str:
 
 
 def get_gobject_version() -> str:
-    gobject_ver = '.'.join(map(str, GObject.pygobject_version))
-    return gobject_ver
+    return '.'.join(map(str, GObject.pygobject_version))
 
 
 def get_glib_version() -> str:
-    glib_ver = '.'.join(map(str, [GLib.MAJOR_VERSION,
-                                  GLib.MINOR_VERSION,
-                                  GLib.MICRO_VERSION]))
-    return glib_ver
+    return '.'.join(map(str, [GLib.MAJOR_VERSION,
+                              GLib.MINOR_VERSION,
+                              GLib.MICRO_VERSION]))
+
+
+def get_soup_version() -> str:
+    return '.'.join(map(str, [Soup.get_major_version(),
+                              Soup.get_minor_version(),
+                              Soup.get_micro_version()]))
+
+
+def package_version(requirement: str) -> bool:
+    req = Requirement(requirement)
+
+    try:
+        installed_version = importlib.metadata.version(req.name)
+    except importlib.metadata.PackageNotFoundError:
+        return False
+
+    return installed_version in req.specifier
 
 
 def make_path_from_jid(base_path: Path, jid: JID) -> Path:
@@ -1494,24 +1581,3 @@ def make_path_from_jid(base_path: Path, jid: JID) -> Path:
     if jid.resource is not None:
         return path / jid.resource[:30]
     return path
-
-
-def make_http_request(uri: str,
-                      callback: Any,
-                      user_data: Optional[Any] = None) -> None:
-
-    proxy = determine_proxy()
-    if proxy is None:
-        resolver = None
-
-    else:
-        resolver = proxy.get_resolver()
-
-    session = Soup.Session()
-    session.props.proxy_resolver = resolver
-    session.props.user_agent = f'Gajim {app.version}'
-    message = Soup.Message.new('GET', uri)
-    if user_data is None:
-        session.queue_message(message, callback)
-    else:
-        session.queue_message(message, callback, user_data)
