@@ -21,26 +21,27 @@ from typing import Optional
 
 import time
 from functools import wraps
+from itertools import chain
 
+from gi.repository import GLib
+from nbxmpp.const import Chatstate as State
 from nbxmpp.namespaces import Namespace
 from nbxmpp.protocol import JID
 from nbxmpp.protocol import Presence
 from nbxmpp.structs import MessageProperties
 from nbxmpp.structs import PresenceProperties
 from nbxmpp.structs import StanzaHandler
-from nbxmpp.const import Chatstate as State
-
-from gi.repository import GLib
 
 from gajim.common import types
 from gajim.common.const import ClientState
-from gajim.common.structs import OutgoingMessage
 from gajim.common.modules.base import BaseModule
-from gajim.common.modules.contacts import GroupchatParticipant
 from gajim.common.modules.contacts import BareContact
+from gajim.common.modules.contacts import GroupchatParticipant
+from gajim.common.structs import OutgoingMessage
 
 INACTIVE_AFTER = 60
 PAUSED_AFTER = 10
+REMOTE_PAUSED_AFTER = 30
 
 
 def ensure_enabled(func: Any) -> Any:
@@ -78,6 +79,8 @@ class Chatstate(BaseModule):
         # The current chatstate we received from a contact
         self._remote_chatstate: dict[JID, State] = {}
 
+        self._remote_chatstate_composing_timeouts: dict[JID, int] = {}
+
         self._last_keyboard_activity: dict[JID, float] = {}
         self._last_mouse_activity: dict[JID, float] = {}
         self._timeout_id = None
@@ -85,8 +88,10 @@ class Chatstate(BaseModule):
         self._blocked: list[JID] = []
         self._enabled = False
 
-        self._con.connect_signal('state-changed', self._on_client_state_changed)
-        self._con.connect_signal('resume-failed', self._on_client_resume_failed)
+        self._client.connect_signal('state-changed',
+                                    self._on_client_state_changed)
+        self._client.connect_signal('resume-failed',
+                                    self._on_client_resume_failed)
 
     def _on_client_resume_failed(self,
                                  _client: types.Client,
@@ -117,7 +122,7 @@ class Chatstate(BaseModule):
                 2, self._check_last_interaction)
         else:
             self.cleanup()
-            self._con.get_module('Contacts').force_chatstate_update()
+            self._client.get_module('Contacts').force_chatstate_update()
 
     @ensure_enabled
     def _presence_received(self,
@@ -162,17 +167,43 @@ class Chatstate(BaseModule):
                 properties.is_carbon_message and properties.carbon.is_sent):
             return
 
-        assert properties.jid is not None
-        self._remote_chatstate[properties.jid] = properties.chatstate
+        jid = properties.jid
+        assert jid is not None
 
-        self._log.info('Recv: %-10s - %s', properties.chatstate, properties.jid)
+        state = properties.chatstate
+        self._remote_chatstate[jid] = state
 
-        contact = self._get_contact(properties.jid)
+        self._log.info('Recv: %-10s - %s', state, jid)
+
+        contact = self._get_contact(jid)
         if contact is None:
             return
 
+        self._remove_remote_composing_timeout(contact)
+        if state == State.COMPOSING:
+            # the spec does not cover any timeout for the composing action,
+            # but if a contact's client does not send another chat state,
+            # we don't want the GUI to show that they are "composing" forever
+            self._remote_chatstate_composing_timeouts[jid] = \
+                GLib.timeout_add_seconds(REMOTE_PAUSED_AFTER,
+                                         self._on_remote_composing_timeout,
+                                         contact)
+
         contact.notify('chatstate-update')
 
+    def _on_remote_composing_timeout(self, contact: types.ContactT):
+        self._log.info(
+            'Automatically switching the chat state of %s to ACTIVE', contact)
+        self._remote_chatstate[contact.jid] = State.ACTIVE
+        contact.notify('chatstate-update')
+
+    def _remove_remote_composing_timeout(self, contact: types.ContactT):
+        source_id = self._remote_chatstate_composing_timeouts.pop(
+            contact.jid, None)
+        if source_id is not None:
+            self._log.debug('Removing remote composing timeout of %s', contact)
+            GLib.source_remove(source_id)
+
     @ensure_enabled
     def _check_last_interaction(self) -> bool:
         now = time.time()
@@ -222,7 +253,7 @@ class Chatstate(BaseModule):
 
         if not contact.is_groupchat:
             # Don’t send chatstates to ourself
-            if self._con.get_own_jid().bare_match(contact.jid):
+            if self._client.is_own_jid(contact.jid):
                 return None
 
             if not contact.supports(Namespace.CHATSTATES):
@@ -255,7 +286,7 @@ class Chatstate(BaseModule):
         # User starts writing again.
 
         # Don’t send chatstates to ourself
-        if self._con.get_own_jid().bare_match(contact.jid):
+        if self._client.is_own_jid(contact.jid):
             return
 
         self.remove_delay_timeout(contact)
@@ -265,7 +296,7 @@ class Chatstate(BaseModule):
     @ensure_enabled
     def set_chatstate(self, contact: types.ChatContactT, state: State) -> None:
         # Don’t send chatstates to ourself
-        if self._con.get_own_jid().bare_match(contact.jid):
+        if self._client.is_own_jid(contact.jid):
             return
 
         if contact.jid in self._blocked:
@@ -335,7 +366,7 @@ class Chatstate(BaseModule):
                                   chatstate=chatstate.value,
                                   play_sound=False)
 
-        self._con.send_message(message)
+        self._client.send_message(message)
 
     @ensure_enabled
     def set_mouse_activity(self,
@@ -361,13 +392,16 @@ class Chatstate(BaseModule):
             GLib.source_remove(timeout)
             del self._delay_timeout_ids[contact.jid]
 
-    def remove_all_delay_timeouts(self) -> None:
-        for timeout in self._delay_timeout_ids.values():
+    def remove_all_timeouts(self) -> None:
+        for timeout in chain(
+                self._delay_timeout_ids.values(),
+                self._remote_chatstate_composing_timeouts.values()):
             GLib.source_remove(timeout)
         self._delay_timeout_ids.clear()
+        self._remote_chatstate_composing_timeouts.clear()
 
     def cleanup(self) -> None:
-        self.remove_all_delay_timeouts()
+        self.remove_all_timeouts()
         if self._timeout_id is not None:
             GLib.source_remove(self._timeout_id)
             self._timeout_id = None
