@@ -17,10 +17,14 @@ from __future__ import annotations
 from typing import Any
 from typing import Iterator
 from typing import Optional
-from typing import Union
 from typing import overload
+from typing import Union
+
+from datetime import datetime
+from datetime import timezone
 
 import cairo
+from gi.repository import GLib
 from nbxmpp.const import Affiliation
 from nbxmpp.const import Chatstate
 from nbxmpp.const import Role
@@ -28,27 +32,27 @@ from nbxmpp.namespaces import Namespace
 from nbxmpp.protocol import JID
 from nbxmpp.structs import DiscoInfo
 from nbxmpp.structs import LocationData
-from nbxmpp.structs import TuneData
 from nbxmpp.structs import MucSubject
+from nbxmpp.structs import TuneData
 
 from gajim.common import app
 from gajim.common import types
 from gajim.common.const import PresenceShowExt
 from gajim.common.const import SimpleClientState
-from gajim.common.setting_values import BoolContactSettings
-from gajim.common.setting_values import StringContactSettings
-from gajim.common.setting_values import BoolGroupChatSettings
-from gajim.common.setting_values import IntGroupChatSettings
-from gajim.common.setting_values import StringGroupChatSettings
-from gajim.common.structs import UNKNOWN_PRESENCE
-from gajim.common.structs import PresenceData
-from gajim.common.structs import UNKNOWN_MUC_PRESENCE
-from gajim.common.structs import MUCPresenceData
-from gajim.common.helpers import Observable
 from gajim.common.helpers import chatstate_to_string
+from gajim.common.helpers import get_groupchat_name
+from gajim.common.helpers import Observable
 from gajim.common.modules.base import BaseModule
 from gajim.common.modules.util import LogAdapter
-from gajim.common.helpers import get_groupchat_name
+from gajim.common.setting_values import BoolContactSettings
+from gajim.common.setting_values import BoolGroupChatSettings
+from gajim.common.setting_values import IntGroupChatSettings
+from gajim.common.setting_values import StringContactSettings
+from gajim.common.setting_values import StringGroupChatSettings
+from gajim.common.structs import MUCPresenceData
+from gajim.common.structs import PresenceData
+from gajim.common.structs import UNKNOWN_MUC_PRESENCE
+from gajim.common.structs import UNKNOWN_PRESENCE
 
 
 class ContactSettings:
@@ -147,7 +151,8 @@ class Contacts(BaseModule):
         if contact is not None:
             if not isinstance(contact, GroupchatContact):
                 raise ValueError(f'Trying to add GroupchatContact {jid}, '
-                                 f'but contact exists already as {contact}')
+                                 f'but contact already exists as {contact} '
+                                 f'(in roster: {contact.is_in_roster})')
             return contact
 
         contact = GroupchatContact(self._log, jid, self._account)
@@ -331,6 +336,19 @@ class CommonContact(Observable):
     def chatstate_string(self) -> str:
         return chatstate_to_string(self.chatstate)
 
+    @property
+    def is_muted(self) -> bool:
+        mute_until = self.settings.get('mute_until')
+        if not mute_until:
+            return False
+
+        until = datetime.fromisoformat(mute_until)
+        is_muted = until > datetime.now(timezone.utc)
+        if not is_muted:
+            # Reset the setting to default
+            GLib.idle_add(self.settings.set, 'mute_until', None)
+        return is_muted
+
     def __repr__(self) -> str:
         return f'{self.jid} ({self._account})'
 
@@ -342,7 +360,8 @@ class BareContact(CommonContact):
         self.settings = ContactSettings(account, jid)
         self._resources: dict[str, ResourceContact] = {}
 
-        self._avatar_sha = app.storage.cache.get_contact(jid, 'avatar')
+        self._avatar_sha = app.storage.cache.get_contact(
+            account, jid, 'avatar')
 
     @property
     def is_self(self):
@@ -350,10 +369,8 @@ class BareContact(CommonContact):
         return own_jid == self.jid
 
     def supports(self, requested_feature: str) -> bool:
-        for resource in self.iter_resources():
-            if resource.supports(requested_feature):
-                return True
-        return False
+        return any(resource.supports(requested_feature)
+                   for resource in self.iter_resources())
 
     def supports_audio(self) -> bool:
         if (self.supports(Namespace.JINGLE_ICE_UDP) and
@@ -432,7 +449,8 @@ class BareContact(CommonContact):
         if item is not None and item.name:
             return item.name
 
-        nickname = app.storage.cache.get_contact(self._jid, 'nickname')
+        nickname = app.storage.cache.get_contact(
+            self._account, self._jid, 'nickname')
         if nickname:
             return nickname
         if self._jid.is_domain:
@@ -450,13 +468,16 @@ class BareContact(CommonContact):
 
     @property
     def avatar_sha(self) -> Optional[str]:
-        return app.storage.cache.get_contact(self._jid, 'avatar')
+        return app.storage.cache.get_contact(
+            self._account, self._jid, 'avatar')
+
+    def set_avatar_sha(self, sha: str) -> None:
+        app.storage.cache.set_contact(self._account, self._jid, 'avatar', sha)
 
     def get_avatar(self,
                    size: int,
                    scale: int,
                    add_show: bool = True,
-                   pixbuf: bool = False,
                    default: bool = False,
                    style: str = 'circle'):
 
@@ -479,15 +500,6 @@ class BareContact(CommonContact):
         if self.avatar_sha is not None:
             transport_icon = None
 
-        if pixbuf:
-            return app.app.avatar_storage.get_pixbuf(
-                self,
-                size,
-                scale,
-                show,
-                default=default,
-                transport_icon=transport_icon,
-                style=style)
         return app.app.avatar_storage.get_surface(
             self,
             size,
@@ -508,7 +520,7 @@ class BareContact(CommonContact):
 
         self._avatar_sha = sha
 
-        app.storage.cache.set_contact(self._jid, 'avatar', sha)
+        app.storage.cache.set_contact(self._account, self._jid, 'avatar', sha)
         app.app.avatar_storage.invalidate_cache(self._jid)
         self.notify('avatar-update')
 
@@ -609,10 +621,8 @@ class ResourceContact(CommonContact):
         if disco_info is None:
             return False
 
-        for identity in disco_info.identities:
-            if identity.type == 'phone':
-                return True
-        return False
+        return any(identity.type == 'phone' for
+                   identity in disco_info.identities)
 
     @property
     def is_available(self) -> bool:
@@ -652,6 +662,10 @@ class ResourceContact(CommonContact):
     def type_string(self) -> str:
         raise NotImplementedError
 
+    @property
+    def is_muted(self) -> bool:
+        raise NotImplementedError
+
 
 class GroupchatContact(CommonContact):
     def __init__(self, logger: LogAdapter, jid: JID, account: str) -> None:
@@ -735,7 +749,10 @@ class GroupchatContact(CommonContact):
 
     @property
     def avatar_sha(self) -> Optional[str]:
-        return app.storage.cache.get_muc(self._jid, 'avatar')
+        return app.storage.cache.get_muc(self._account, self._jid, 'avatar')
+
+    def set_avatar_sha(self, sha: str) -> None:
+        app.storage.cache.set_muc(self._account, self._jid, 'avatar', sha)
 
     def get_avatar(self, size: int, scale: int) -> Optional[cairo.ImageSurface]:
         transport_icon = None
@@ -873,6 +890,10 @@ class GroupchatParticipant(CommonContact):
     def room(self) -> GroupchatContact:
         return self._module('Contacts').get_bare_contact(self.jid.bare)
 
+    @property
+    def muc_context(self) -> Optional[str]:
+        return self.room.muc_context
+
     @property
     def presence(self) -> MUCPresenceData:
         return self._presence
