@@ -20,8 +20,8 @@ from typing import Optional
 from typing import Union
 
 import nbxmpp
-from nbxmpp.errors import StanzaError
 from nbxmpp.errors import is_error
+from nbxmpp.errors import StanzaError
 from nbxmpp.namespaces import Namespace
 from nbxmpp.protocol import Iq
 from nbxmpp.protocol import JID
@@ -32,10 +32,10 @@ from nbxmpp.task import Task
 
 from gajim.common import app
 from gajim.common import types
-from gajim.common.events import ServerDiscoReceived
 from gajim.common.events import MucDiscoUpdate
-from gajim.common.modules.util import as_task
+from gajim.common.events import ServerDiscoReceived
 from gajim.common.modules.base import BaseModule
+from gajim.common.modules.util import as_task
 
 
 class Discovery(BaseModule):
@@ -232,10 +232,11 @@ class Discovery(BaseModule):
         if result.vcard is not None:
             avatar, avatar_sha = result.vcard.get_avatar()
             if avatar is not None:
-                if not app.interface.avatar_exists(avatar_sha):
-                    app.interface.save_avatar(avatar)
+                if not app.app.avatar_storage.avatar_exists(avatar_sha):
+                    app.app.avatar_storage.save_avatar(avatar)
 
-                app.storage.cache.set_muc(result.info.jid, 'avatar', avatar_sha)
+                app.storage.cache.set_muc(
+                    self._account, result.info.jid, 'avatar', avatar_sha)
                 app.app.avatar_storage.invalidate_cache(result.info.jid)
 
         self._con.get_module('VCardAvatars').muc_disco_info_update(result.info)
