@@ -22,11 +22,13 @@ from typing import Optional
 import time
 from datetime import datetime
 from datetime import timedelta
+from datetime import timezone
 
 import nbxmpp
-from nbxmpp.errors import StanzaError
-from nbxmpp.errors import MalformedStanzaError
 from nbxmpp.errors import is_error
+from nbxmpp.errors import MalformedStanzaError
+from nbxmpp.errors import StanzaError
+from nbxmpp.modules.util import raise_if_error
 from nbxmpp.namespaces import Namespace
 from nbxmpp.protocol import JID
 from nbxmpp.protocol import Message
@@ -36,24 +38,24 @@ from nbxmpp.structs import StanzaHandler
 from nbxmpp.structs import StanzaIDData
 from nbxmpp.task import Task
 from nbxmpp.util import generate_id
-from nbxmpp.modules.util import raise_if_error
 
 from gajim.common import app
 from gajim.common import types
+from gajim.common.const import ArchiveState
+from gajim.common.const import ClientState
+from gajim.common.const import KindConstant
+from gajim.common.const import SyncThreshold
 from gajim.common.events import ArchivingIntervalFinished
 from gajim.common.events import FeatureDiscovered
 from gajim.common.events import MamMessageReceived
 from gajim.common.events import RawMamMessageReceived
-from gajim.common.const import ArchiveState, ClientState
-from gajim.common.const import KindConstant
-from gajim.common.const import SyncThreshold
 from gajim.common.helpers import AdditionalDataDict
 from gajim.common.helpers import get_retraction_text
+from gajim.common.modules.base import BaseModule
 from gajim.common.modules.misc import parse_oob
+from gajim.common.modules.util import as_task
 from gajim.common.modules.util import check_if_message_correction
 from gajim.common.modules.util import get_eme_message
-from gajim.common.modules.util import as_task
-from gajim.common.modules.base import BaseModule
 
 
 class MAM(BaseModule):
@@ -341,7 +343,7 @@ class MAM(BaseModule):
         self._mam_query_ids[jid] = query_id
         return query_id
 
-    def _get_query_params(self) -> tuple[Optional[ſtr], Optional[datetime]]:
+    def _get_query_params(self) -> tuple[Optional[str], Optional[datetime]]:
         own_jid = self._con.get_own_jid().bare
         archive = app.storage.archive.get_archive_infos(own_jid)
 
@@ -356,7 +358,7 @@ class MAM(BaseModule):
 
         else:
             # First Start, we request the last week
-            start_date = datetime.utcnow() - timedelta(days=7)
+            start_date = datetime.now(timezone.utc) - timedelta(days=7)
             self._log.info('Request archive: %s, after date %s',
                            own_jid, start_date)
         return mam_id, start_date
@@ -365,13 +367,15 @@ class MAM(BaseModule):
                               jid: JID,
                               threshold: SyncThreshold
                               ) -> tuple[Optional[str], Optional[datetime]]:
+
         archive = app.storage.archive.get_archive_infos(jid)
         mam_id = None
         start_date = None
+        now = datetime.now(timezone.utc)
 
         if archive is None or archive.last_mam_id is None:
             # First join
-            start_date = datetime.utcnow() - timedelta(days=1)
+            start_date = now - timedelta(days=1)
             self._log.info('Request archive: %s, after date %s',
                            jid, start_date)
 
@@ -390,10 +394,10 @@ class MAM(BaseModule):
                 self._log.info('No last muc timestamp found: %s', jid)
                 last_timestamp = 0
 
-            last = datetime.utcfromtimestamp(float(last_timestamp))
-            if datetime.utcnow() - last > timedelta(days=threshold):
+            last = datetime.fromtimestamp(float(last_timestamp), timezone.utc)
+            if now - last > timedelta(days=threshold):
                 # To much time has elapsed since last join, apply threshold
-                start_date = datetime.utcnow() - timedelta(days=threshold)
+                start_date = now - timedelta(days=threshold)
                 self._log.info('Too much time elapsed since last join, '
                                'request archive: %s, after date %s, '
                                'threshold: %s', jid, start_date, threshold)
@@ -460,12 +464,16 @@ class MAM(BaseModule):
         if threshold == SyncThreshold.NO_SYNC:
             return
 
+        contact = self._get_contact(jid, groupchat=True)
+        contact.notify('mam-sync-started')
+
         mam_id, start_date = self._get_muc_query_params(jid, threshold)
 
         result = yield self._execute_query(jid, mam_id, start_date)
         if is_error(result):
             if result.condition != 'item-not-found':
                 self._log.warning(result)
+                contact.notify('mam-sync-error', result.get_text())
                 return
 
             app.storage.archive.reset_archive_infos(result.jid)
@@ -473,6 +481,7 @@ class MAM(BaseModule):
             result = yield self._execute_query(result.jid, None, start_date)
             if is_error(result):
                 self._log.warning(result)
+                contact.notify('mam-sync-error', result.get_text())
                 return
 
         if result.rsm.last is not None:
@@ -484,6 +493,8 @@ class MAM(BaseModule):
                 last_mam_id=result.rsm.last,
                 last_muc_timestamp=time.time())
 
+        contact.notify('mam-sync-finished')
+
     @as_task
     def _execute_query(self,
                        jid: JID,
