@@ -19,17 +19,17 @@ from __future__ import annotations
 from typing import Generator
 from typing import Optional
 
-from nbxmpp.namespaces import Namespace
 from nbxmpp.modules.user_avatar import AvatarData
 from nbxmpp.modules.util import is_error
+from nbxmpp.namespaces import Namespace
 from nbxmpp.protocol import Message
 from nbxmpp.structs import MessageProperties
 
 from gajim.common import app
 from gajim.common import types
 from gajim.common.modules.base import BaseModule
-from gajim.common.modules.util import event_node
 from gajim.common.modules.util import as_task
+from gajim.common.modules.util import event_node
 
 
 class UserAvatar(BaseModule):
@@ -66,11 +66,12 @@ class UserAvatar(BaseModule):
 
         sha = contact.avatar_sha
         if sha is not None:
-            if sha in metadata.avatar_shas and app.interface.avatar_exists(sha):
+            if (sha in metadata.avatar_shas and
+                    app.app.avatar_storage.avatar_exists(sha)):
                 self._log.info('Avatar already known: %s %s', jid, sha)
                 return
 
-        if app.interface.avatar_exists(metadata.default):
+        if app.app.avatar_storage.avatar_exists(metadata.default):
             self._log.info('Avatar found in cache, update: %s %s',
                            jid, metadata.default)
             contact.update_avatar(metadata.default)
@@ -109,5 +110,5 @@ class UserAvatar(BaseModule):
             return
 
         self._log.info('Received Avatar: %s %s', contact.jid, avatar.sha)
-        app.interface.save_avatar(avatar.data)
+        app.app.avatar_storage.save_avatar(avatar.data)
         contact.update_avatar(avatar.sha)
