@@ -12,46 +12,72 @@
 # You should have received a copy of the GNU General Public License
 # along with Gajim. If not, see <http://www.gnu.org/licenses/>.
 
+from __future__ import annotations
+
 from typing import Any
+from typing import Callable
+from typing import cast
 from typing import Optional
 
 import logging
 import os
+import re
+import uuid
+from dataclasses import dataclass
+from dataclasses import field
 from pathlib import Path
-from urllib.parse import urlparse
 from urllib.parse import ParseResult
+from urllib.parse import urlparse
 
 from gi.repository import Gio
 from gi.repository import GLib
-from gi.repository import Soup
+from nbxmpp.const import HTTPRequestError
+from nbxmpp.http import HTTPRequest
+from nbxmpp.util import convert_tls_error_flags
 
 from gajim.common import app
 from gajim.common import configpaths
+from gajim.common import regex
 from gajim.common.const import MIME_TYPES
 from gajim.common.helpers import AdditionalDataDict
+from gajim.common.helpers import get_tls_error_phrases
 from gajim.common.helpers import load_file_async
 from gajim.common.helpers import write_file_async
-from gajim.common.helpers import get_tls_error_phrase
-from gajim.common.helpers import get_account_proxy
 from gajim.common.i18n import _
 from gajim.common.preview_helpers import aes_decrypt
-from gajim.common.preview_helpers import filename_from_uri
-from gajim.common.preview_helpers import parse_fragment
 from gajim.common.preview_helpers import create_thumbnail
-from gajim.common.preview_helpers import split_geo_uri
-from gajim.common.preview_helpers import get_previewable_mime_types
+from gajim.common.preview_helpers import filename_from_uri
 from gajim.common.preview_helpers import get_image_paths
+from gajim.common.preview_helpers import get_previewable_mime_types
 from gajim.common.preview_helpers import guess_mime_type
+from gajim.common.preview_helpers import parse_fragment
 from gajim.common.preview_helpers import pixbuf_from_data
+from gajim.common.preview_helpers import split_geo_uri
 from gajim.common.types import GdkPixbufType
+from gajim.common.util.http import create_http_request
 
 log = logging.getLogger('gajim.c.preview')
 
+IRI_RX = re.compile(regex.IRI)
+
 PREVIEWABLE_MIME_TYPES = get_previewable_mime_types()
 mime_types = set(MIME_TYPES)
 # Merge both: if it’s a previewable image, it should be allowed
 ALLOWED_MIME_TYPES = mime_types.union(PREVIEWABLE_MIME_TYPES)
 
+AudioSampleT = list[tuple[float, float]]
+
+
+@dataclass
+class AudioPreviewState:
+    duration: float = 0.0
+    position: float = 0.0
+    is_eos: bool = False
+    speed: float = 1.0
+    is_timestamp_positive: bool = True
+    samples: AudioSampleT = field(default_factory=list)
+    is_audio_analyzed = False
+
 
 class Preview:
     def __init__(self,
@@ -64,6 +90,8 @@ class Preview:
                  from_us: bool = False,
                  context: Optional[str] = None
                  ) -> None:
+
+        self.id = str(uuid.uuid4())
         self._uri = uri
         self._urlparts = urlparts
         self._filename = filename_from_uri(uri)
@@ -79,17 +107,20 @@ class Preview:
         self.thumbnail: Optional[bytes] = None
         self.mime_type: str = ''
         self.file_size: int = 0
-        self._received_size: int = 0
         self.download_in_progress = False
 
         self.info_message: Optional[str] = None
 
-        self._soup_message: Optional[Soup.Message] = None
+        self._request = None
 
         self.key: Optional[bytes] = None
         self.iv: Optional[bytes] = None
         if self.is_aes_encrypted and urlparts is not None:
-            self.key, self.iv = parse_fragment(urlparts.fragment)
+            try:
+                self.key, self.iv = parse_fragment(urlparts.fragment)
+            except ValueError as err:
+                log.error('Parsing fragment for AES decryption '
+                          'failed: %s', err)
 
     @property
     def is_geo_uri(self) -> bool:
@@ -125,8 +156,8 @@ class Preview:
         return self._filename
 
     @property
-    def soup_message(self) -> Soup.Message:
-        return self._soup_message
+    def request(self) -> HTTPRequest:
+        return self._request
 
     @property
     def request_uri(self) -> Optional[str]:
@@ -157,36 +188,24 @@ class Preview:
         return self.orig_path.exists()
 
     def create_thumbnail(self, data: bytes) -> bool:
-        self.thumbnail = create_thumbnail(data, self.size)
+        self.thumbnail = create_thumbnail(data, self.size, self.mime_type)
         if self.thumbnail is None:
             self.info_message = _('Creating thumbnail failed')
             log.warning('Creating thumbnail failed for: %s', self.orig_path)
             return False
         return True
 
-    def reset_received_size(self) -> None:
-        self._received_size = 0
-
     def update_widget(self, data: Optional[GdkPixbufType] = None) -> None:
         self._widget.update(self, data)
 
-    def update_progress(self, size: int, message: Soup.Message) -> None:
+    def update_progress(self, progress: float, request: HTTPRequest) -> None:
         self.download_in_progress = True
-        self._soup_message = message
-        self._received_size += size
-        if self.file_size == 0 or self._received_size == 0:
-            return
-
-        progress = self._received_size / self.file_size
+        self._request = request
         self._widget.update_progress(self, progress)
 
 
 class PreviewManager:
     def __init__(self) -> None:
-        self._sessions: dict[
-            str,
-            tuple[Soup.Session, Optional[Gio.SimpleProxyResolver]]] = {}
-
         self._orig_dir = Path(configpaths.get('MY_DATA')) / 'downloads'
         self._thumb_dir = Path(configpaths.get('MY_CACHE')) / 'downloads.thumb'
 
@@ -196,27 +215,51 @@ class PreviewManager:
         if GLib.mkdir_with_parents(str(self._thumb_dir), 0o700) != 0:
             log.error('Failed to create: %s', self._thumb_dir)
 
-    def _get_session(self, account: str) -> Soup.Session:
-        if account not in self._sessions:
-            self._sessions[account] = self._create_session(account)
-        return self._sessions[account][0]
+        self._previews: dict[str, Preview] = {}
 
-    @staticmethod
-    def _create_session(account: str) -> tuple[
-            Soup.Session, Optional[Gio.SimpleProxyResolver]]:
-        session = Soup.Session()
-        session.add_feature_by_type(Soup.ContentSniffer)
-        session.props.https_aliases = ['aesgcm']
-        session.props.ssl_strict = False
+        # Holds active audio preview sessions
+        # for resuming after switching chats
+        self._audio_sessions: dict[int, AudioPreviewState] = {}
 
-        proxy = get_account_proxy(account)
-        if proxy is None:
-            resolver = None
-        else:
-            resolver = proxy.get_resolver()
+        # References a stop function for each audio preview, which allows us
+        # to stop previews by preview_id, see stop_audio_except(preview_id)
+        self._audio_stop_functions: dict[int, Callable[..., None]] = {}
 
-        session.props.proxy_resolver = resolver
-        return session, resolver
+        log.info('Supported mime types for preview')
+        log.info(sorted(PREVIEWABLE_MIME_TYPES))
+
+    def get_preview(self, preview_id: str) -> Optional[Preview]:
+        return self._previews.get(preview_id)
+
+    def clear_previews(self) -> None:
+        self._previews.clear()
+
+    def get_audio_state(self,
+                        preview_id: int
+                        ) -> AudioPreviewState:
+
+        state = self._audio_sessions.get(preview_id)
+        if state is not None:
+            return state
+        self._audio_sessions[preview_id] = AudioPreviewState()
+        return self._audio_sessions[preview_id]
+
+    def register_audio_stop_func(self,
+                                 preview_id: int,
+                                 stop_func: Callable[..., None]
+                                 ) -> None:
+
+        self._audio_stop_functions[preview_id] = stop_func
+
+    def unregister_audio_stop_func(self, preview_id: int) -> None:
+        self._audio_stop_functions.pop(preview_id, None)
+
+    def stop_audio_except(self, preview_id: int) -> None:
+        # Stops playback of all audio previews except of for preview_id.
+        # This makes sure that only one preview is played at the time.
+        for id_, stop_func in self._audio_stop_functions.items():
+            if id_ != preview_id:
+                stop_func()
 
     @staticmethod
     def _accept_uri(urlparts: ParseResult,
@@ -240,6 +283,7 @@ class PreviewManager:
             if app.settings.get('preview_allow_all_images'):
                 mime_type = guess_mime_type(uri)
                 if mime_type not in MIME_TYPES:
+                    log.info('%s not in allowed mime types', mime_type)
                     return False
 
                 if mime_type == 'application/octet-stream' and uri != oob_url:
@@ -264,8 +308,9 @@ class PreviewManager:
     def is_previewable(self,
                        text: str,
                        additional_data: AdditionalDataDict) -> bool:
-        if len(text.split(' ')) > 1:
-            # urlparse doesn't recognise spaces as URL delimiter
+        if not IRI_RX.fullmatch(text):
+            # urlparse removes whitespace (and who knows what else) from URLs,
+            # so can't be used for validation.
             return False
 
         uri = text
@@ -277,12 +322,11 @@ class PreviewManager:
         if not self._accept_uri(urlparts, uri, additional_data):
             return False
 
-        if uri.startswith('geo:'):
+        if urlparts.scheme == 'geo':
             try:
                 split_geo_uri(uri)
             except Exception as err:
-                log.error(uri)
-                log.error(err)
+                log.info('Bad geo URI %s: %s', uri, err)
                 return False
 
         return True
@@ -296,9 +340,11 @@ class PreviewManager:
         if uri.startswith('geo:'):
             preview = Preview(uri, None, None, None, 96, widget)
             preview.update_widget()
+            self._previews[preview.id] = preview
             return
 
         preview = self._process_web_uri(uri, widget, from_us, context)
+        self._previews[preview.id] = preview
 
         if not preview.orig_exists:
             if context is not None and not from_us:
@@ -401,107 +447,105 @@ class PreviewManager:
             log.info('Download already in progress')
             return
 
-        preview.reset_received_size()
         if preview.account is None:
             # History Window can be opened without account context
             # This means we can not apply proxy settings
             return
         log.info('Start downloading: %s', preview.request_uri)
-        message = Soup.Message.new('GET', preview.request_uri)
-        message.connect('starting', self._check_certificate, preview)
-        message.connect(
-            'content-sniffed', self._on_content_sniffed, preview, force)
-        message.connect('got-chunk', self._on_got_chunk, preview)
 
-        session = self._get_session(preview.account)
-        session.queue_message(message, self._on_finished, preview)
+        request = create_http_request(preview.account)
+        request.set_user_data(preview)
+        request.connect('accept-certificate', self._accept_certificate)
+        request.connect('content-sniffed', self._on_content_sniffed, force)
+        request.connect('response-progress', self._on_response_progress)
 
-    def _check_certificate(self,
-                           message: Soup.Message,
-                           preview: Preview) -> None:
-        _https_used, _tls_certificate, tls_errors = message.get_https_status()
+        request.send('GET', preview.request_uri, callback=self._on_finished)
+
+    def _accept_certificate(self,
+                            request: HTTPRequest,
+                            certificate: Gio.TlsCertificate,
+                            certificate_errors: Gio.TlsCertificateFlags,
+                            ) -> bool:
 
         if not app.settings.get('preview_verify_https'):
-            return
+            return True
 
-        if tls_errors:
-            phrase = get_tls_error_phrase(tls_errors)
-            log.warning('TLS verification failed: %s', phrase)
-            session = self._get_session(preview.account)
-            session.cancel_message(message, Soup.Status.CANCELLED)
-            preview.info_message = _('TLS verification failed: %s') % phrase
-            preview.update_widget()
+        phrases = get_tls_error_phrases(
+            convert_tls_error_flags(certificate_errors))
+        log.warning(
+            'TLS verification failed: %s (0x%02x)', phrases, certificate_errors)
+
+        preview = cast(Preview, request.get_user_data())
+        preview.info_message = _('TLS verification failed: %s') % phrases[0]
+        preview.update_widget()
+        return False
 
     def _on_content_sniffed(self,
-                            message: Soup.Message,
-                            type_: str,
-                            _params: GLib.HashTable,
-                            preview: Preview,
-                            force: bool) -> None:
-        file_size = message.props.response_headers.get_content_length()
-        uri = message.props.uri.to_string(False)
-        session = self._get_session(preview.account)
-        preview.mime_type = type_
-        preview.file_size = file_size
+                            request: HTTPRequest,
+                            content_length: int,
+                            content_type: str,
+                            force: bool
+                            ) -> None:
 
-        if type_ not in ALLOWED_MIME_TYPES and not force:
-            log.info('Not an allowed content type: %s, %s', type_, uri)
-            session.cancel_message(message, Soup.Status.CANCELLED)
+        uri = request.get_uri().to_string()
+        preview = cast(Preview, request.get_user_data())
+        preview.mime_type = content_type
+        preview.file_size = content_length
+
+        if content_type not in ALLOWED_MIME_TYPES and not force:
+            log.info('Not an allowed content type: %s, %s', content_type, uri)
+            request.cancel()
             return
 
-        if file_size == 0:
-            log.info('File size is unknown (zero) for URL: "%s"', uri)
-            session.cancel_message(message, Soup.Status.CANCELLED)
-            return
-
-        if file_size > int(app.settings.get('preview_max_file_size')):
+        if content_length > int(app.settings.get('preview_max_file_size')):
             log.info(
                 'File size (%s) too big for URL: "%s"',
-                file_size, uri)
+                content_length, uri)
             if force:
                 preview.info_message = None
             else:
-                session.cancel_message(message, Soup.Status.CANCELLED)
+                request.cancel()
                 preview.info_message = _('Automatic preview disabled '
                                          '(file too big)')
 
         preview.update_widget()
 
-    def _on_got_chunk(self,
-                      message: Soup.Message,
-                      chunk: Soup.Buffer,
-                      preview: Preview
-                      ) -> None:
-        preview.update_progress(len(chunk.get_data()), message)
+    def _on_response_progress(self,
+                              request: HTTPRequest,
+                              progress: float,
+                              ) -> None:
 
-    def _on_finished(self,
-                     _session: Soup.Session,
-                     message: Soup.Message,
-                     preview: Preview
-                     ) -> None:
+        preview = cast(Preview, request.get_user_data())
+        preview.update_progress(progress, request)
 
+    def _on_finished(self, request: HTTPRequest) -> None:
+
+        preview = cast(Preview, request.get_user_data())
         preview.download_in_progress = False
 
-        if message.status_code != Soup.Status.OK:
-            log.warning('Download failed: %s', preview.request_uri)
-            status_code = Soup.Status.get_phrase(message.status_code)
-            log.warning(status_code)
-            preview.reset_received_size()
-            if message.status_code != 1:
-                # status_code 1: 'Cancelled'
-                preview.info_message = _('Download failed (%s)') % status_code
+        if not request.is_complete():
+            error = request.get_error_string()
+            log.warning('Download failed: %s - %s', preview.request_uri, error)
+            if request.get_error() != HTTPRequestError.CANCELLED:
+                preview.info_message = _('Download failed (%s)') % error
             preview.update_widget()
             return
 
         preview.info_message = None
 
-        data = message.props.response_body_data.get_data()
-        if data is None:
+        data = request.get_data()
+        if not data:
             return
 
         if preview.is_aes_encrypted:
             if preview.key is not None and preview.iv is not None:
-                data = aes_decrypt(preview.key, preview.iv, data)
+                try:
+                    data = aes_decrypt(preview.key, preview.iv, data)
+                except Exception as error:
+                    log.exception('Decryption failed')
+                    preview.info_message = _('Decryption failed')
+                    preview.update_widget()
+                    return
 
         if preview.mime_type == 'application/octet-stream':
             if preview.orig_path is not None:
@@ -573,6 +617,4 @@ class PreviewManager:
         preview.update_widget(data=pixbuf)
 
     def cancel_download(self, preview: Preview) -> None:
-        preview.reset_received_size()
-        session = self._get_session(preview.account)
-        session.cancel_message(preview.soup_message, Soup.Status.CANCELLED)
+        preview.request.cancel()
