@@ -27,15 +27,16 @@ class _DEFAULT:
     pass
 
 
-class _ACCOUNT_DEFAULT:
+class _ACCOUNTDEFAULT:
     pass
 
 
 HAS_APP_DEFAULT = _DEFAULT()
-HAS_ACCOUNT_DEFAULT = _ACCOUNT_DEFAULT()
+HAS_ACCOUNT_DEFAULT = _ACCOUNTDEFAULT()
 
 
 BoolSettings = Literal[
+    'allow_open_file_uris',
     'always_english_wikipedia',
     'always_english_wiktionary',
     'ascii_formatting',
@@ -46,11 +47,10 @@ BoolSettings = Literal[
     'check_for_update',
     'confirm_close_muc',
     'confirm_on_window_delete',
-    'confirm_paste_image',
     'dev_force_bookmark_2',
-    'dev_use_message_label',
     'developer_modus',
     'enable_emoji_shortcodes',
+    'enable_keepassxc_integration',
     'enable_negative_priority',
     'enable_file_preview',
     'escape_key_closes',
@@ -58,8 +58,6 @@ BoolSettings = Literal[
     'gc_notify_on_all_messages_public_default',
     'gc_print_join_left_default',
     'gc_print_status_default',
-    'hide_chat_banner',
-    'hide_groupchat_banner',
     'hide_groupchat_occupants_list',
     'ignore_incoming_attention',
     'is_window_visible',
@@ -130,15 +128,15 @@ IntSettings = Literal[
 
 StringSettings = Literal[
     'action_on_close',
+    'additional_uri_schemes',
     'audio_input_device',
     'audio_output_device',
     'autoaway_message',
     'autoxa_message',
-    'chat_timestamp_format',
     'confirm_block',
-    'date_timestamp_format',
+    'date_format',
+    'date_time_format',
     'dictionary_url',
-    'emoticons_theme',
     'gc_refer_to_nick_char',
     'global_proxy',
     'last_save_dir',
@@ -156,6 +154,7 @@ StringSettings = Literal[
     'show_main_window_on_startup',
     'speller_language',
     'stun_server',
+    'time_format',
     'video_framerate',
     'video_input_device',
     'video_size',
@@ -165,6 +164,8 @@ AllSettings = Literal[BoolSettings, IntSettings, StringSettings]
 AllSettingsT = Union[str, int, bool, list[str]]
 
 APP_SETTINGS = {
+    'additional_uri_schemes': '',
+    'allow_open_file_uris': False,
     'always_english_wikipedia': False,
     'always_english_wiktionary': True,
     'ascii_formatting': True,
@@ -181,20 +182,19 @@ APP_SETTINGS = {
     'autoxatime': 15,
     'chat_handle_position': 350,
     'chat_merge_consecutive_nickname': True,
-    'chat_timestamp_format': '%H:%M',
     'check_for_update': True,
     'confirm_block': '',
     'confirm_close_muc': True,
     'confirm_on_window_delete': True,
-    'confirm_paste_image': True,
     'dark_theme': 2,
-    'date_timestamp_format': '%x',
+    'date_format': '%x',
+    'date_time_format': '%c',
     'dev_force_bookmark_2': False,
     'dev_use_message_label': True,
     'developer_modus': False,
     'dictionary_url': 'WIKTIONARY',
-    'emoticons_theme': 'noto-emoticons',
     'enable_emoji_shortcodes': True,
+    'enable_keepassxc_integration': False,
     'enable_negative_priority': False,
     'enable_file_preview': True,
     'escape_key_closes': False,
@@ -208,9 +208,7 @@ APP_SETTINGS = {
     'gc_sync_threshold_private_default': 0,
     'gc_sync_threshold_public_default': 1,
     'global_proxy': '',
-    'groupchat_roster_width': 210,
-    'hide_chat_banner': False,
-    'hide_groupchat_banner': False,
+    'groupchat_roster_width': 250,
     'hide_groupchat_occupants_list': False,
     'ignore_incoming_attention': False,
     'is_window_visible': True,
@@ -272,11 +270,12 @@ APP_SETTINGS = {
     'sounds_on': True,
     'speller_language': '',
     'stun_server': '',
+    'time_format': '%H:%M',
     'trayicon_notification_on_events': True,
     'use_keyring': True,
     'use_kib_mib': False,
     'use_libappindicator': True,
-    'use_speller': False,
+    'use_speller': True,
     'use_stun_server': False,
     'use_urgency_hint': True,
     'video_framerate': '',
@@ -315,7 +314,8 @@ BoolAccountSettings = Literal[
     'test_ft_proxies_on_startup',
     'use_custom_host',
     'use_ft_proxies',
-    'use_plain_connection'
+    'use_plain_connection',
+    'omemo_blind_trust'
 ]
 
 
@@ -327,6 +327,7 @@ StringAccountSettings = Literal[
     'client_cert',
     'custom_host',
     'custom_type',
+    'default_workspace',
     'file_transfer_proxies',
     'filetransfer_preference',
     'gc_send_chatstate_default',
@@ -339,7 +340,6 @@ StringAccountSettings = Literal[
     'name',
     'password',
     'proxy',
-    'recent_groupchats',
     'resource',
     'roster_version',
     'send_chatstate_default',
@@ -376,6 +376,7 @@ BoolGroupChatSettings = Literal[
 
 StringGroupChatSettings = Literal[
     'encryption',
+    'mute_until',
     'speller_language',
     'send_chatstate',
 ]
@@ -398,6 +399,7 @@ BoolContactSettings = Literal[
 
 StringContactSettings = Literal[
     'encryption',
+    'mute_until',
     'speller_language',
     'send_chatstate',
 ]
@@ -412,13 +414,13 @@ ACCOUNT_SETTINGS = {
     'account': {
         'account_color': 'rgb(85, 85, 85)',
         'account_label': '',
-        'active': True,
+        'active': False,
         'adjust_priority_with_status': False,
         'anonymous_auth': False,
         'answer_receipts': True,
         'attached_gpg_keys': '',
         'autoauth': False,
-        'autoconnect': False,
+        'autoconnect': True,
         'autopriority_away': 40,
         'autopriority_chat': 50,
         'autopriority_dnd': 20,
@@ -432,6 +434,7 @@ ACCOUNT_SETTINGS = {
         'custom_host': '',
         'custom_port': 5222,
         'custom_type': 'START TLS',
+        'default_workspace': '',
         'enable_gssapi': False,
         'enable_security_labels': False,
         'file_transfer_proxies': '',
@@ -454,19 +457,18 @@ ACCOUNT_SETTINGS = {
         'proxy': '',
         'publish_location': False,
         'publish_tune': False,
-        'recent_groupchats': '',
         'request_user_data': True,
         'resource': 'gajim.$rand',
         'restore_last_status': False,
         'roster_version': '',
-        'savepass': False,
+        'savepass': True,
         'send_chatstate_default': 'composing_only',
         'send_idle_time': True,
         'send_marker_default': True,
         'send_os_info': True,
         'send_time_info': True,
         'subscription_request_msg': '',
-        'sync_with_global_status': False,
+        'sync_with_global_status': True,
         'test_ft_proxies_on_startup': False,
         'use_custom_host': False,
         'use_ft_proxies': False,
@@ -475,10 +477,12 @@ ACCOUNT_SETTINGS = {
         'zeroconf_first_name': '',
         'zeroconf_jabber_id': '',
         'zeroconf_last_name': '',
+        'omemo_blind_trust': True,
     },
 
     'contact': {
         'encryption': '',
+        'mute_until': '',
         'send_chatstate': HAS_ACCOUNT_DEFAULT,
         'send_marker': HAS_ACCOUNT_DEFAULT,
         'speller_language': '',
@@ -486,6 +490,7 @@ ACCOUNT_SETTINGS = {
 
     'group_chat': {
         'encryption': '',
+        'mute_until': '',
         'notify_on_all_messages': HAS_APP_DEFAULT,
         'print_join_left': HAS_APP_DEFAULT,
         'print_status': HAS_APP_DEFAULT,
@@ -585,6 +590,11 @@ PROXY_EXAMPLES = {
         'host': 'localhost',
         'port': 9050
     },
+    'Tor (Browser)': {
+        'type': 'socks5',
+        'host': 'localhost',
+        'port': 9150
+    },
 }
 
 
@@ -630,6 +640,10 @@ DEFAULT_SOUNDEVENT_SETTINGS = {
 
 ADVANCED_SETTINGS = {
     'app': {
+        'additional_uri_schemes': _(
+            'Clickable schemes in addition to the hard-coded list of '
+            'IANA-registered ones. Space-separated, lower-case, no colons.'),
+        'allow_open_file_uris': _('Allows opening file:/ URIs when clicked.'),
         'always_english_wikipedia': '',
         'always_english_wiktionary': '',
         'ascii_formatting': _(
@@ -638,15 +652,14 @@ ADVANCED_SETTINGS = {
             'Show message meta data (avatar, nickname, timestamp) only once, '
             'if there are multiple messages from the same sender within a '
             'specific timespan.'),
-        'chat_timestamp_format': 'https://docs.python.org/3/library/time.html#time.strftime',  # noqa: E501
         'confirm_block': _(
             'Show a confirmation dialog to block a contact? Empty string '
             'means never show the dialog.'),
         'confirm_close_muc': _('Ask before closing a group chat tab/window.'),
         'confirm_on_window_delete': _(
             'Ask before quitting when Gajim’s window is closed'),
-        'confirm_paste_image': _('Ask before pasting an image.'),
-        'date_timestamp_format': 'https://docs.python.org/3/library/time.html#time.strftime',  # noqa: E501
+        'date_format': 'https://docs.python.org/3/library/time.html#time.strftime',  # noqa: E501
+        'date_time_format': 'https://docs.python.org/3/library/time.html#time.strftime',  # noqa: E501
         'dev_force_bookmark_2': _('Force Bookmark 2 usage'),
         'dev_use_message_label': '',
         'developer_modus': '',
@@ -658,7 +671,6 @@ ADVANCED_SETTINGS = {
             'account in the Accounts window. BE CAREFUL, when you are logged '
             'in with a negative priority, you will NOT receive any message '
             'from your server.'),
-        'escape_key_closes': _('If enabled, pressing Esc closes a tab/window.'),
         'file_transfers_port': '',
         'ft_add_hosts_to_send': _(
             'List of send hosts (comma separated) in '
@@ -670,8 +682,6 @@ ADVANCED_SETTINGS = {
             'Character to add after nickname when using nickname completion '
             '(tab) in group chat.'),
         'groupchat_roster_width': _('Width of group chat roster in pixel'),
-        'hide_chat_banner': _('Hides the banner in a 1:1 chat window.'),
-        'hide_groupchat_banner': _('Hides the banner in a group chat window.'),
         'ignore_incoming_attention': _(
             'If enabled, Gajim will ignore incoming attention '
             'requests ("wizz").'),
@@ -694,7 +704,10 @@ ADVANCED_SETTINGS = {
         'search_engine': '',
         'send_on_ctrl_enter': _(
             'Send message on Ctrl+Enter and make a new line with Enter.'),
+        'show_chatstate_in_banner': _(
+            'Show chat state (e.g. "is typing…") next to your contact’s name'),
         'stun_server': _('STUN server to use when using Jingle'),
+        'time_format': 'https://docs.python.org/3/library/time.html#time.strftime',  # noqa: E501
         'trayicon_notification_on_events': _(
             'Notify of events in the notification area.'),
         'use_kib_mib': _(
