@@ -24,15 +24,16 @@
 from typing import Any
 from typing import Iterator
 from typing import KeysView
+from typing import Literal
 from typing import NamedTuple
 from typing import Optional
 
-import time
-import datetime
 import calendar
+import datetime
 import json
 import logging
 import sqlite3 as sqlite
+import time
 from collections import namedtuple
 
 from nbxmpp import JID
@@ -41,16 +42,15 @@ from nbxmpp.structs import MessageProperties
 
 from gajim.common import app
 from gajim.common import configpaths
-from gajim.common.helpers import AdditionalDataDict
-from gajim.common.const import MAX_MESSAGE_CORRECTION_DELAY
-from gajim.common.const import KindConstant
 from gajim.common.const import JIDConstant
+from gajim.common.const import KindConstant
+from gajim.common.const import MAX_MESSAGE_CORRECTION_DELAY
+from gajim.common.helpers import AdditionalDataDict
 from gajim.common.modules.contacts import GroupchatContact
-
 from gajim.common.storage.base import SqliteStorage
 from gajim.common.storage.base import timeit
 
-CURRENT_USER_VERSION = 6
+CURRENT_USER_VERSION = 7
 
 ARCHIVE_SQL_STATEMENT = '''
     CREATE TABLE jids(
@@ -63,6 +63,8 @@ ARCHIVE_SQL_STATEMENT = '''
             account_id INTEGER,
             jid_id INTEGER,
             contact_name TEXT,
+            occupant_id TEXT,
+            real_jid TEXT,
             time INTEGER,
             kind INTEGER,
             show INTEGER,
@@ -98,13 +100,13 @@ class JidsTableRow(NamedTuple):
 
 
 class ConversationRow(NamedTuple):
-    contact_name: str
+    contact_name: str | None
     time: float
     kind: int
     show: int
     message: str
     subject: str
-    additional_data: AdditionalDataDict
+    additional_data: AdditionalDataDict | None
     log_line_id: int
     message_id: str
     stanza_id: str
@@ -116,8 +118,8 @@ class LastConversationRow(NamedTuple):
     contact_name: str
     time: float
     kind: int
-    message: str
-    additional_data: AdditionalDataDict
+    message: str | None
+    additional_data: AdditionalDataDict | None
     message_id: str
     stanza_id: str
 
@@ -147,7 +149,7 @@ class MessageMetaRow(NamedTuple):
 class LastArchiveMessageRow(NamedTuple):
     id: int
     last_mam_id: str
-    oldest_mam_timestamp: str
+    oldest_mam_timestamp: str | None
     last_muc_timestamp: str
 
 
@@ -160,10 +162,11 @@ class MessageExportRow(NamedTuple):
 
 
 class MessageArchiveStorage(SqliteStorage):
-    def __init__(self):
+    def __init__(self, in_memory: bool = False):
+        path = None if in_memory else configpaths.get('LOG_DB')
         SqliteStorage.__init__(self,
                                log,
-                               configpaths.get('LOG_DB'),
+                               path,
                                ARCHIVE_SQL_STATEMENT)
 
         self._jid_ids: dict[JID, JidsTableRow] = {}
@@ -188,7 +191,7 @@ class MessageArchiveStorage(SqliteStorage):
 
         assert cursor.description is not None
         fields = [col[0] for col in cursor.description]
-        Row = namedtuple('Row', fields)  # type: ignore
+        Row = namedtuple('Row', fields)  # pyright: ignore
         named_row = Row(*row)
         if 'additional_data' in fields:
             _dict = json.loads(named_row.additional_data or '{}')
@@ -258,6 +261,14 @@ class MessageArchiveStorage(SqliteStorage):
             ]
             self._execute_multiple(statements)
 
+        if user_version < 7:
+            statements = [
+                'ALTER TABLE logs ADD COLUMN "real_jid" TEXT',
+                'ALTER TABLE logs ADD COLUMN "occupant_id" TEXT',
+                'PRAGMA user_version=7'
+            ]
+            self._execute_multiple(statements)
+
     @staticmethod
     def _like(search_str: str) -> str:
         return f'%{search_str}%'
@@ -394,7 +405,7 @@ class MessageArchiveStorage(SqliteStorage):
     def get_last_conversation_line(self,
                                    account: str,
                                    jid: JID
-                                   ) -> LastConversationRow:
+                                   ) -> LastConversationRow | None:
         '''
         Load the last line of a conversation with jid for account.
         Loads messages, but no status messages or error messages.
@@ -446,7 +457,7 @@ class MessageArchiveStorage(SqliteStorage):
         jids = [jid]
         account_id = self.get_account_id(account)
         kinds = map(str, [KindConstant.ERROR])
-        n_lines = 20
+        n_lines = 50
 
         sql_before = '''
             SELECT contact_name, time, kind, show, message, subject,
@@ -527,7 +538,7 @@ class MessageArchiveStorage(SqliteStorage):
 
     @timeit
     def search_log(self,
-                   _account: str,
+                   account: str,
                    jid: JID,
                    query: str,
                    from_users: Optional[list[str]] = None,
@@ -555,6 +566,7 @@ class MessageArchiveStorage(SqliteStorage):
 
         returns a list of namedtuples
         '''
+        account_id = self.get_account_id(account)
         jids = [jid]
 
         kinds = map(str, [KindConstant.STATUS,
@@ -565,9 +577,8 @@ class MessageArchiveStorage(SqliteStorage):
         else:
             before_ts = before.timestamp()
 
-        if after is None:
-            after_ts = datetime.datetime(1971, 1, 1).timestamp()
-        else:
+        after_ts = 0
+        if after is not None:
             after_ts = after.timestamp()
 
         if from_users is None:
@@ -579,12 +590,14 @@ class MessageArchiveStorage(SqliteStorage):
             SELECT account_id, jid_id, contact_name, time, kind, show, message,
                    subject, additional_data, log_line_id
             FROM logs NATURAL JOIN jids WHERE jid IN ({jids})
+            AND account_id = {account_id}
             AND message LIKE like(?)
             AND kind NOT IN ({kinds})
             {users_query}
             AND time BETWEEN ? AND ?
             ORDER BY time DESC, log_line_id
             '''.format(jids=', '.join('?' * len(jids)),
+                       account_id=account_id,
                        kinds=', '.join(kinds),
                        users_query=users_query_string)
 
@@ -641,9 +654,8 @@ class MessageArchiveStorage(SqliteStorage):
         else:
             before_ts = before.timestamp()
 
-        if after is None:
-            after_ts = datetime.datetime(1971, 1, 1).timestamp()
-        else:
+        after_ts = 0
+        if after is not None:
             after_ts = after.timestamp()
 
         if from_users is None:
@@ -839,90 +851,35 @@ class MessageArchiveStorage(SqliteStorage):
         '''
         jids = [contact.jid]
         account_id = self.get_account_id(contact.account)
-        kinds = map(str, [KindConstant.STATUS,
-                          KindConstant.GCSTATUS,
-                          KindConstant.ERROR])
+        kinds = map(str, [KindConstant.GC_MSG])
 
         sql = '''
             SELECT contact_name
             FROM logs NATURAL JOIN jids WHERE jid IN ({jids})
             AND account_id = {account_id}
-            AND kind NOT IN ({kinds})
+            AND kind IN ({kinds})
             ORDER BY time DESC
             '''.format(jids=', '.join('?' * len(jids)),
                        account_id=account_id,
                        kinds=', '.join(kinds))
 
-        result = self._con.execute(
-            sql,
-            tuple(jids)).fetchmany(50)
+
+        result = self._con.execute(sql, (tuple(jids))).fetchmany(50)
 
         nicknames: list[str] = []
         for row in result:
-            if row.contact_name not in nicknames:
+            if (row.contact_name not in nicknames and
+                    row.contact_name is not None):
                 nicknames.append(row.contact_name)
 
         return nicknames
 
-    @timeit
-    def deduplicate_muc_message(self,
-                                account: str,
-                                jid: str,
-                                resource: str,
-                                timestamp: float,
-                                message_id: str
-                                ) -> bool:
-        '''
-        Check if a message is already in the `logs` table
-
-        :param account:     The account
-
-        :param jid:         The muc jid as string
-
-        :param resource:    The resource
-
-        :param timestamp:   The timestamp in UTC epoch
-
-        :param message_id:  The message-id
-        '''
-
-        # Add 60 seconds around the timestamp
-        start_time = timestamp - 60
-        end_time = timestamp + 60
-
-        account_id = self.get_account_id(account)
-        log.debug('Search for MUC duplicate')
-        log.debug('start: %s, end: %s, jid: %s, resource: %s, message-id: %s',
-                  start_time, end_time, jid, resource, message_id)
-
-        sql = '''
-            SELECT * FROM logs
-            NATURAL JOIN jids WHERE
-            jid = ? AND
-            contact_name = ? AND
-            message_id = ? AND
-            account_id = ? AND
-            time BETWEEN ? AND ?
-            '''
-
-        result = self._con.execute(sql, (jid,
-                                         resource,
-                                         message_id,
-                                         account_id,
-                                         start_time,
-                                         end_time)).fetchone()
-
-        if result is not None:
-            log.debug('Found duplicate')
-            return True
-        return False
-
     @timeit
     def find_stanza_id(self,
                        account: str,
                        archive_jid: str,
-                       stanza_id: str,
-                       origin_id: Optional[str] = None,
+                       stanza_id: str | None,
+                       origin_id: str | None = None,
                        groupchat: bool = False
                        ) -> bool:
         '''
@@ -1241,6 +1198,19 @@ class MessageArchiveStorage(SqliteStorage):
 
         return lastrowid
 
+    @timeit
+    def delete_message_from_logs(self, log_line_id: int) -> None:
+        '''
+        Delete a message from the `logs` table
+
+        :param log_line_id: The message's log_line_id
+        '''
+        sql = 'DELETE FROM logs WHERE log_line_id = ?'
+        self._con.execute(sql, (log_line_id, ))
+
+        self._delayed_commit()
+        log.info('Deleted message with log_line_id %s', log_line_id)
+
     @timeit
     def set_message_error(self,
                           account_jid: str,
@@ -1280,20 +1250,12 @@ class MessageArchiveStorage(SqliteStorage):
                    account_jid: str,
                    jid: str,
                    message_id: str,
-                   state: str
+                   state: Literal['received', 'displayed']
                    ) -> None:
         '''
         Update the marker state of the corresponding message
-
-        :param account_jid: The jid of the account
-
-        :param jid:         The jid that belongs to the avatar
-
-        :param message_id:  The id of the message
-
-        :param state:       The state, 'received' or 'displayed'
-
         '''
+
         if state not in ('received', 'displayed'):
             raise ValueError('Invalid marker state')
 
@@ -1417,8 +1379,7 @@ class MessageArchiveStorage(SqliteStorage):
             results = cursor.fetchmany(10)
             if not results:
                 break
-            for result in results:
-                yield result
+            yield from results
 
     def remove_history(self, account: str, jid: JID) -> None:
         '''
