@@ -15,18 +15,15 @@
 from __future__ import annotations
 
 from typing import Any
-from typing import Type
-from typing import TypeVar
 from typing import NamedTuple
 from typing import Optional
-from typing import Union
+from typing import TypeVar
 
 import time
 from dataclasses import dataclass
 from dataclasses import fields
 
 from gi.repository import GLib
-
 from nbxmpp.const import Affiliation
 from nbxmpp.const import Chatstate
 from nbxmpp.const import PresenceShow
@@ -38,20 +35,20 @@ from nbxmpp.structs import MucSubject
 from nbxmpp.structs import PresenceProperties
 
 from gajim.common import types
-from gajim.common.const import MUCJoinedState
 from gajim.common.const import KindConstant
+from gajim.common.const import MUCJoinedState
 from gajim.common.const import PresenceShowExt
 from gajim.common.const import URIType
-from gajim.common.const import URIAction
-
 
 _T = TypeVar('_T')
 
 
 class URI(NamedTuple):
     type: URIType
-    action: Optional[URIAction] = None
-    data: Optional[Union[dict[str, str], str]] = None
+    source: str
+    query_type: str = ''
+    query_params: dict[str, str] = {}
+    data: dict[str, str] = {}
 
 
 class MUCData:
@@ -158,7 +155,6 @@ class OutgoingMessage:
         self.timestamp = None
         self.message_id = None
         self.stanza = None
-        self.session = None
         self.delayed = None  # TODO never set
 
         self.is_loggable = True
@@ -274,7 +270,7 @@ class VariantMixin:
     }
 
     def _get_type_and_variant_string(self,
-                                     field_type: str) -> tuple[Type[Any], str]:
+                                     field_type: str) -> tuple[type[Any], str]:
         variant_str = ''
         if 'Optional' in field_type:
             variant_str = 'm'
@@ -319,7 +315,7 @@ class VariantMixin:
         return GLib.Variant('a{sv}', vdict)
 
     @classmethod
-    def from_variant(cls: Type[_T], variant: GLib.Variant) -> _T:
+    def from_variant(cls: type[_T], variant: GLib.Variant) -> _T:
         vdict = variant.unpack()
         __types = vdict.pop('__types', None)
         if __types is not None:
