@@ -15,22 +15,22 @@
 # You should have received a copy of the GNU General Public License
 # along with Gajim. If not, see <http://www.gnu.org/licenses/>.
 
-from types import FrameType
 from typing import Optional
 
 import os
-import sys
+import platform
 import signal
 import sqlite3
-import platform
-from ctypes import CDLL, byref, create_string_buffer
+import sys
+from ctypes import byref
+from ctypes import CDLL
+from ctypes import create_string_buffer
 from ctypes.util import find_library
+from types import FrameType
+
 from packaging.version import Version as V
 
-import gajim.gui
-
-
-_MIN_NBXMPP_VER = '3.2.4'
+_MIN_NBXMPP_VER = '4.3.0'
 _MIN_GTK_VER = '3.24.30'
 _MIN_CAIRO_VER = '1.16.0'
 _MIN_PYGOBJECT_VER = '3.42.0'
@@ -65,8 +65,7 @@ def _check_required_deps() -> None:
                              'GtkSource': '4',
                              'GObject': '2.0',
                              'Pango': '1.0',
-                             'PangoCairo': '1.0',
-                             'Soup': '2.4'})
+                             'PangoCairo': '1.0'})
     except ValueError as error:
         sys.exit('Missing dependency: %s' % error)
 
@@ -102,7 +101,10 @@ def _init_gui(gui: str) -> None:
         _init_gtk()
 
 
-def _disable_csd() -> None:
+def _set_env_vars() -> None:
+    # Disable legacy ciphers in cryptography
+    os.environ['CRYPTOGRAPHY_OPENSSL_NO_LEGACY'] = '1'
+
     if sys.platform != 'win32':
         return
 
@@ -114,14 +116,12 @@ def _disable_csd() -> None:
 
 
 def _init_gtk() -> None:
-    gajim.gui.init('gtk')
-
-    from gajim.gui import exception
+    from gajim.gtk import exception
     exception.init()
 
 
 def _run_app() -> None:
-    from gajim.gui.application import GajimApplication
+    from gajim.gtk.application import GajimApplication
     application = GajimApplication()
 
     def sigint_cb(num: int, stack: Optional[FrameType]) -> None:
@@ -162,6 +162,6 @@ def main() -> None:
 
     _check_required_deps()
     _set_proc_title()
-    _disable_csd()
+    _set_env_vars()
     _init_gui('GTK')
     _run_app()
