@@ -22,31 +22,34 @@ import locale
 import logging
 from collections import defaultdict
 
-from gi.repository import Gtk
 from gi.repository import Gdk
-from gi.repository import Pango
 from gi.repository import GObject
+from gi.repository import Gtk
+from gi.repository import Pango
+from nbxmpp.stringprep import saslprep
 
 from gajim.common import app
 from gajim.common import passwords
+from gajim.common import types
 from gajim.common.const import ClientState
 from gajim.common.i18n import _
-from gajim.common.i18n import Q_
+from gajim.common.i18n import p_
 from gajim.common.settings import AllSettingsT
-from gajim.common import types
 
-from .dialogs import DialogButton
-from .dialogs import ConfirmationDialog
-from .const import Setting
-from .const import SettingKind
-from .const import SettingType
-from .settings import SettingsDialog
-from .settings import SettingsBox
-from .settings import PopoverSetting
-from .util import get_app_window
-from .util import open_window
+from gajim.gtk.const import Setting
+from gajim.gtk.const import SettingKind
+from gajim.gtk.const import SettingType
+from gajim.gtk.dialogs import ConfirmationDialog
+from gajim.gtk.dialogs import DialogButton
+from gajim.gtk.menus import build_accounts_menu
+from gajim.gtk.omemo_trust_manager import OMEMOTrustManager
+from gajim.gtk.settings import PopoverSetting
+from gajim.gtk.settings import SettingsBox
+from gajim.gtk.settings import SettingsDialog
+from gajim.gtk.util import get_app_window
+from gajim.gtk.util import open_window
 
-log = logging.getLogger('gajim.gui.accounts')
+log = logging.getLogger('gajim.gtk.accounts')
 
 
 class AccountsWindow(Gtk.ApplicationWindow):
@@ -166,9 +169,9 @@ class AccountsWindow(Gtk.ApplicationWindow):
         if not initial:
             self._accounts[account].show()
 
-    def select_account(self, account: str) -> None:
+    def select_account(self, account: str, page: Optional[str] = None) -> None:
         try:
-            self._accounts[account].select()
+            self._accounts[account].select(page)
         except KeyError:
             log.warning('select_account() failed, account %s not found',
                         account)
@@ -185,8 +188,8 @@ class Settings(Gtk.ScrolledWindow):
         self.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
         self._stack = Gtk.Stack(vhomogeneous=False)
         self._stack.set_transition_type(Gtk.StackTransitionType.CROSSFADE)
+        self._stack.get_style_context().add_class('settings-stack')
         self._stack.add_named(AddNewAccountPage(), 'add-account')
-        self.get_style_context().add_class('accounts-settings')
 
         self.add(self._stack)
         self._page_signal_ids: dict[GenericSettingPage, int] = {}
@@ -235,7 +238,8 @@ class AccountMenu(Gtk.Box):
 
         self._accounts_listbox = Gtk.ListBox()
         self._accounts_listbox.set_sort_func(self._sort_func)
-        self._accounts_listbox.get_style_context().add_class('settings-box')
+        self._accounts_listbox.get_style_context().add_class(
+            'accounts-menu-listbox')
         self._accounts_listbox.connect('row-activated',
                                        self._on_account_row_activated)
 
@@ -264,6 +268,7 @@ class AccountMenu(Gtk.Box):
             back_row.emit('activate')
         self._accounts_listbox.remove(row)
         sub_menu = self._stack.get_child_by_name(f'{row.account}-menu')
+        assert sub_menu is not None
         self._stack.remove(sub_menu)
         row.destroy()
         sub_menu.destroy()
@@ -292,6 +297,12 @@ class AccountMenu(Gtk.Box):
                       listbox.account,
                       f'{listbox.account}-{row.name}')
 
+    def set_page(self, account: str, page_name: str) -> None:
+        sub_menu = cast(
+            AccountSubMenu, self._stack.get_child_by_name(f'{account}-menu'))
+        sub_menu.select_row_by_name(page_name)
+        self.emit('menu-activated', account, f'{account}-{page_name}')
+
     def update_account_label(self, account: str) -> None:
         self._accounts_listbox.invalidate_sort()
         sub_menu = cast(
@@ -309,7 +320,7 @@ class AccountSubMenu(Gtk.ListBox):
         Gtk.ListBox.__init__(self)
         self.set_vexpand(True)
         self.set_hexpand(True)
-        self.get_style_context().add_class('settings-box')
+        self.get_style_context().add_class('accounts-menu-listbox')
 
         self._account = account
 
@@ -317,6 +328,7 @@ class AccountSubMenu(Gtk.ListBox):
         self.add(BackMenuItem())
         self.add(PageMenuItem('general', _('General')))
         self.add(PageMenuItem('privacy', _('Privacy')))
+        self.add(PageMenuItem('encryption-omemo', _('Encryption (OMEMO)')))
         self.add(PageMenuItem('connection', _('Connection')))
         self.add(PageMenuItem('advanced', _('Advanced')))
         self.add(RemoveMenuItem())
@@ -325,6 +337,14 @@ class AccountSubMenu(Gtk.ListBox):
     def account(self) -> str:
         return self._account
 
+    def select_row_by_name(self, row_name: str) -> None:
+        for row in self.get_children():
+            if not isinstance(row, PageMenuItem):
+                continue
+            if row.name == row_name:
+                self.select_row(row)
+                return
+
     def update(self) -> None:
         self.emit('update', self._account)
 
@@ -408,13 +428,15 @@ class PageMenuItem(MenuItem):
         super().__init__(name)
 
         if name == 'general':
-            icon = 'preferences-system-symbolic'
+            icon = 'avatar-default-symbolic'
         elif name == 'privacy':
             icon = 'preferences-system-privacy-symbolic'
+        elif name == 'encryption-omemo':
+            icon = 'channel-secure-symbolic'
         elif name == 'connection':
             icon = 'preferences-system-network-symbolic'
         elif name == 'advanced':
-            icon = 'preferences-other-symbolic'
+            icon = 'preferences-system-symbolic'
         else:
             icon = 'dialog-error-symbolic'
 
@@ -436,15 +458,18 @@ class Account:
         self._settings = settings
 
         self._settings.add_page(GeneralPage(account))
-        self._settings.add_page(ConnectionPage(account))
         self._settings.add_page(PrivacyPage(account))
+        self._settings.add_page(EncryptionOMEMOPage(account))
+        self._settings.add_page(ConnectionPage(account))
         self._settings.add_page(AdvancedPage(account))
 
         self._account_row = AccountRow(account)
         self._menu.add_account(self._account_row)
 
-    def select(self) -> None:
+    def select(self, page_name: Optional[str] = None) -> None:
         self._account_row.emit('activate')
+        if page_name is not None:
+            self._menu.set_page(self._account, page_name)
 
     def show(self) -> None:
         self._menu.show_all()
@@ -502,8 +527,10 @@ class AccountRow(Gtk.ListBoxRow):
         self._switch.set_vexpand(False)
 
         self._switch_state_label = Gtk.Label()
-        self._switch_state_label.set_xalign(1)
+        self._switch_state_label.set_xalign(0)
         self._switch_state_label.set_valign(Gtk.Align.CENTER)
+        label_width = max(len(p_('Switch', 'On')), len(p_('Switch', 'Off')))
+        self._switch_state_label.set_width_chars(label_width)
         self._set_label(account_enabled)
 
         self._switch.connect(
@@ -532,7 +559,7 @@ class AccountRow(Gtk.ListBoxRow):
         self._set_label(state)
 
     def _set_label(self, active: bool) -> None:
-        text = Q_('?switch:On') if active else Q_('?switch:Off')
+        text = p_('Switch', 'On') if active else p_('Switch', 'Off')
         self._switch_state_label.set_text(text)
 
     def _on_enable_switch(self,
@@ -556,9 +583,10 @@ class AccountRow(Gtk.ListBoxRow):
                 not app.get_client(account).state.is_disconnected):
             # Connecting or connected
             window = cast(AccountsWindow, get_app_window('AccountsWindow'))
+            account_label = app.get_account_label(account)
             ConfirmationDialog(
                 _('Disable Account'),
-                _('Account %s is still connected') % account,
+                _('Account %s is still connected') % account_label,
                 _('All chat and group chat windows will be closed.'),
                 [DialogButton.make('Cancel',
                                    callback=lambda: switch.set_active(True)),
@@ -569,9 +597,9 @@ class AccountRow(Gtk.ListBoxRow):
             return Gdk.EVENT_STOP
 
         if state:
-            app.interface.enable_account(account)
+            app.app.enable_account(account)
         else:
-            app.interface.disable_account(account)
+            app.app.disable_account(account)
 
         return Gdk.EVENT_PROPAGATE
 
@@ -582,7 +610,7 @@ class AccountRow(Gtk.ListBoxRow):
                           ) -> None:
 
         if client_state.is_disconnected:
-            app.interface.disable_account(client.account)
+            app.app.disable_account(client.account)
 
 
 class AddNewAccountPage(Gtk.Box):
@@ -616,10 +644,11 @@ class GenericSettingPage(Gtk.Box):
         Gtk.Box.__init__(self, orientation=Gtk.Orientation.VERTICAL, spacing=12)
         self.set_valign(Gtk.Align.START)
         self.set_vexpand(True)
+        self.get_style_context().add_class('settings-page')
         self.account = account
 
         self.listbox = SettingsBox(account)
-        self.listbox.get_style_context().add_class('accounts-settings-border')
+        self.listbox.get_style_context().add_class('border')
         self.listbox.set_selection_mode(Gtk.SelectionMode.NONE)
         self.listbox.set_vexpand(False)
         self.listbox.set_valign(Gtk.Align.END)
@@ -645,6 +674,8 @@ class GeneralPage(GenericSettingPage):
 
     def __init__(self, account: str) -> None:
 
+        workspaces = self._get_workspaces()
+
         settings = [
             Setting(SettingKind.ENTRY,
                     _('Label'),
@@ -652,6 +683,14 @@ class GeneralPage(GenericSettingPage):
                     'account_label',
                     callback=self._on_account_name_change),
 
+            Setting(SettingKind.POPOVER,
+                    _('Default Workspace'),
+                    SettingType.ACCOUNT_CONFIG,
+                    'default_workspace',
+                    props={'entries': workspaces},
+                    desc=_('Chats from this account will use this workspace by '
+                           'default')),
+
             Setting(SettingKind.COLOR,
                     _('Color'),
                     SettingType.ACCOUNT_CONFIG,
@@ -704,9 +743,18 @@ class GeneralPage(GenericSettingPage):
         ]
         GenericSettingPage.__init__(self, account, settings)
 
+    @staticmethod
+    def _get_workspaces() -> dict[str, str]:
+        workspaces: dict[str, str] = {'': _('Disabled')}
+        for workspace_id in app.settings.get_workspaces():
+            name = app.settings.get_workspace_setting(workspace_id, 'name')
+            workspaces[workspace_id] = name
+        return workspaces
+
     def _on_account_name_change(self, *args: Any) -> None:
         window = cast(AccountsWindow, get_app_window('AccountsWindow'))
         window.update_account_label(self.account)
+        build_accounts_menu()
 
 
 class PrivacyPage(GenericSettingPage):
@@ -873,6 +921,39 @@ class PrivacyPage(GenericSettingPage):
             'send_marker', None, context='private')
 
 
+class EncryptionOMEMOPage(GenericSettingPage):
+
+    name = 'encryption-omemo'
+
+    def __init__(self, account: str) -> None:
+        settings = [
+            Setting(SettingKind.SWITCH,
+                    _('Blind Trust'),
+                    SettingType.ACCOUNT_CONFIG,
+                    'omemo_blind_trust',
+                    desc=_('Blindly trust new devices until you verify them'))
+        ]
+        GenericSettingPage.__init__(self, account, settings)
+
+        heading = Gtk.Label(label=_('Trust Management'))
+        heading.get_style_context().add_class('bold')
+        heading.set_xalign(0)
+        self.add(heading)
+
+        btbv_label = Gtk.Label()
+        btbv_label.set_xalign(0)
+        markup = '<a href="%s">%s</a>' % (
+            'https://dev.gajim.org/gajim/gajim/-/wikis/help/OMEMO',
+            _('Read more about blind trust'))
+        btbv_label.set_markup(markup)
+        self.add(btbv_label)
+
+        omemo_trust_manager = OMEMOTrustManager(account)
+        omemo_trust_manager.set_margin_top(18)
+        self.pack_end(omemo_trust_manager, True, True, 0)
+        self.reorder_child(omemo_trust_manager, 0)
+
+
 class ConnectionPage(GenericSettingPage):
 
     name = 'connection'
@@ -1101,6 +1182,12 @@ class LoginDialog(SettingsDialog):
         self.connect('destroy', self.on_destroy)
 
     def on_password_change(self, new_password: str, _data: Any) -> None:
+        try:
+            new_password = saslprep(new_password)
+        except Exception:
+            # TODO: Show warning for invalid passwords
+            return
+
         passwords.save_password(self.account, new_password)
 
     def on_destroy(self, *args: Any) -> None:
