@@ -37,20 +37,25 @@ from __future__ import annotations
 
 from typing import Any
 from typing import Callable
-from typing import Optional
 from typing import cast
+from typing import Optional
 
 import os
 import sys
+from datetime import datetime
+from datetime import timedelta
+from datetime import timezone
 from urllib.parse import unquote
 
-from nbxmpp.namespaces import Namespace
-from nbxmpp import JID
-from nbxmpp.protocol import InvalidJid
+from gi.repository import Gdk
 from gi.repository import Gio
 from gi.repository import GLib
 from gi.repository import Gtk
-from gi.repository import Gdk
+from nbxmpp import JID
+from nbxmpp.const import ConnectionProtocol
+from nbxmpp.const import ConnectionType
+from nbxmpp.namespaces import Namespace
+from nbxmpp.protocol import InvalidJid
 
 import gajim
 from gajim.common import app
@@ -60,32 +65,35 @@ from gajim.common import ged
 from gajim.common import idle
 from gajim.common.application import CoreApplication
 from gajim.common.const import GAJIM_FAQ_URI
-from gajim.common.const import GAJIM_WIKI_URI
+from gajim.common.const import GAJIM_PRIVACY_POLICY_URI
 from gajim.common.const import GAJIM_SUPPORT_JID
+from gajim.common.const import GAJIM_WIKI_URI
 from gajim.common.exceptions import GajimGeneralException
 from gajim.common.helpers import load_json
 from gajim.common.helpers import open_uri
 from gajim.common.i18n import _
 
-from gajim.gui import menus
-from gajim.gui import structs
-from gajim.gui.about import AboutDialog
-from gajim.gui.avatar import AvatarStorage
-from gajim.gui.builder import get_builder
-from gajim.gui.const import ACCOUNT_ACTIONS
-from gajim.gui.const import ALWAYS_ACCOUNT_ACTIONS
-from gajim.gui.const import APP_ACTIONS
-from gajim.gui.const import FEATURE_ACCOUNT_ACTIONS
-from gajim.gui.const import ONLINE_ACCOUNT_ACTIONS
-from gajim.gui.dialogs import DialogButton
-from gajim.gui.dialogs import ConfirmationDialog
-from gajim.gui.dialogs import ShortcutsWindow
-from gajim.gui.discovery import ServiceDiscoveryWindow
-from gajim.gui.start_chat import StartChatDialog
-from gajim.gui.util import get_app_window
-from gajim.gui.util import load_user_iconsets
-from gajim.gui.util import open_window
-
+from gajim.gtk import menus
+from gajim.gtk import structs
+from gajim.gtk.about import AboutDialog
+from gajim.gtk.accounts import AccountsWindow
+from gajim.gtk.avatar import AvatarStorage
+from gajim.gtk.builder import get_builder
+from gajim.gtk.const import ACCOUNT_ACTIONS
+from gajim.gtk.const import ALWAYS_ACCOUNT_ACTIONS
+from gajim.gtk.const import APP_ACTIONS
+from gajim.gtk.const import FEATURE_ACCOUNT_ACTIONS
+from gajim.gtk.const import MuteState
+from gajim.gtk.const import ONLINE_ACCOUNT_ACTIONS
+from gajim.gtk.dialogs import ConfirmationDialog
+from gajim.gtk.dialogs import DialogButton
+from gajim.gtk.dialogs import ShortcutsWindow
+from gajim.gtk.discovery import ServiceDiscoveryWindow
+from gajim.gtk.start_chat import StartChatDialog
+from gajim.gtk.util import get_app_window
+from gajim.gtk.util import get_app_windows
+from gajim.gtk.util import load_user_iconsets
+from gajim.gtk.util import open_window
 
 ActionListT = list[tuple[str,
                          Callable[[Gio.SimpleAction, GLib.Variant], Any]]]
@@ -186,14 +194,9 @@ class GajimApplication(Gtk.Application, CoreApplication):
             GLib.OptionArg.NONE,
             _('Start a new chat'))
 
-        self.add_main_option(
-            'show', 0,
-            GLib.OptionFlags.NONE,
-            GLib.OptionArg.NONE,
-            _('Show Gajim'))
-
         self.add_main_option_entries(self._get_remaining_entry())
 
+        self.connect('activate', self._on_activate)
         self.connect('handle-local-options', self._handle_local_options)
         self.connect('command-line', self._command_line)
         self.connect('shutdown', self._shutdown)
@@ -216,8 +219,8 @@ class GajimApplication(Gtk.Application, CoreApplication):
         return [option]
 
     def _startup(self) -> None:
-        if sys.platform == 'win32':
-            # Changing the PANGOCAIRO_BACKEND is necessary on Windows
+        if sys.platform in ('win32', 'darwin'):
+            # Changing the PANGOCAIRO_BACKEND is necessary on Windows/MacOS
             # to render colored emoji glyphs
             os.environ['PANGOCAIRO_BACKEND'] = 'fontconfig'
 
@@ -230,7 +233,7 @@ class GajimApplication(Gtk.Application, CoreApplication):
         builder = get_builder('application_menu.ui')
         self.set_menubar(cast(Gio.Menu, builder.get_object('menubar')))
 
-        from gajim.gui import notification
+        from gajim.gtk import notification
         notification.init()
 
         self.avatar_storage = AvatarStorage()
@@ -240,18 +243,12 @@ class GajimApplication(Gtk.Application, CoreApplication):
         idle.Monitor.set_interval(app.settings.get('autoawaytime') * 60,
                                   app.settings.get('autoxatime') * 60)
 
-        if sys.platform == 'darwin':
-            # TODO: Remove if colored emoji rendering works well on
-            # Windows and MacOS
-            from gajim.gui.emoji_chooser import emoji_chooser
-            emoji_chooser.load()
-
         from gajim.gui_interface import Interface
 
         self.interface = Interface()
         self.interface.run(self)
 
-        from gajim.gui.status_icon import StatusIcon
+        from gajim.gtk.status_icon import StatusIcon
         self.systray = StatusIcon()
 
         self._add_app_actions()
@@ -263,13 +260,15 @@ class GajimApplication(Gtk.Application, CoreApplication):
         menus.build_accounts_menu()
         self.update_app_actions_state()
 
-        app.ged.register_event_handler('feature-discovered',
-                                       ged.CORE,
-                                       self._on_feature_discovered)
+        self.register_event('feature-discovered',
+                            ged.CORE,
+                            self._on_feature_discovered)
 
-        from gajim.gui.main import MainWindow
+        from gajim.gtk.main import MainWindow
         MainWindow()
 
+        GLib.timeout_add(100, self._auto_connect)
+
     def _open_uris(self, uris: list[str]) -> None:
         accounts = app.settings.get_active_accounts()
         if not accounts:
@@ -302,7 +301,7 @@ class GajimApplication(Gtk.Application, CoreApplication):
             if cmd == 'join':
                 if len(accounts) == 1:
                     self.activate_action(
-                        'groupchat-join',
+                        'open-chat',
                         GLib.Variant('as', [accounts[0], jid]))
                 else:
                     self.activate_action(
@@ -322,7 +321,7 @@ class GajimApplication(Gtk.Application, CoreApplication):
                     except Exception:
                         app.log('uri_handler').error('Invalid URI: %s', cmd)
 
-                app.interface.start_chat_from_jid(accounts[0], jid, message)
+                app.window.start_chat_from_jid(accounts[0], jid, message)
 
     def _shutdown(self, _application: GajimApplication) -> None:
         self._shutdown_core()
@@ -340,21 +339,25 @@ class GajimApplication(Gtk.Application, CoreApplication):
 
         remote_commands = [
             ('start-chat', GLib.Variant('as', ['', ''])),
-            ('show', None)
         ]
 
-        remaining = options.lookup_value(GLib.OPTION_REMAINING,
-                                         GLib.VariantType.new('as'))
-
         for cmd, parameter in remote_commands:
             if options.contains(cmd):
                 self.activate_action(cmd, parameter)
                 return 0
 
+        remaining = options.lookup_value(GLib.OPTION_REMAINING,
+                                         GLib.VariantType.new('as'))
+
         if remaining is not None:
             self._open_uris(remaining.unpack())
             return 0
 
+        if not options.contains('is-first-startup'):
+            # If no commands have been handled and it's not the first
+            # startup, raise the application.
+            self.activate()
+
         return 0
 
     def _handle_local_options(self,
@@ -380,10 +383,14 @@ class GajimApplication(Gtk.Application, CoreApplication):
                   'The primary instance will handle remote commands')
             return -1
 
+        options.insert_value('is-first-startup',  GLib.Variant('b', True))
         self._core_command_line(options)
         self._startup()
         return -1
 
+    def _on_activate(self, _application: Gtk.Application) -> None:
+        app.window.show()
+
     def _add_app_actions(self) -> None:
         for action in APP_ACTIONS:
             action_name, variant = action
@@ -410,17 +417,17 @@ class GajimApplication(Gtk.Application, CoreApplication):
             ('join-support-chat', self._on_join_support_chat),
             ('about', self._on_about_action),
             ('faq', self._on_faq_action),
+            ('privacy-policy', self._on_privacy_policy_action),
             ('start-chat', self._on_new_chat_action),
             ('accounts', self._on_accounts_action),
             ('add-contact', self._on_add_contact_action),
             ('copy-text', self._on_copy_text_action),
             ('open-link', self._on_open_link_action),
-            ('open-mail', self._on_open_mail_action),
             ('remove-history', self._on_remove_history_action),
             ('create-groupchat', self._on_create_groupchat_action),
             ('forget-groupchat', self._on_forget_groupchat_action),
-            ('groupchat-join', self._on_groupchat_join_action),
-            ('show', self._on_show),
+            ('open-chat', self._on_open_chat_action),
+            ('mute-chat', self._on_mute_chat_action),
         ]
 
         for action in actions:
@@ -446,7 +453,6 @@ class GajimApplication(Gtk.Application, CoreApplication):
     def _connect_account_actions(self, account: str) -> None:
         actions = [
             ('bookmarks', self._on_bookmarks_action),
-            ('open-chat', self._on_open_chat_action),
             ('add-contact', self._on_add_contact_account_action),
             ('services', self._on_services_action),
             ('profile', self._on_profile_action),
@@ -474,7 +480,7 @@ class GajimApplication(Gtk.Application, CoreApplication):
 
     def set_action_state(self, action_name: str, state: bool) -> None:
         action = self.lookup_action(action_name)
-        assert action is not None
+        assert isinstance(action, Gio.SimpleAction)
         action.set_enabled(state)
 
     def set_account_actions_state(self,
@@ -521,6 +527,66 @@ class GajimApplication(Gtk.Application, CoreApplication):
             action = '%s-block-contact' % event.account
             self.set_action_state(action, True)
 
+    def create_account(self,
+                       account: str,
+                       username: str,
+                       domain: str,
+                       password: str,
+                       proxy_name: str | None,
+                       custom_host: tuple[str,
+                                          ConnectionProtocol,
+                                          ConnectionType] | None,
+                       anonymous: bool = False
+                       ) -> None:
+
+        CoreApplication.create_account(self,
+                                       account,
+                                       username,
+                                       domain,
+                                       password,
+                                       proxy_name,
+                                       custom_host,
+                                       anonymous)
+
+        app.css_config.refresh()
+
+        # Action must be added before account window is updated
+        self.add_account_actions(account)
+
+        window = cast(AccountsWindow | None, get_app_window('AccountsWindow'))
+        if window is not None:
+            window.add_account(account)
+
+    def enable_account(self, account: str) -> None:
+        CoreApplication.enable_account(self, account)
+        menus.build_accounts_menu()
+        self.update_app_actions_state()
+        window = cast(AccountsWindow | None, get_app_window('AccountsWindow'))
+        if window is not None:
+            window.enable_account(account, True)
+
+    def disable_account(self, account: str) -> None:
+        for win in get_app_windows(account):
+            # Close all account specific windows, except the RemoveAccount
+            # dialog. It shows if the removal was successful.
+            if type(win).__name__ == 'RemoveAccount':
+                continue
+            win.destroy()
+
+        CoreApplication.disable_account(self, account)
+
+        menus.build_accounts_menu()
+        self.update_app_actions_state()
+
+    def remove_account(self, account: str) -> None:
+        CoreApplication.remove_account(self, account)
+
+        self.remove_account_actions(account)
+
+        window = cast(AccountsWindow | None, get_app_window('AccountsWindow'))
+        if window is not None:
+            window.remove_account(account)
+
     # Action Callbacks
 
     @staticmethod
@@ -566,7 +632,7 @@ class GajimApplication(Gtk.Application, CoreApplication):
 
         jid, initial_message = param.get_strv()
         open_window('StartChatDialog',
-                    jid=jid or None,
+                    initial_jid=jid or None,
                     initial_message=initial_message or None)
 
     @staticmethod
@@ -580,6 +646,9 @@ class GajimApplication(Gtk.Application, CoreApplication):
                             param: GLib.Variant) -> None:
         account = param.get_string()
         server_jid = app.settings.get_account_setting(account, 'hostname')
+        if account not in app.interface.instances:
+            app.interface.instances[account] = {'disco': {}}
+
         disco = app.interface.instances[account]['disco']
         if server_jid in disco:
             disco[server_jid].window.present()
@@ -600,7 +669,7 @@ class GajimApplication(Gtk.Application, CoreApplication):
     def _on_add_contact_account_action(_action: Gio.SimpleAction,
                                        param: GLib.Variant) -> None:
         account, jid = param.get_strv()
-        if jid is not None:
+        if jid:
             jid = JID.from_string(jid)
         open_window('AddContact', account=account or None, jid=jid or None)
 
@@ -670,16 +739,21 @@ class GajimApplication(Gtk.Application, CoreApplication):
                               _param: Optional[GLib.Variant]) -> None:
         accounts = app.settings.get_active_accounts()
         if len(accounts) == 1:
-            app.interface.show_add_join_groupchat(
+            app.window.show_add_join_groupchat(
                 accounts[0], GAJIM_SUPPORT_JID)
             return
-        open_window('StartChatDialog', jid=GAJIM_SUPPORT_JID)
+        open_window('StartChatDialog', initial_jid=GAJIM_SUPPORT_JID)
 
     @staticmethod
     def _on_faq_action(_action: Gio.SimpleAction,
                        _param: Optional[GLib.Variant]) -> None:
         open_uri(GAJIM_FAQ_URI)
 
+    @staticmethod
+    def _on_privacy_policy_action(_action: Gio.SimpleAction,
+                                  _param: Optional[GLib.Variant]) -> None:
+        open_uri(GAJIM_PRIVACY_POLICY_URI)
+
     @staticmethod
     def _on_shortcuts_action(_action: Gio.SimpleAction,
                              _param: Optional[GLib.Variant]) -> None:
@@ -735,14 +809,6 @@ class GajimApplication(Gtk.Application, CoreApplication):
 
         app.window.mark_as_read(params.account, params.jid)
 
-    @staticmethod
-    def _on_open_mail_action(_action: Gio.SimpleAction,
-                             param: GLib.Variant) -> None:
-        uri = param.get_string()
-        if not uri.startswith('mailto:'):
-            uri = 'mailto:%s' % uri
-        open_uri(uri)
-
     @staticmethod
     def _on_open_link_action(_action: Gio.SimpleAction,
                              param: GLib.Variant) -> None:
@@ -760,7 +826,23 @@ class GajimApplication(Gtk.Application, CoreApplication):
     def _on_open_chat_action(_action: Gio.SimpleAction,
                              param: GLib.Variant) -> None:
         account, jid = param.get_strv()
-        app.interface.start_chat_from_jid(account, jid)
+        app.window.start_chat_from_jid(account, jid)
+
+    @staticmethod
+    @structs.actionfunction
+    def _on_mute_chat_action(_action: Gio.SimpleAction,
+                             params: structs.MuteContactParam
+                             ) -> None:
+
+        client = app.get_client(params.account)
+        contact = client.get_module('Contacts').get_contact(params.jid)
+
+        if params.state == MuteState.UNMUTE:
+            contact.settings.set('mute_until', None)
+            return
+
+        until = datetime.now(timezone.utc) + timedelta(minutes=params.state)
+        contact.settings.set('mute_until', until.isoformat())
 
     @staticmethod
     @structs.actionfunction
@@ -769,10 +851,11 @@ class GajimApplication(Gtk.Application, CoreApplication):
                                   ) -> None:
         def _remove() -> None:
             app.storage.archive.remove_history(params.account, params.jid)
+
+            app.window.clear_chat_list_row(params.account, params.jid)
             control = app.window.get_control()
-            if params.jid is not None:
-                if not control.is_loaded(params.account, params.jid):
-                    return
+            if not control.is_loaded(params.account, params.jid):
+                return
 
             control.reset_view()
 
@@ -799,13 +882,3 @@ class GajimApplication(Gtk.Application, CoreApplication):
         client.get_module('Bookmarks').remove(params.jid)
 
         app.storage.archive.remove_history(params.account, params.jid)
-
-    @staticmethod
-    def _on_groupchat_join_action(_action: Gio.SimpleAction,
-                                  param: GLib.Variant) -> None:
-        account, jid = param.get_strv()
-        open_window('GroupchatJoin', account=account, jid=jid)
-
-    @staticmethod
-    def _on_show(_action: Gio.SimpleAction, param: GLib.Variant) -> None:
-        app.window.show()
