@@ -17,40 +17,40 @@ from __future__ import annotations
 from typing import Optional
 from typing import Union
 
-import logging
+import functools
 import hashlib
-from math import pi
-from functools import lru_cache
+import logging
 from collections import defaultdict
+from math import pi
 from pathlib import Path
 
+import cairo
 from gi.repository import Gdk
 from gi.repository import GdkPixbuf
 from gi.repository import Pango
 from gi.repository import PangoCairo
-import cairo
 from nbxmpp.protocol import JID
 
-from gajim.common import types
 from gajim.common import app
 from gajim.common import configpaths
-from gajim.common.helpers import Singleton
-from gajim.common.helpers import get_groupchat_name
+from gajim.common import types
 from gajim.common.const import AvatarSize
 from gajim.common.const import StyleAttr
+from gajim.common.helpers import get_groupchat_name
+from gajim.common.util.classes import Singleton
 
-from .const import DEFAULT_WORKSPACE_COLOR
-from .emoji_data_gtk import get_emoji_data
-from .util import load_icon_surface
-from .util import load_pixbuf
-from .util import text_to_color
-from .util import scale_with_ratio
-from .util import get_css_show_class
-from .util import convert_rgb_string_to_float
-from .util import rgba_to_float
-from .util import make_rgba
+from gajim.gtk.const import DEFAULT_WORKSPACE_COLOR
+from gajim.gtk.util import convert_rgb_string_to_float
+from gajim.gtk.util import get_contact_color
+from gajim.gtk.util import get_css_show_class
+from gajim.gtk.util import get_first_graphemes
+from gajim.gtk.util import load_icon_surface
+from gajim.gtk.util import load_pixbuf
+from gajim.gtk.util import make_rgba
+from gajim.gtk.util import rgba_to_float
+from gajim.gtk.util import scale_with_ratio
 
-log = logging.getLogger('gajim.gui.avatar')
+log = logging.getLogger('gajim.gtk.avatar')
 
 
 AvatarCacheT = dict[Union[JID, str], dict[tuple[int, int, Optional[str]],
@@ -58,20 +58,7 @@ AvatarCacheT = dict[Union[JID, str], dict[tuple[int, int, Optional[str]],
 
 
 def generate_avatar_letter(text: str) -> str:
-    if not text:
-        return ''
-
-    if text[0].isalpha():
-        return text[0].upper()
-
-    emoji_data = get_emoji_data()
-
-    # Max (arbitrary) length for emoji ZJW sequences: 11
-    for length in range(11, 0, -1):
-        if text[:length] in emoji_data.values():
-            return text[:length]
-
-    return text[0].upper()
+    return get_first_graphemes(text.lstrip(), 1).upper()
 
 
 def generate_avatar(letters: str,
@@ -83,8 +70,7 @@ def generate_avatar(letters: str,
     color_r, color_g, color_b = color
 
     # Set up colors and size
-    if scale is not None:
-        size = size * scale
+    size = size * scale
 
     width = size
     height = size
@@ -119,24 +105,23 @@ def generate_avatar(letters: str,
     context.set_operator(cairo.Operator.OVER)
     PangoCairo.show_layout(context, layout)
 
-    return context.get_target()  # pyright: ignore
+    return context.get_target()
 
 
-@lru_cache(maxsize=None)
+@functools.cache
 def generate_default_avatar(letter: str,
-                            color_string: str,
+                            color: tuple[float, float, float],
                             size: int,
                             scale: int,
                             style: str = 'circle') -> cairo.ImageSurface:
 
-    color = text_to_color(color_string)
     surface = generate_avatar(letter, color, size, scale)
     surface = clip(surface, style)
     surface.set_device_scale(scale, scale)
     return surface
 
 
-@lru_cache(maxsize=None)
+@functools.cache
 def make_workspace_avatar(letter: str,
                           color: tuple[float, float, float],
                           size: int,
@@ -197,10 +182,10 @@ def add_status_to_avatar(surface: cairo.ImageSurface,
         context.set_line_width(clip_radius / 4)
         context.stroke()
 
-    return context.get_target()  # pyright: ignore
+    return context.get_target()
 
 
-@lru_cache(maxsize=128)
+@functools.lru_cache(maxsize=128)
 def get_show_circle(show: Union[str, types.PresenceShowT],
                     size: int,
                     scale: int
@@ -234,7 +219,7 @@ def get_show_circle(show: Union[str, types.PresenceShowT],
         context.set_line_width(size / 10)
         context.stroke()
 
-    return context.get_target()  # pyright: ignore
+    return context.get_target()
 
 
 def fit(surface: cairo.ImageSurface, size: int) -> cairo.ImageSurface:
@@ -257,7 +242,7 @@ def fit(surface: cairo.ImageSurface, size: int) -> cairo.ImageSurface:
     context.scale(factor, factor)
     context.set_source_surface(square_surface, 0, 0)
     context.paint()
-    return context.get_target()  # pyright: ignore
+    return context.get_target()
 
 
 def square(surface: cairo.ImageSurface, size: int) -> cairo.ImageSurface:
@@ -281,7 +266,7 @@ def square(surface: cairo.ImageSurface, size: int) -> cairo.ImageSurface:
 
     context.set_source_surface(surface, x_pos, y_pos)
     context.paint()
-    return context.get_target()  # pyright: ignore
+    return context.get_target()
 
 
 def clip(surface: cairo.ImageSurface, mode: str) -> cairo.ImageSurface:
@@ -311,7 +296,7 @@ def clip_circle(surface: cairo.ImageSurface) -> cairo.ImageSurface:
     context.clip()
     context.paint()
 
-    return context.get_target()  # pyright: ignore
+    return context.get_target()
 
 
 def round_corners(surface: cairo.ImageSurface) -> cairo.ImageSurface:
@@ -342,7 +327,7 @@ def round_corners(surface: cairo.ImageSurface) -> cairo.ImageSurface:
 
     context.paint()
 
-    return context.get_target()  # pyright: ignore
+    return context.get_target()
 
 
 def convert_to_greyscale(surface: cairo.ImageSurface) -> cairo.ImageSurface:
@@ -355,7 +340,7 @@ def convert_to_greyscale(surface: cairo.ImageSurface) -> cairo.ImageSurface:
     context.set_source_rgba(1, 1, 1, 0.5)
     context.rectangle(0, 0, surface.get_width(), surface.get_height())
     context.fill()
-    return context.get_target()  # pyright: ignore
+    return context.get_target()
 
 
 class AvatarStorage(metaclass=Singleton):
@@ -378,7 +363,11 @@ class AvatarStorage(metaclass=Singleton):
 
         surface = self.get_surface(
             contact, size, scale, show, default, transport_icon, style)
-        return Gdk.pixbuf_get_from_surface(surface, 0, 0, size, size)
+        return Gdk.pixbuf_get_from_surface(surface,
+                                           0,
+                                           0,
+                                           size * scale,
+                                           size * scale)
 
     def get_surface(self,
                     contact: Union[types.BareContact,
@@ -414,11 +403,10 @@ class AvatarStorage(metaclass=Singleton):
                 return surface
 
         name = contact.name
-        color_string = str(contact.jid)
-
+        color = get_contact_color(contact)
         letter = generate_avatar_letter(name)
         surface = generate_default_avatar(
-            letter, color_string, size, scale, style=style)
+            letter, color, size, scale, style=style)
         if show is not None:
             surface = add_status_to_avatar(surface, show)
         self._cache[jid][(size, scale, show)] = surface
@@ -444,7 +432,7 @@ class AvatarStorage(metaclass=Singleton):
             if surface is not None:
                 return surface
 
-            avatar_sha = app.storage.cache.get_muc(jid, 'avatar')
+            avatar_sha = app.storage.cache.get_muc(account, jid, 'avatar')
             if avatar_sha is not None:
                 surface = self.surface_from_filename(avatar_sha, size, scale)
                 if surface is not None:
@@ -454,12 +442,15 @@ class AvatarStorage(metaclass=Singleton):
 
                 # avatar_sha set, but image is missing
                 # (e.g. avatar cache deleted)
-                app.storage.cache.set_muc(jid, 'avatar', None)
+                app.storage.cache.set_muc(account, jid, 'avatar', None)
 
         client = app.get_client(account)
+        contact = client.get_module('Contacts').get_bare_contact(jid)
+
         name = get_groupchat_name(client, jid)
+        color = get_contact_color(contact)
         letter = generate_avatar_letter(name)
-        surface = generate_default_avatar(letter, str(jid), size, scale, style)
+        surface = generate_default_avatar(letter, color, size, scale, style)
         self._cache[jid][(size, scale, None)] = surface
         return surface
 
@@ -518,8 +509,6 @@ class AvatarStorage(metaclass=Singleton):
 
         returns SHA1 value of the avatar or None on error
         '''
-        if data is None:
-            return None
 
         sha = hashlib.sha1(data).hexdigest()
         path = configpaths.get('AVATAR') / sha
@@ -538,6 +527,9 @@ class AvatarStorage(metaclass=Singleton):
             return None
         return path
 
+    def avatar_exists(self, filename: str) -> bool:
+        return self.get_avatar_path(filename) is not None
+
     def surface_from_filename(self,
                               filename: str,
                               size: int,
