@@ -15,31 +15,34 @@
 # You should have received a copy of the GNU General Public License
 # along with Gajim. If not, see <http://www.gnu.org/licenses/>.
 
+from __future__ import annotations
+
 from typing import Any
 from typing import cast
 from typing import Optional
 
-import os
 import logging
 from enum import IntEnum
 from enum import unique
 
+import cairo
 from gi.repository import Gdk
 from gi.repository import GdkPixbuf
 from gi.repository import GLib
 from gi.repository import Gtk
-import cairo
 
 from gajim.common import app
 from gajim.common.const import AvatarSize
-from gajim.common.i18n import _
 from gajim.common.helpers import get_file_path_from_dnd_dropped_uri
+from gajim.common.i18n import _
 
-from .dialogs import ErrorDialog
-from .filechoosers import AvatarChooserDialog
-from .util import scale_with_ratio
+from gajim.gtk import types
+from gajim.gtk.const import TARGET_TYPE_URI_LIST
+from gajim.gtk.dialogs import ErrorDialog
+from gajim.gtk.filechoosers import AvatarChooserDialog
+from gajim.gtk.util import scale_with_ratio
 
-log = logging.getLogger('gajim.gui.avatar_selector')
+log = logging.getLogger('gajim.gtk.avatar_selector')
 
 
 @unique
@@ -71,8 +74,13 @@ class AvatarSelector(Gtk.Box):
                          spacing=12)
         self.get_style_context().add_class('avatar-selector')
 
+        if app.is_flatpak():
+            target = 'application/vnd.portal.files'
+        else:
+            target = 'text/uri-list'
+
         uri_entry = Gtk.TargetEntry.new(
-            'text/uri-list', Gtk.TargetFlags.OTHER_APP, 80)
+            target, Gtk.TargetFlags.OTHER_APP, TARGET_TYPE_URI_LIST)
         dst_targets = Gtk.TargetList.new([uri_entry])
 
         self.drag_dest_set(
@@ -113,6 +121,7 @@ class AvatarSelector(Gtk.Box):
         self._load_button.show()
         self._helper_label.show()
         self._crop_area.hide()
+        self._crop_area.set_pixbuf(None)
 
     def prepare_crop_area(self, path: str) -> None:
         pixbuf = self._get_pixbuf_from_path(path)
@@ -126,11 +135,11 @@ class AvatarSelector(Gtk.Box):
         self._crop_area.show()
 
     def _on_load_clicked(self, _button: Gtk.Button) -> None:
-        def _on_file_selected(path: str) -> None:
-            self.prepare_crop_area(path)
+        def _on_file_selected(paths: list[str]) -> None:
+            self.prepare_crop_area(paths[0])
 
         AvatarChooserDialog(_on_file_selected,
-                            transient_for=self.get_toplevel(),
+                            transient_for=cast(Gtk.Window, self.get_toplevel()),
                             modal=True)
 
     def _on_drag_data_received(self,
@@ -145,12 +154,12 @@ class AvatarSelector(Gtk.Box):
         if not selection.get_data():
             return
 
-        if target_type == 80:
+        if target_type == TARGET_TYPE_URI_LIST:
             uri_split = selection.get_uris()  # Might be more than one
             path = get_file_path_from_dnd_dropped_uri(uri_split[0])
-            if not os.path.isfile(path):
+            if not path or not path.is_file():
                 return
-            self.prepare_crop_area(path)
+            self.prepare_crop_area(str(path))
 
     @staticmethod
     def _get_pixbuf_from_path(path: str) -> Optional[GdkPixbuf.Pixbuf]:
@@ -215,7 +224,7 @@ class CropArea(Gtk.DrawingArea):
         self._color_shifted_pixbuf: Optional[GdkPixbuf.Pixbuf] = None
         self._current_cursor: Optional[Gdk.CursorType] = None
 
-        self._scale = float(0.0)
+        self._scale = 1.0
         self._image.x = 0
         self._image.y = 0
         self._image.width = 0
@@ -264,12 +273,14 @@ class CropArea(Gtk.DrawingArea):
         self._crop.x = int(abs((width - self._crop.width) / 2))
         self._crop.y = int(abs((height - self._crop.height) / 2))
 
-        self._scale = 0.0
+        self._scale = 1.0
         self._image.x = 0
         self._image.y = 0
         self._image.width = 0
         self._image.height = 0
 
+        self._update_pixbufs()
+
         self.queue_draw()
 
     def get_pixbuf(self) -> Optional[GdkPixbuf.Pixbuf]:
@@ -287,7 +298,11 @@ class CropArea(Gtk.DrawingArea):
         return GdkPixbuf.Pixbuf.new_subpixbuf(
             self._browse_pixbuf, self._crop.x, self._crop.y, width, height)
 
-    def _on_draw(self, _widget: Gtk.Widget, context: cairo.Context) -> bool:
+    def _on_draw(self,
+                 _widget: Gtk.Widget,
+                 context: cairo.Context[types.SomeSurface]
+                 ) -> bool:
+
         if self._browse_pixbuf is None:
             return False
 
@@ -656,7 +671,7 @@ class CropArea(Gtk.DrawingArea):
 
             self._generate_color_shifted_pixbuf()
 
-            if self._scale == 0.0:
+            if self._scale == 1.0:
                 scale_to_80 = float(min(
                     (self._pixbuf.get_width() * 0.8 / self._base_width),
                     (self._pixbuf.get_height() * 0.8 / self._base_height)))
