@@ -14,9 +14,12 @@
 
 from __future__ import annotations
 
+from typing import Any
 from typing import cast
 from typing import Optional
 
+import logging
+
 from gi.repository import Gdk
 from gi.repository import Gio
 from gi.repository import GLib
@@ -27,11 +30,14 @@ from gajim.common import types
 from gajim.common.const import Direction
 from gajim.common.modules.contacts import GroupchatContact
 
-from .emoji_data_gtk import get_emoji_data
-from .groupchat_nick_completion import GroupChatNickCompletion
+from gajim.gtk.emoji_data_gtk import get_emoji_data
+from gajim.gtk.groupchat_nick_completion import GroupChatNickCompletion
+from gajim.gtk.menus import escape_mnemonic
 
+EMOJI_NUM_GENDERS = len(['f', 'm', 'n'])
+MENUS_MAX_ENTRIES = 2 * EMOJI_NUM_GENDERS
 
-MAX_ENTRIES = 5
+log = logging.getLogger('gajim.gtk.chat_action_processor')
 
 
 class ChatActionProcessor(Gtk.Popover):
@@ -51,6 +57,7 @@ class ChatActionProcessor(Gtk.Popover):
 
         self._message_input = message_input
         self._message_input.connect('key-press-event', self._on_key_press)
+        self._message_input.connect('focus-out-event', self._on_focus_out)
 
         self._buf = message_input.get_buffer()
         self._buf.connect('changed', self._on_changed)
@@ -103,6 +110,21 @@ class ChatActionProcessor(Gtk.Popover):
 
         return False
 
+    def _on_focus_out(self, *args: Any) -> bool:
+        win = self.get_window()
+        default_display = Gdk.Display.get_default()
+        if default_display is not None:
+            pointer = default_display.get_default_seat().get_pointer()
+            if win is not None and pointer is not None:
+                _win, x_pos, y_pos, _mod = win.get_device_position(pointer)
+                rect = self.get_allocation()
+                if (x_pos not in range(0, rect.width) or
+                        y_pos not in range(0, rect.height)):
+                    # Only popdown if click is outside of Popover's Rectangle
+                    self.popdown()
+
+        return False
+
     def _on_popover_closed(self, _popover: Gtk.Popover) -> None:
         self._active = False
         self._message_input.grab_focus()
@@ -173,7 +195,7 @@ class ChatActionProcessor(Gtk.Popover):
         for command, usage in command_list:
             if not command.startswith(action_text[1:]):
                 continue
-            if num_entries >= MAX_ENTRIES:
+            if num_entries >= MENUS_MAX_ENTRIES:
                 continue
 
             action_data = GLib.Variant('s', f'/{command}')
@@ -210,11 +232,16 @@ class ChatActionProcessor(Gtk.Popover):
                 if not text.startswith(':'):
                     return False
 
-            action_text = self._buf.get_text(start, self._current_iter, False)
+            action_text = self._buf.get_text(
+                start, self._current_iter, False)[1:]
             if self._start_mark is None:
                 self._start_mark = Gtk.TextMark.new('chat-action-start', True)
                 self._buf.add_mark(self._start_mark, start)
-            self._update_emoji_menu(action_text, start)
+            if self._active or len(action_text) > 1:
+                # Don't activate until a sufficient # of chars has been typed,
+                # which is chosen to be > 1 to not interfere with ASCII smilies
+                # consisting of a colon and 1 single other char.
+                self._update_emoji_menu(action_text.casefold(), start)
             return True
 
         return False
@@ -225,18 +252,37 @@ class ChatActionProcessor(Gtk.Popover):
                            ) -> None:
         self._menu.remove_all()
         emoji_data = get_emoji_data()
-        menu_entry_count = 0
-        for shortcode, codepoint in emoji_data.items():
-            if not shortcode.startswith(action_text[1:]):
+
+        sn_matches: dict[str, str] = {}
+        kw_matches: dict[str, str] = {}
+
+        for keyword, entries in emoji_data.items():
+            if not keyword.startswith(action_text):
                 continue
-            if menu_entry_count >= MAX_ENTRIES:
-                continue
-            action_data = GLib.Variant('s', codepoint)
+            for short_name, emoji in entries.items():
+                label = f'{emoji} {short_name}'
+                if keyword == short_name:
+                    # Replace a possible keyword match with the shortname match:
+                    sn_matches[emoji] = label
+                    if kw_matches.get(emoji) is not None:
+                        del kw_matches[emoji]
+                else:
+                    # Only add a keyword match if no shortname match:
+                    if sn_matches.get(emoji) is None:
+                        kw_matches[emoji] = f'{label}  [{keyword}]'
+
+        log.debug('Found %d "%s…" emoji by short name, %d more by keyword',
+                  len(sn_matches), action_text, len(kw_matches))
+
+        # Put all shortname matches before keyword matches:
+        for emoji, label in list(sn_matches.items()) + list(kw_matches.items()):
+            action_data = GLib.Variant('s', emoji)
             menu_item = Gio.MenuItem()
-            menu_item.set_label(f'{codepoint} {shortcode}')
+            menu_item.set_label(escape_mnemonic(label))
             menu_item.set_attribute_value('action-data', action_data)
             self._menu.append_item(menu_item)
-            menu_entry_count += 1
+            if self._menu.get_n_items() >= MENUS_MAX_ENTRIES:
+                break
 
         if self._menu.get_n_items() > 0:
             self._show_menu(start)
@@ -258,6 +304,7 @@ class ChatActionProcessor(Gtk.Popover):
     def _on_item_clicked(self, _button: Gtk.MenuButton, index: int) -> None:
         variant = self._menu.get_item_attribute_value(
             index, 'action-data')
+        assert variant is not None
         self._replace_text(variant.get_string())
         self.popdown()
 
@@ -274,6 +321,7 @@ class ChatActionProcessor(Gtk.Popover):
             if self._item_has_focus(item):
                 variant = self._menu.get_item_attribute_value(
                     items.index(item), 'action-data')
+                assert variant is not None
                 return variant.get_string()
         return ''
 
