@@ -14,9 +14,9 @@
 
 from __future__ import annotations
 
-from typing import Optional
 from typing import Any
 from typing import cast
+from typing import Optional
 from typing import Union
 
 import logging
@@ -26,24 +26,22 @@ from gi.repository import Gdk
 from gi.repository import GLib
 from gi.repository import GObject
 from gi.repository import Gtk
-
 from nbxmpp import JID
 
 from gajim.common import app
-from gajim.common import ged
 from gajim.common import events
+from gajim.common import ged
 from gajim.common.const import Direction
 from gajim.common.const import RowHeaderType
-from gajim.common.i18n import _
 from gajim.common.helpers import get_group_chat_nick
 from gajim.common.helpers import get_retraction_text
+from gajim.common.i18n import _
 from gajim.common.setting_values import OpenChatsSettingT
 
-from .chat_list_row import ChatListRow
-from .util import EventHelper
+from gajim.gtk.chat_list_row import ChatListRow
+from gajim.gtk.util import EventHelper
 
-
-log = logging.getLogger('gajim.gui.chatlist')
+log = logging.getLogger('gajim.gtk.chatlist')
 
 MessageEventT = Union[events.MessageReceived,
                       events.GcMessageReceived,
@@ -73,7 +71,10 @@ class ChatList(Gtk.ListBox, EventHelper):
         self.set_sort_func(self._sort_func)
         self._set_placeholder()
 
-        self._mouseover: bool = False
+        self._rows_need_sort = False
+        self._context_menu_visible = False
+        self._mouseover = False
+        self._pinned_order_change = False
         self.connect('enter-notify-event', self._on_mouse_focus_changed)
         self.connect('leave-notify-event', self._on_mouse_focus_changed)
 
@@ -146,7 +147,7 @@ class ChatList(Gtk.ListBox, EventHelper):
         return None
 
     def get_selected_chat(self) -> Optional[ChatListRow]:
-        row = cast(ChatListRow, self.get_selected_row())
+        row = cast(ChatListRow | None, self.get_selected_row())
         if row is None:
             return None
         return row
@@ -164,8 +165,9 @@ class ChatList(Gtk.ListBox, EventHelper):
 
     def mark_as_read(self, account: str, jid: JID) -> None:
         chat = self._chats.get((account, jid))
-        if chat is not None:
+        if chat is not None and chat.get_real_unread_count() > 0:
             chat.reset_unread()
+            app.ged.raise_event(events.ChatRead(account=account, jid=jid))
 
     def toggle_chat_pinned(self, account: str, jid: JID) -> None:
         row = self._chats[(account, jid)]
@@ -206,6 +208,8 @@ class ChatList(Gtk.ListBox, EventHelper):
 
         row.connect('drag-begin', self._on_row_drag_begin)
         row.connect('unread-changed', self._on_row_unread_changed)
+        row.connect('context-menu-state-changed',
+                    self._on_context_menu_state_changed)
 
         self.add(row)
 
@@ -303,6 +307,11 @@ class ChatList(Gtk.ListBox, EventHelper):
             self.remove_chat(account, jid)
         self._emit_unread_changed()
 
+    def clear_chat_list_row(self, account: str, jid: JID) -> None:
+        chat = self._chats.get((account, jid))
+        if chat is not None:
+            chat.clear()
+
     def contains_chat(self, account: str, jid: JID) -> bool:
         return self._chats.get((account, jid)) is not None
 
@@ -321,7 +330,7 @@ class ChatList(Gtk.ListBox, EventHelper):
             self._on_message_sent(event)
         elif isinstance(event, events.JingleRequestReceived):
             self._on_jingle_request_received(event)
-        elif isinstance(event, events.FileRequestReceivedEvent):  # pyright: ignore [reportUnnecessaryIsInstance] # noqa
+        elif isinstance(event, events.FileRequestReceivedEvent):  # pyright: ignore [reportUnnecessaryIsInstance] # noqa: E501
             self._on_file_request_received(event)
         else:
             log.warning('Unhandled Event: %s', event.name)
@@ -371,6 +380,14 @@ class ChatList(Gtk.ListBox, EventHelper):
     def _on_row_unread_changed(self, row: ChatListRow) -> None:
         self._emit_unread_changed()
 
+    def _on_context_menu_state_changed(self,
+                                       row: ChatListRow,
+                                       menu_is_visible: bool
+                                       ) -> None:
+
+        self._context_menu_visible = menu_is_visible
+        self._schedule_check_sort_inhibit()
+
     def _on_drag_data_received(self,
                                _widget: Gtk.Widget,
                                _drag_context: Gdk.DragContext,
@@ -392,7 +409,7 @@ class ChatList(Gtk.ListBox, EventHelper):
             log.debug('Dropped row is not pinned')
             return
 
-        row = cast(ChatListRow, self.get_row_at_y(y_coord))
+        row = cast(ChatListRow | None, self.get_row_at_y(y_coord))
         if row is not None:
             alloc = row.get_allocation()
             hover_row_y = alloc.y
@@ -436,7 +453,9 @@ class ChatList(Gtk.ListBox, EventHelper):
             row.position = self._chat_order.index(row)
 
         self.emit('chat-order-changed')
+        self._pinned_order_change = True
         self.invalidate_sort()
+        self._pinned_order_change = False
 
     def _update_time(self) -> bool:
         for _key, row in self._chats.items():
@@ -457,7 +476,7 @@ class ChatList(Gtk.ListBox, EventHelper):
         return text in row.contact_name.lower()
 
     @staticmethod
-    def _header_func(row: ChatListRow, before: ChatListRow) -> None:
+    def _header_func(row: ChatListRow, before: ChatListRow | None) -> None:
         if before is None:
             if row.is_pinned:
                 row.set_header_type(RowHeaderType.PINNED)
@@ -476,8 +495,9 @@ class ChatList(Gtk.ListBox, EventHelper):
                     row.set_header_type(None)
 
     def _sort_func(self, row1: ChatListRow, row2: ChatListRow) -> int:
-        if self._mouseover:
-            log.debug('Mouseover active, don’t sort rows')
+        if self._is_sort_inhibited():
+            self._rows_need_sort = True
+            log.debug('Sort inhibited')
             return 0
 
         # Sort pinned rows according to stored order
@@ -495,6 +515,33 @@ class ChatList(Gtk.ListBox, EventHelper):
         # Sort by timestamp
         return -1 if row1.timestamp > row2.timestamp else 1
 
+    def invalidate_sort(self) -> None:
+        if self._is_sort_inhibited():
+            return
+
+        self._rows_need_sort = False
+        log.debug('Sort Chatlist')
+        Gtk.ListBox.invalidate_sort(self)
+
+    def _is_sort_inhibited(self) -> bool:
+        if self._pinned_order_change:
+            return False
+        return self._mouseover or self._context_menu_visible
+
+    def _schedule_check_sort_inhibit(self) -> None:
+        if self._is_sort_inhibited():
+            return
+        GLib.timeout_add(100, self._check_sort_inhibit)
+
+    def _check_sort_inhibit(self) -> None:
+        if self._is_sort_inhibited():
+            return
+
+        if not self._rows_need_sort:
+            return
+
+        self.invalidate_sort()
+
     def _on_mouse_focus_changed(self,
                                 _widget: Gtk.ListBox,
                                 event: Gdk.EventCrossing
@@ -508,6 +555,8 @@ class ChatList(Gtk.ListBox, EventHelper):
                 # Not hovering a Gtk.ListBoxRow (row is INFERIOR)
                 self._mouseover = False
 
+        self._schedule_check_sort_inhibit()
+
     @staticmethod
     def _on_start_chat_clicked(_button: Gtk.Button) -> None:
         app.app.activate_action('start-chat', GLib.Variant('as', ['', '']))
