@@ -14,42 +14,46 @@
 
 from __future__ import annotations
 
-from typing import Optional
 from typing import Any
+from typing import Optional
 
 import pickle
+from datetime import datetime
+from urllib.parse import urlparse
 
+import cairo
 from gi.repository import Gdk
 from gi.repository import Gio
 from gi.repository import GLib
-from gi.repository import Gtk
 from gi.repository import GObject
-import cairo
-
+from gi.repository import Gtk
 from nbxmpp import JID
 
 from gajim.common import app
 from gajim.common.const import AvatarSize
 from gajim.common.const import KindConstant
 from gajim.common.const import RowHeaderType
-from gajim.common.i18n import _
-from gajim.common.helpers import get_groupchat_name
+from gajim.common.helpers import AdditionalDataDict
 from gajim.common.helpers import get_group_chat_nick
+from gajim.common.helpers import get_groupchat_name
 from gajim.common.helpers import get_retraction_text
 from gajim.common.helpers import get_uf_relative_time
 from gajim.common.helpers import message_needs_highlight
-from gajim.common.helpers import AdditionalDataDict
-from gajim.common.preview_helpers import filename_from_uri
-from gajim.common.preview_helpers import guess_simple_file_type
-from gajim.common.types import ChatContactT
-from gajim.common.types import OneOnOneContactT
+from gajim.common.i18n import _
 from gajim.common.modules.contacts import BareContact
 from gajim.common.modules.contacts import GroupchatContact
 from gajim.common.modules.contacts import GroupchatParticipant
+from gajim.common.preview_helpers import filename_from_uri
+from gajim.common.preview_helpers import format_geo_coords
+from gajim.common.preview_helpers import guess_simple_file_type
+from gajim.common.preview_helpers import split_geo_uri
+from gajim.common.storage.draft import DraftStorage
+from gajim.common.types import ChatContactT
+from gajim.common.types import OneOnOneContactT
 
-from .menus import get_chat_list_row_menu
-from .builder import get_builder
-from .util import GajimPopover
+from gajim.gtk.builder import get_builder
+from gajim.gtk.menus import get_chat_list_row_menu
+from gajim.gtk.util import GajimPopover
 
 
 class ChatListRow(Gtk.ListBoxRow):
@@ -59,6 +63,10 @@ class ChatListRow(Gtk.ListBoxRow):
             GObject.SignalFlags.RUN_LAST,
             None,
             ()),
+        'context-menu-state-changed': (
+            GObject.SignalFlags.RUN_LAST,
+            None,
+            (bool,)),
     }
 
     def __init__(self,
@@ -104,6 +112,13 @@ class ChatListRow(Gtk.ListBoxRow):
         self.connect('state-flags-changed', self._on_state_flags_changed)
         self.connect('destroy', self._on_destroy)
 
+        app.settings.connect_signal(
+            'mute_until',
+            self._on_mute_setting_changed,
+            self.contact.account,
+            self.contact.jid)
+        self._on_mute_setting_changed()
+
         # Drag and Drop
         entries = [Gtk.TargetEntry.new(
             'CHAT_LIST_ITEM',
@@ -124,14 +139,18 @@ class ChatListRow(Gtk.ListBoxRow):
         self.update_name()
         self.update_account_identifier()
 
+        app.storage.drafts.connect('draft-update', self._on_draft_update)
+
         if (isinstance(self.contact, GroupchatContact) and
                 not self.contact.can_notify()):
             self._ui.unread_label.get_style_context().add_class(
                 'unread-counter-silent')
 
-        # Get last chat message from archive
-        line = app.storage.archive.get_last_conversation_line(account, jid)
+        self._display_last_conversation_line()
 
+    def _display_last_conversation_line(self) -> None:
+        line = app.storage.archive.get_last_conversation_line(
+            self.contact.account, self.contact.jid)
         if line is None:
             self.show_all()
             return
@@ -152,10 +171,11 @@ class ChatListRow(Gtk.ListBoxRow):
             if line.kind in (KindConstant.CHAT_MSG_SENT,
                              KindConstant.SINGLE_MSG_SENT):
                 self.set_nick(_('Me'))
-                me_nickname = app.nicks[account]
+                me_nickname = app.nicks[self.contact.account]
 
             if line.kind == KindConstant.GC_MSG:
-                our_nick = get_group_chat_nick(account, jid)
+                our_nick = get_group_chat_nick(
+                    self.contact.account, self.contact.jid)
                 if line.contact_name == our_nick:
                     self.set_nick(_('Me'))
                     me_nickname = our_nick
@@ -224,6 +244,13 @@ class ChatListRow(Gtk.ListBoxRow):
         else:
             self.set_header(self._conversations_header)
 
+    def clear(self) -> None:
+        self.timestamp = 0
+        self.stanza_id = None
+        self.message_id = None
+        self.set_message_text('')
+        self.set_nick('')
+
     def set_message_id(self, message_id: str) -> None:
         self.message_id = message_id
 
@@ -233,14 +260,29 @@ class ChatListRow(Gtk.ListBoxRow):
                          icon_name: Optional[str] = None,
                          additional_data: Optional[AdditionalDataDict] = None
                          ) -> None:
+
+        draft = app.storage.drafts.get(self.contact)
+        if draft is not None:
+            self._show_draft(draft)
+            return
+
+        self._ui.message_label.get_style_context().remove_class('draft')
+
         icon = None
         if icon_name is not None:
             icon = Gio.Icon.new_for_string(icon_name)
         if additional_data is not None:
             if app.preview_manager.is_previewable(text, additional_data):
-                file_name = filename_from_uri(text)
-                icon, file_type = guess_simple_file_type(text)
-                text = f'{file_type} ({file_name})'
+                scheme = urlparse(text).scheme
+                if scheme == 'geo':
+                    location = split_geo_uri(text)
+                    icon = Gio.Icon.new_for_string('mark-location')
+                    text = format_geo_coords(
+                        float(location.lat), float(location.lon))
+                else:
+                    file_name = filename_from_uri(text)
+                    icon, file_type = guess_simple_file_type(text)
+                    text = f'{file_type} ({file_name})'
 
         text = GLib.markup_escape_text(text)
         if text.startswith('/me') and nickname is not None:
@@ -301,7 +343,7 @@ class ChatListRow(Gtk.ListBoxRow):
         if self.timestamp == 0:
             return
         self._ui.timestamp_label.set_text(
-            get_uf_relative_time(self.timestamp))
+            get_uf_relative_time(datetime.fromtimestamp(self.timestamp)))
 
     def add_unread(self, text: str) -> None:
         self._unread_count += 1
@@ -352,6 +394,31 @@ class ChatListRow(Gtk.ListBoxRow):
             return str(count)
         return '999+'
 
+    def _on_mute_setting_changed(self, *args: Any) -> None:
+        self._ui.mute_image.set_visible(self.contact.is_muted)
+
+    def _on_draft_update(self,
+                         _draft_storage: DraftStorage,
+                         _signal_name: str,
+                         contact: ChatContactT,
+                         draft: str
+                         ) -> None:
+
+        if contact != self.contact:
+            return
+
+        self._show_draft(draft)
+
+    def _show_draft(self, draft: str) -> None:
+        if not draft:
+            self._ui.message_label.get_style_context().remove_class('draft')
+            self._display_last_conversation_line()
+            return
+
+        self.set_nick('')
+        self._ui.message_label.set_text(_('Draft: %s') % draft)
+        self._ui.message_label.get_style_context().add_class('draft')
+
     def _on_state_flags_changed(self,
                                 _row: ChatListRow,
                                 _flags: Gtk.StateFlags
@@ -363,6 +430,7 @@ class ChatListRow(Gtk.ListBoxRow):
             self._ui.revealer.set_reveal_child(False)
 
     def _on_destroy(self, _row: ChatListRow) -> None:
+        app.settings.disconnect_signals(self)
         self.contact.disconnect_all_from_obj(self)
         if isinstance(self.contact, GroupchatParticipant):
             self.contact.room.disconnect_all_from_obj(self)
@@ -380,14 +448,14 @@ class ChatListRow(Gtk.ListBoxRow):
                                    ) -> None:
 
         if event.button == Gdk.BUTTON_SECONDARY:
-            self._popup_menu(event)
+            self._raise_context_popover(event)
 
         elif event.button == Gdk.BUTTON_MIDDLE:
             app.window.activate_action(
                 'remove-chat',
                 GLib.Variant('as', [self.account, str(self.jid)]))
 
-    def _popup_menu(self, event: Gdk.EventButton):
+    def _raise_context_popover(self, event: Gdk.EventButton):
         menu = get_chat_list_row_menu(
             self.workspace_id, self.account, self.jid, self._pinned)
 
@@ -403,8 +471,13 @@ class ChatListRow(Gtk.ListBoxRow):
 
         popover = GajimPopover(menu, relative_to=self)
         popover.set_pointing_to_coord(x=x, y=event.y)
+        popover.connect('closed', self._on_context_popover_closed)
+        self.emit('context-menu-state-changed', True)
         popover.popup()
 
+    def _on_context_popover_closed(self, _popover: Gtk.Popover) -> None:
+        self.emit('context-menu-state-changed', False)
+
     def _on_drag_begin(self,
                        row: ChatListRow,
                        drag_context: Gdk.DragContext
@@ -428,15 +501,16 @@ class ChatListRow(Gtk.ListBoxRow):
         app.window.highlight_dnd_targets(row, False)
 
     def _on_drag_data_get(self,
-                          _widget: Gtk.Widget,
+                          row: ChatListRow,
                           _drag_context: Gdk.DragContext,
                           selection_data: Gtk.SelectionData,
                           _info: int,
                           _time: int
                           ) -> None:
 
+        app.window.highlight_dnd_targets(row, False)
         drop_type = Gdk.Atom.intern_static_string('CHAT_LIST_ITEM')
-        byte_data = pickle.dumps((self.account, self.jid))
+        byte_data = pickle.dumps((self.account, self.jid, self.workspace_id))
         selection_data.set(drop_type, 8, byte_data)
 
     def _connect_contact_signals(self) -> None:
@@ -449,6 +523,13 @@ class ChatListRow(Gtk.ListBoxRow):
 
         elif isinstance(self.contact, GroupchatContact):
             self.contact.connect('avatar-update', self._on_avatar_update)
+            self.contact.connect('state-changed', self._on_muc_state_changed)
+            self.contact.connect('mam-sync-started', self._on_mam_sync_changed)
+            self.contact.connect('mam-sync-finished', self._on_mam_sync_changed)
+            self.contact.connect('mam-sync-error', self._on_mam_sync_changed)
+
+            self._client.connect_signal('state-changed',
+                                        self._on_client_state_changed)
 
         elif isinstance(self.contact, GroupchatParticipant):
             self.contact.connect('chatstate-update', self._on_chatstate_update)
@@ -477,6 +558,63 @@ class ChatListRow(Gtk.ListBoxRow):
                           ) -> None:
         self.update_avatar()
 
+    def _on_client_state_changed(self, *args: Any) -> None:
+        self._update_joined_state()
+
+    def _on_muc_state_changed(self, *args: Any) -> None:
+        self._update_joined_state()
+
+    def _update_joined_state(self) -> None:
+        self._reset_connection_icon()
+        context = self._ui.connection_icon.get_style_context()
+
+        if self.contact.is_joining:
+            self._ui.connection_icon.set_from_icon_name(
+                'feather-refresh-cw-symbolic', Gtk.IconSize.MENU)
+            context.add_class('spin')
+            context.add_class('dim-label')
+            self._ui.connection_icon.set_tooltip_text(_('Joining Group Chat…'))
+            self._ui.connection_icon.show()
+        elif (self.contact.is_not_joined or
+                not self._client.state.is_available):
+            self._ui.connection_icon.set_from_icon_name(
+                'feather-zap-symbolic', Gtk.IconSize.MENU)
+            context.add_class('warning-color')
+            self._ui.connection_icon.set_tooltip_text(_('Not connected'))
+            self._ui.connection_icon.show()
+
+    def _on_mam_sync_changed(self,
+                             _contact: GroupchatContact,
+                             signal_name: str
+                             ) -> None:
+
+        self._reset_connection_icon()
+        context = self._ui.connection_icon.get_style_context()
+
+        if signal_name == 'mam-sync-started':
+            self._ui.connection_icon.set_from_icon_name(
+                'feather-refresh-cw-symbolic', Gtk.IconSize.MENU)
+            context.add_class('spin')
+            context.add_class('info-color')
+            self._ui.connection_icon.set_tooltip_text(_('Fetching messages…'))
+            self._ui.connection_icon.show()
+        elif signal_name == 'mam-sync-error':
+            self._ui.connection_icon.set_from_icon_name(
+                'feather-zap-symbolic', Gtk.IconSize.MENU)
+            context.add_class('error-color')
+            self._ui.connection_icon.set_tooltip_text(
+                _('There has been an error while trying to fetch messages.'))
+            self._ui.connection_icon.show()
+
+    def _reset_connection_icon(self) -> None:
+        self._ui.connection_icon.hide()
+        context = self._ui.connection_icon.get_style_context()
+        context.remove_class('spin')
+        context.remove_class('dim-label')
+        context.remove_class('info-color')
+        context.remove_class('warning-color')
+        context.remove_class('error-color')
+
     def _on_muc_user_update(self,
                             _contact: GroupchatParticipant,
                             _signal_name: str,
