@@ -14,14 +14,13 @@
 
 from __future__ import annotations
 
-from typing import Optional
 from typing import cast
+from typing import Optional
 
-from gi.repository import Gtk
-from gi.repository import GObject
 from gi.repository import Gio
 from gi.repository import GLib
-
+from gi.repository import GObject
+from gi.repository import Gtk
 from nbxmpp import JID
 
 from gajim.common import app
@@ -30,12 +29,12 @@ from gajim.common import ged
 from gajim.common.ged import EventHelper
 from gajim.common.i18n import _
 
-from . import structs
-from .chat_filter import ChatFilter
-from .chat_list import ChatList
-from .chat_list import ChatListRow
-from .dialogs import DialogButton
-from .dialogs import ConfirmationCheckDialog
+from gajim.gtk import structs
+from gajim.gtk.chat_filter import ChatFilter
+from gajim.gtk.chat_list import ChatList
+from gajim.gtk.chat_list import ChatListRow
+from gajim.gtk.dialogs import ConfirmationCheckDialog
+from gajim.gtk.dialogs import DialogButton
 
 
 class ChatListStack(Gtk.Stack, EventHelper):
@@ -99,8 +98,7 @@ class ChatListStack(Gtk.Stack, EventHelper):
 
         for action in actions:
             action_name, variant, func = action
-            if variant is not None:
-                variant = GLib.VariantType.new(variant)
+            variant = GLib.VariantType.new(variant)
             act = Gio.SimpleAction.new(action_name, variant)
             act.connect('activate', func)
             app.window.add_action(act)
@@ -185,9 +183,6 @@ class ChatListStack(Gtk.Stack, EventHelper):
 
     def show_chat_list(self, workspace_id: str) -> None:
         cur_workspace_id = self.get_visible_child_name()
-        if cur_workspace_id == workspace_id:
-            return
-
         if cur_workspace_id != 'default' and cur_workspace_id is not None:
             self._chat_lists[cur_workspace_id].unselect_all()
 
@@ -283,7 +278,8 @@ class ChatListStack(Gtk.Stack, EventHelper):
         client = app.get_client(account)
         contact = client.get_module('Contacts').get_contact(jid, groupchat=True)
 
-        if contact.is_not_joined:
+        if contact.is_not_joined and client.state.is_available:
+            # For example a chat opened from the search bar
             _remove()
             return
 
@@ -313,10 +309,8 @@ class ChatListStack(Gtk.Stack, EventHelper):
     def contains_chat(self, account: str, jid: JID,
                       workspace_id: Optional[str] = None) -> bool:
         if workspace_id is None:
-            for chat_list in self._chat_lists.values():
-                if chat_list.contains_chat(account, jid):
-                    return True
-            return False
+            return any(chat_list.contains_chat(account, jid) for
+                       chat_list in self._chat_lists.values())
 
         chat_list = self._chat_lists[workspace_id]
         return chat_list.contains_chat(account, jid)
