@@ -17,44 +17,46 @@ from __future__ import annotations
 from typing import Optional
 from typing import Union
 
-import sys
 import logging
+import sys
 import time
+from urllib.parse import urlparse
 
 from gi.repository import Gdk
-from gi.repository import GLib
 from gi.repository import Gio
+from gi.repository import GLib
 from gi.repository import Gtk
-
-from nbxmpp.errors import StanzaError
 from nbxmpp.protocol import JID
-from nbxmpp.structs import MessageProperties
 
 from gajim.common import app
 from gajim.common import events
 from gajim.common import helpers
 from gajim.common import preview_helpers
 from gajim.common.commands import ChatCommands
-from gajim.common.i18n import _
 from gajim.common.const import CallType
+from gajim.common.i18n import _
 from gajim.common.modules.contacts import BareContact
 from gajim.common.modules.contacts import GroupchatContact
 from gajim.common.modules.contacts import GroupchatParticipant
+from gajim.common.preview_helpers import format_geo_coords
+from gajim.common.preview_helpers import split_geo_uri
 from gajim.common.structs import OutgoingMessage
 from gajim.common.types import ChatContactT
-from gajim.gui.dialogs import ErrorDialog
+from gajim.common.util.text import remove_invalid_xml_chars
 
-from .chat_banner import ChatBanner
-from .chat_function_page import ChatFunctionPage
-from .const import TARGET_TYPE_URI_LIST
-from .control import ChatControl
-from .message_actions_box import MessageActionsBox
-from .message_input import MessageInputTextView
-from .util import EventHelper
-from .util import open_window
-from .util import set_urgency_hint
+from gajim.gtk.chat_banner import ChatBanner
+from gajim.gtk.chat_function_page import ChatFunctionPage
+from gajim.gtk.chat_function_page import FunctionMode
+from gajim.gtk.const import TARGET_TYPE_URI_LIST
+from gajim.gtk.control import ChatControl
+from gajim.gtk.dialogs import ErrorDialog
+from gajim.gtk.message_actions_box import MessageActionsBox
+from gajim.gtk.message_input import MessageInputTextView
+from gajim.gtk.util import EventHelper
+from gajim.gtk.util import open_window
+from gajim.gtk.util import set_urgency_hint
 
-log = logging.getLogger('gajim.gui.chatstack')
+log = logging.getLogger('gajim.gtk.chatstack')
 
 
 class ChatStack(Gtk.Stack, EventHelper):
@@ -110,7 +112,7 @@ class ChatStack(Gtk.Stack, EventHelper):
         self._drop_area.set_no_show_all(True)
         self._drop_area.set_hexpand(True)
         self._drop_area.set_vexpand(True)
-        self._drop_area.set_name('DropArea')
+        self._drop_area.get_style_context().add_class('solid-background')
         self._drop_area.add(dnd_icon)
         self._drop_area.add(dnd_label)
 
@@ -121,8 +123,13 @@ class ChatStack(Gtk.Stack, EventHelper):
         overlay.connect('drag-motion', self._on_drag_motion)
         overlay.connect('drag-leave', self._on_drag_leave)
 
+        if app.is_flatpak():
+            target = 'application/vnd.portal.files'
+        else:
+            target = 'text/uri-list'
+
         uri_entry = Gtk.TargetEntry.new(
-            'text/uri-list',
+            target,
             Gtk.TargetFlags.OTHER_APP,
             TARGET_TYPE_URI_LIST)
         dnd_list = [uri_entry,
@@ -187,6 +194,8 @@ class ChatStack(Gtk.Stack, EventHelper):
         client = app.get_client(account)
         self._current_contact = client.get_module('Contacts').get_contact(jid)
 
+        app.preview_manager.clear_previews()
+
         self._chat_banner.switch_contact(self._current_contact)
         self._chat_control.switch_contact(self._current_contact)
         self._message_action_box.switch_contact(self._current_contact)
@@ -219,21 +228,25 @@ class ChatStack(Gtk.Stack, EventHelper):
                 str(self._current_contact.jid))
             if muc_data is not None:
                 if muc_data.state.is_captcha_request:
-                    self._show_chat_function_page('captcha-request')
+                    self._show_chat_function_page(FunctionMode.CAPTCHA_REQUEST)
                     return
 
                 if muc_data.state.is_password_request:
-                    self._show_chat_function_page('password-request')
+                    self._show_chat_function_page(FunctionMode.PASSWORD_REQUEST)
                     return
 
                 if not muc_data.state.is_joined:
                     if muc_data.error == 'captcha-failed':
                         self._show_chat_function_page(
-                            'captcha-error', muc_data.error_text)
+                            FunctionMode.CAPTCHA_ERROR, muc_data.error_text)
                         return
-                    if muc_data.error in ('join-failed', 'creation-failed'):
+                    if muc_data.error == 'join-failed':
                         self._show_chat_function_page(
-                            muc_data.error, muc_data.error_text)
+                            FunctionMode.JOIN_FAILED, muc_data.error_text)
+                        return
+                    if muc_data.error == 'creation-failed':
+                        self._show_chat_function_page(
+                            FunctionMode.CREATION_FAILED, muc_data.error_text)
                         return
 
         self.set_transition_type(Gtk.StackTransitionType.NONE)
@@ -243,7 +256,7 @@ class ChatStack(Gtk.Stack, EventHelper):
             'switch_contact', self._current_contact)
 
         if old_primary_clipboard is not None:
-            GLib.idle_add(clipboard.set_text,  # pyright: ignore
+            GLib.idle_add(clipboard.set_text,
                           old_primary_clipboard,
                           -1)
 
@@ -251,50 +264,49 @@ class ChatStack(Gtk.Stack, EventHelper):
 
     def _on_room_password_required(self,
                                    _contact: GroupchatContact,
-                                   _signal_name: str,
-                                   _properties: MessageProperties
+                                   _signal_name: str
                                    ) -> None:
-        self._show_chat_function_page('password-request')
+
+        self._show_chat_function_page(FunctionMode.PASSWORD_REQUEST)
 
     def _on_room_captcha_challenge(self,
                                    contact: GroupchatContact,
-                                   _signal_name: str,
-                                   properties: MessageProperties
+                                   _signal_name: str
                                    ) -> None:
-        self._show_chat_function_page('captcha-request')
+
+        self._show_chat_function_page(FunctionMode.CAPTCHA_REQUEST)
 
     def _on_room_captcha_error(self,
                                _contact: GroupchatContact,
                                _signal_name: str,
-                               error: StanzaError
+                               error: str
                                ) -> None:
-        error_text = helpers.to_user_string(error)
-        self._show_chat_function_page('captcha-error', error_text)
+
+        self._show_chat_function_page(FunctionMode.CAPTCHA_ERROR, error)
 
     def _on_room_creation_failed(self,
                                  _contact: GroupchatContact,
                                  _signal_name: str,
-                                 properties: MessageProperties
+                                 error: str
                                  ) -> None:
-        assert properties.error is not None
-        error_text = helpers.to_user_string(properties.error)
-        self._show_chat_function_page('creation-failed', error_text)
+
+        self._show_chat_function_page(FunctionMode.CREATION_FAILED, error)
 
     def _on_room_join_failed(self,
                              _contact: GroupchatContact,
                              _signal_name: str,
-                             error: StanzaError
+                             error: str
                              ) -> None:
-        self._show_chat_function_page(
-            'join-failed', helpers.to_user_string(error))
+
+        self._show_chat_function_page(FunctionMode.JOIN_FAILED, error)
 
     def _on_room_config_failed(self,
                                _contact: GroupchatContact,
                                _signal_name: str,
-                               error: StanzaError
+                               error: str
                                ) -> None:
-        self._show_chat_function_page(
-            'config-failed', helpers.to_user_string(error))
+
+        self._show_chat_function_page(FunctionMode.CONFIG_FAILED)
 
     def _on_muc_state_changed(self,
                               contact: GroupchatContact,
@@ -361,6 +373,16 @@ class ChatStack(Gtk.Stack, EventHelper):
         if not event.msgtxt or event.properties.is_sent_carbon:
             return
 
+        client = app.get_client(event.account)
+        contact = client.get_module('Contacts').get_contact(event.jid)
+        if isinstance(contact, GroupchatContact):
+            # MUC messages may be received after some delay, so make sure we
+            # don't issue notifications for our own messages.
+            self_contact = contact.get_self()
+            if (self_contact is not None and
+                    self_contact.name == event.properties.muc_nickname):
+                return
+
         if app.window.is_chat_active(event.account, event.jid):
             return
 
@@ -378,8 +400,11 @@ class ChatStack(Gtk.Stack, EventHelper):
         is_previewable = app.preview_manager.is_previewable(
             text, additional_data)
         if is_previewable:
-            if text.startswith('geo:'):
-                text = _('Location')
+            scheme = urlparse(text).scheme
+            if scheme == 'geo':
+                location = split_geo_uri(text)
+                text = format_geo_coords(
+                    float(location.lat), float(location.lon))
             else:
                 file_name = preview_helpers.filename_from_uri(text)
                 _icon, file_type = preview_helpers.guess_simple_file_type(text)
@@ -423,8 +448,10 @@ class ChatStack(Gtk.Stack, EventHelper):
             if text.startswith('/me') or text.startswith('/me\n'):
                 name = contact.name
                 if isinstance(contact, GroupchatContact):
-                    name = contact.nickname
+                    name = event.properties.muc_nickname
                 text = f'* {name} {text[3:]}'
+        else:
+            text = ''
 
         app.ged.raise_event(
             events.Notification(account=contact.account,
@@ -438,7 +465,6 @@ class ChatStack(Gtk.Stack, EventHelper):
     def _connect_actions(self) -> None:
         actions = [
             'add-to-roster',
-            'invite-contacts',
             'send-file',
             'send-file-httpupload',
             'send-file-jingle',
@@ -481,14 +507,8 @@ class ChatStack(Gtk.Stack, EventHelper):
             jingle.get_enabled() or
             httpupload.get_enabled())
 
-        app.window.get_action('show-contact-info').set_enabled(online)
         app.window.get_action('correct-message').set_enabled(online)
 
-        app.window.get_action('input-bold').set_enabled(True)
-        app.window.get_action('input-italic').set_enabled(True)
-        app.window.get_action('input-strike').set_enabled(True)
-        app.window.get_action('input-clear').set_enabled(True)
-
     def _update_chat_actions(self, contact: BareContact) -> None:
         account = contact.account
         online = app.account_is_connected(account)
@@ -526,6 +546,7 @@ class ChatStack(Gtk.Stack, EventHelper):
 
         app.window.get_action('quote').set_enabled(joined)
         app.window.get_action('mention').set_enabled(joined)
+        app.window.get_action('retract-message').set_enabled(joined)
 
     def _update_participant_actions(self,
                                     contact: GroupchatParticipant) -> None:
@@ -552,19 +573,19 @@ class ChatStack(Gtk.Stack, EventHelper):
             app.call_manager.start_call(account, jid, CallType.VIDEO)
 
         elif action_name.startswith('send-file'):
+            method = None
             name = action.get_name()
             if 'httpupload' in name:
-                app.interface.start_file_transfer(contact, method='httpupload')
-                return
+                method = 'httpupload'
 
             if 'jingle' in name:
-                app.interface.start_file_transfer(contact, method='jingle')
-                return
+                method = 'jingle'
 
-            app.interface.start_file_transfer(contact)
-
-        elif action_name == 'invite-contacts':
-            open_window('AdhocMUC', account=account, contact=contact)
+            uris = None
+            if param is not None:
+                uris = param.get_strv() or None
+            self._show_chat_function_page(
+                FunctionMode.SEND_FILE, method, uris)
 
         elif action_name == 'add-to-roster':
             if (isinstance(contact, GroupchatParticipant) and
@@ -587,10 +608,10 @@ class ChatStack(Gtk.Stack, EventHelper):
                 'ContactInfo', account=account, contact=resource_contact)
 
         elif action_name == 'muc-invite':
-            self._show_chat_function_page('invite')
+            self._show_chat_function_page(FunctionMode.INVITE)
 
         elif action_name == 'muc-change-nickname':
-            self._show_chat_function_page('change-nickname')
+            self._show_chat_function_page(FunctionMode.CHANGE_NICKNAME)
 
         elif action_name == 'muc-execute-command':
             nick = None
@@ -605,12 +626,12 @@ class ChatStack(Gtk.Stack, EventHelper):
         elif action_name == 'muc-kick':
             assert param is not None
             kick_nick = param.get_string()
-            self._show_chat_function_page('kick', data=kick_nick)
+            self._show_chat_function_page(FunctionMode.KICK, data=kick_nick)
 
         elif action_name == 'muc-ban':
             assert param is not None
             ban_jid = param.get_string()
-            self._show_chat_function_page('ban', data=ban_jid)
+            self._show_chat_function_page(FunctionMode.BAN, data=ban_jid)
 
         elif action_name == 'muc-change-role':
             assert param is not None
@@ -656,22 +677,33 @@ class ChatStack(Gtk.Stack, EventHelper):
 
     def _on_drag_motion(self,
                         _widget: Gtk.Widget,
-                        _context: Gdk.DragContext,
+                        context: Gdk.DragContext,
                         _x_coord: int,
                         _y_coord: int,
-                        _time: int
+                        time: int
                         ) -> bool:
-        self._drop_area.set_no_show_all(False)
-        self._drop_area.show_all()
-        return True
+
+        if time == 0:
+            # Workaround for https://gitlab.gnome.org/GNOME/gtk/-/issues/5518
+            return False
+
+        for targ in context.list_targets():
+            if targ.name() in {'text/uri-list', 'application/vnd.portal.files'}:
+                self._drop_area.set_no_show_all(False)
+                self._drop_area.show_all()
+                return True
+
+        return False
 
     def _show_chat_function_page(self,
-                                 function: str,
-                                 data: Optional[str] = None
+                                 function_mode: FunctionMode,
+                                 data: Optional[str] = None,
+                                 files: Optional[list[str]] = None
                                  ) -> None:
+
         assert self._current_contact is not None
         self._chat_function_page.set_mode(
-            self._current_contact, function, data)
+            self._current_contact, function_mode, data, files)
         self.set_transition_type(Gtk.StackTransitionType.SLIDE_UP_DOWN)
         self.set_visible_child_name('function')
 
@@ -696,7 +728,6 @@ class ChatStack(Gtk.Stack, EventHelper):
         self._chat_control.add_info_message(message)
 
     def _on_send_message(self) -> None:
-        self._message_action_box.msg_textview.replace_emojis()
         message = self._message_action_box.msg_textview.get_text()
         if message.startswith('//'):
             # Escape sequence for chat commands
@@ -705,8 +736,14 @@ class ChatStack(Gtk.Stack, EventHelper):
         contact = self._current_contact
         assert contact is not None
 
+        client = app.get_client(contact.account)
+
         encryption = contact.settings.get('encryption')
-        if encryption:
+        if encryption == 'OMEMO':
+            if not client.get_module('OMEMO').check_send_preconditions(contact):
+                return
+
+        elif encryption:
             if encryption not in app.plugin_manager.encryption_plugins:
                 ErrorDialog(_('Encryption error'),
                             _('Missing necessary encryption plugin'))
@@ -719,32 +756,17 @@ class ChatStack(Gtk.Stack, EventHelper):
             if not self._chat_control.sendmessage:
                 return
 
-        client = app.get_client(contact.account)
-
-        message = helpers.remove_invalid_xml_chars(message)
-        if message in ('', None, '\n'):
+        message = remove_invalid_xml_chars(message)
+        if message in ('', '\n'):
             return
 
         label = self._message_action_box.get_seclabel()
 
         correct_id = None
         if self._message_action_box.is_correcting:
-            self._message_action_box.is_correcting = False
-            context = self._message_action_box.msg_textview.get_style_context()
-            context.remove_class('gajim-msg-correcting')
-
-            correct_id = self._message_action_box.last_message_id.get(
-                (contact.account, contact.jid))
-            message_row = app.storage.archive.get_last_correctable_message(
-                contact.account, contact.jid, correct_id)
-            if message_row is None:
-                self._message_action_box.msg_textview.clear()
-                log.info('Trying to correct message older than threshold')
-                return
-
-            if message_row.message == message:
-                self._message_action_box.msg_textview.clear()
-                log.info('Trying to correct message with original text')
+            correct_id = self._message_action_box.try_message_correction(
+                message)
+            if correct_id is None:
                 return
 
         chatstate = client.get_module('Chatstate').get_active_chatstate(
@@ -766,9 +788,10 @@ class ChatStack(Gtk.Stack, EventHelper):
         client.send_message(message_)
 
         self._message_action_box.msg_textview.clear()
+        app.storage.drafts.set(contact, '')
 
-    def get_last_message_id(self, account: str, jid: JID) -> Optional[str]:
-        return self._message_action_box.last_message_id.get((account, jid))
+    def get_last_message_id(self, contact: ChatContactT) -> Optional[str]:
+        return self._message_action_box.get_last_message_id(contact)
 
     def _close_control(self) -> None:
         assert self._current_contact is not None
