@@ -14,48 +14,50 @@
 
 from __future__ import annotations
 
+from typing import cast
 from typing import Optional
 from typing import Union
-from typing import cast
 
 import logging
 from enum import IntEnum
 
-
 from gi.repository import Gdk
 from gi.repository import GdkPixbuf
 from gi.repository import GObject
 from gi.repository import Gtk
-
-from nbxmpp.task import Task
 from nbxmpp.errors import StanzaError
-from nbxmpp.structs import AnnotationNote, SoftwareVersionResult
 from nbxmpp.modules.vcard4 import VCard
+from nbxmpp.structs import AnnotationNote
+from nbxmpp.structs import SoftwareVersionResult
+from nbxmpp.task import Task
 
 from gajim.common import app
 from gajim.common import ged
+from gajim.common import types
 from gajim.common.const import AvatarSize
+from gajim.common.const import SimpleClientState
 from gajim.common.events import SubscribedPresenceReceived
 from gajim.common.events import UnsubscribedPresenceReceived
+from gajim.common.ged import EventHelper
 from gajim.common.helpers import get_uf_affiliation
 from gajim.common.helpers import get_uf_role
 from gajim.common.helpers import get_uf_show
 from gajim.common.i18n import _
-from gajim.common.ged import EventHelper
 from gajim.common.modules.contacts import BareContact
-from gajim.common.modules.contacts import ResourceContact
 from gajim.common.modules.contacts import GroupchatParticipant
+from gajim.common.modules.contacts import ResourceContact
 
-from .contact_settings import ContactSettings
-from .dialogs import ConfirmationDialog
-from .dialogs import DialogButton
-from .sidebar_switcher import SideBarSwitcher
-from .builder import get_builder
-from .util import connect_destroy
-from .vcard_grid import VCardGrid
-from .structs import RemoveHistoryActionParams
+from gajim.gtk.builder import get_builder
+from gajim.gtk.contact_settings import ContactSettings
+from gajim.gtk.dialogs import ConfirmationDialog
+from gajim.gtk.dialogs import DialogButton
+from gajim.gtk.omemo_trust_manager import OMEMOTrustManager
+from gajim.gtk.sidebar_switcher import SideBarSwitcher
+from gajim.gtk.structs import RemoveHistoryActionParams
+from gajim.gtk.util import connect_destroy
+from gajim.gtk.vcard_grid import VCardGrid
 
-log = logging.getLogger('gajim.gui.contact_info')
+log = logging.getLogger('gajim.gtk.contact_info')
 
 
 ContactT = Union[BareContact, GroupchatParticipant]
@@ -87,6 +89,8 @@ class ContactInfo(Gtk.ApplicationWindow, EventHelper):
         self.account = account
         self.contact = contact
         self._client = app.get_client(account)
+        self._client.connect_signal(
+            'state-changed', self._on_client_state_changed)
 
         self._ui = get_builder('contact_info.ui')
 
@@ -109,6 +113,7 @@ class ContactInfo(Gtk.ApplicationWindow, EventHelper):
 
         if isinstance(self.contact, BareContact):
             self._fill_settings_page(self.contact)
+            self._fill_encryption_page(self.contact)
             self._fill_device_info(self.contact)
             self._fill_groups_page(self.contact)
             self._fill_note_page(self.contact)
@@ -120,7 +125,6 @@ class ContactInfo(Gtk.ApplicationWindow, EventHelper):
         self.connect('key-press-event', self._on_key_press)
         self.connect('destroy', self._on_destroy)
 
-        # pylint: disable=line-too-long
         connect_destroy(self._ui.tree_selection,
                         'changed', self._on_group_selection_changed)
         connect_destroy(self._ui.toggle_renderer,
@@ -134,7 +138,6 @@ class ContactInfo(Gtk.ApplicationWindow, EventHelper):
             ('unsubscribed-presence-received', ged.GUI1,
              self._on_unsubscribed_presence_received),
         ])
-        # pylint: enable=line-too-long
 
         self.add(self._ui.main_grid)
         self.show_all()
@@ -143,6 +146,24 @@ class ContactInfo(Gtk.ApplicationWindow, EventHelper):
         if event.keyval == Gdk.KEY_Escape:
             self.destroy()
 
+    def _on_client_state_changed(self,
+                                 _client: types.Client,
+                                 _signal_name: str,
+                                 state: SimpleClientState
+                                 ) -> None:
+
+        self._ui.edit_name_button.set_sensitive(state.is_connected)
+        self._ui.edit_name_button.set_tooltip_text(
+            _('Not connected') if not state.is_connected else _('Edit Name…'))
+        self._ui.subscription_listbox.set_sensitive(state.is_connected)
+
+        if state.is_connected:
+            self._ui.groups_page_stack.set_visible_child_name('groups')
+            self._ui.notes_page_stack.set_visible_child_name('notes')
+        else:
+            self._ui.groups_page_stack.set_visible_child_name('offline')
+            self._ui.notes_page_stack.set_visible_child_name('offline')
+
     def _on_stack_child_changed(self,
                                 _widget: Gtk.Stack,
                                 _pspec: GObject.ParamSpec) -> None:
@@ -154,7 +175,7 @@ class ContactInfo(Gtk.ApplicationWindow, EventHelper):
         for task in self._tasks:
             task.cancel()
 
-        if self.contact.is_in_roster:
+        if self.contact.is_in_roster and self._client.state.is_available:
             self._save_annotation()
 
         for device_grid in self._devices.values():
@@ -167,13 +188,17 @@ class ContactInfo(Gtk.ApplicationWindow, EventHelper):
     def _fill_information_page(self, contact: ContactT) -> None:
         self._vcard_grid = VCardGrid(self.account)
         self._ui.vcard_box.add(self._vcard_grid)
-        self._client.get_module('VCard4').request_vcard(
-            jid=self.contact.jid,
-            callback=self._on_vcard_received)
+        if self._client.state.is_available:
+            self._client.get_module('VCard4').request_vcard(
+                jid=self.contact.jid,
+                callback=self._on_vcard_received)
 
         jid = str(contact.get_address())
 
         self._ui.contact_name_label.set_text(contact.name)
+        self._ui.edit_name_button.set_sensitive(
+            self._client.state.is_available)
+
         self._ui.contact_jid_label.set_text(jid)
         self._ui.contact_jid_label.set_tooltip_text(jid)
 
@@ -185,6 +210,11 @@ class ContactInfo(Gtk.ApplicationWindow, EventHelper):
 
         self._switcher.set_row_visible('information', True)
 
+    def _fill_encryption_page(self, contact: ContactT) -> None:
+        self._ui.encryption_box.add(
+            OMEMOTrustManager(self.contact.account, self.contact))
+        self._switcher.set_row_visible('encryption-omemo', True)
+
     def _fill_note_page(self, contact: BareContact) -> None:
         if not contact.is_in_roster:
             return
@@ -206,6 +236,9 @@ class ContactInfo(Gtk.ApplicationWindow, EventHelper):
 
         self._ui.from_subscription_switch.set_state(contact.is_subscribed)
 
+        self._ui.subscription_listbox.set_sensitive(
+            self._client.state.is_available)
+
         if contact.subscription in ('to', 'both'):
             self._ui.to_subscription_stack.set_visible_child_name('checkmark')
 
@@ -227,7 +260,7 @@ class ContactInfo(Gtk.ApplicationWindow, EventHelper):
             params.to_variant())
 
     def _fill_groups_page(self, contact: BareContact) -> None:
-        if not contact.is_in_roster:
+        if not contact.is_in_roster or not self._client.state.is_available:
             return
 
         model = self._ui.groups_treeview.get_model()
@@ -270,9 +303,12 @@ class ContactInfo(Gtk.ApplicationWindow, EventHelper):
 
     def _on_vcard_received(self, task: Task) -> None:
         try:
-            vcard = cast(VCard, task.finish())
+            vcard = cast(VCard | None, task.finish())
         except StanzaError as err:
             log.info('Error loading VCard: %s', err)
+            vcard = None
+
+        if vcard is None:
             vcard = VCard()
 
         self._vcard_grid.set_vcard(vcard)
@@ -332,7 +368,11 @@ class ContactInfo(Gtk.ApplicationWindow, EventHelper):
         self._ui.name_entry.set_sensitive(active)
         if active:
             self._ui.name_entry.grab_focus()
+            self._ui.edit_name_button_image.set_from_icon_name(
+                'document-save-symbolic', Gtk.IconSize.BUTTON)
         else:
+            self._ui.edit_name_button_image.set_from_icon_name(
+                'document-edit-symbolic', Gtk.IconSize.BUTTON)
             name = self._ui.name_entry.get_text()
             if name == self.contact.name:
                 return
