@@ -15,47 +15,50 @@
 from __future__ import annotations
 
 from typing import Any
+from typing import cast
 from typing import Optional
 from typing import Union
-from typing import cast
 
-import os
 import logging
 import time
 
-from gi.repository import Gtk
+from gi.repository import Gio
 from gi.repository import GLib
-
+from gi.repository import Gtk
 from nbxmpp import JID
 from nbxmpp.const import StatusCode
-from nbxmpp.structs import MucSubject
 from nbxmpp.modules.security_labels import Displaymarking
+from nbxmpp.structs import MucSubject
 
 from gajim.common import app
 from gajim.common import events
 from gajim.common import ged
 from gajim.common import helpers
 from gajim.common import types
-from gajim.common.helpers import get_file_path_from_dnd_dropped_uri
-from gajim.common.helpers import to_user_string
-from gajim.common.i18n import _
+from gajim.common.const import KindConstant
+from gajim.common.const import XmppUriQuery
 from gajim.common.ged import EventHelper
 from gajim.common.helpers import AdditionalDataDict
 from gajim.common.helpers import get_retraction_text
-from gajim.common.const import KindConstant
+from gajim.common.i18n import _
 from gajim.common.modules.contacts import BareContact
-from gajim.common.modules.contacts import GroupchatParticipant
 from gajim.common.modules.contacts import GroupchatContact
+from gajim.common.modules.contacts import GroupchatParticipant
 from gajim.common.modules.httpupload import HTTPFileTransfer
 from gajim.common.storage.archive import ConversationRow
+
+from gajim.gtk.builder import get_builder
+from gajim.gtk.conversation.jump_to_end_button import JumpToEndButton
+from gajim.gtk.conversation.message_selection import MessageSelection
 from gajim.gtk.conversation.view import ConversationView
+from gajim.gtk.groupchat_roster import GroupchatRoster
+from gajim.gtk.groupchat_state import GroupchatState
 
-from gajim.gui.conversation.jump_to_end_button import JumpToEndButton
-from gajim.gui.builder import get_builder
-from gajim.gui.groupchat_roster import GroupchatRoster
-from gajim.gui.groupchat_state import GroupchatState
+HistoryRowT = Union[events.ApplicationEvent, ConversationRow]
 
-log = logging.getLogger('gajim.gui.control')
+REQUEST_LINES_COUNT = 20
+
+log = logging.getLogger('gajim.gtk.control')
 
 
 class ChatControl(EventHelper):
@@ -63,7 +66,6 @@ class ChatControl(EventHelper):
         EventHelper.__init__(self)
 
         self.handlers: dict[int, Any] = {}
-
         self._contact = None
         self._client = None
 
@@ -72,24 +74,30 @@ class ChatControl(EventHelper):
         self._scrolled_view = ConversationView()
         self._scrolled_view.connect('autoscroll-changed',
                                     self._on_autoscroll_changed)
-        self._scrolled_view.connect('request-history',
-                                    self._fetch_n_lines_history, 20)
+        self._scrolled_view.connect('request-history', self._request_history)
         self._ui.conv_view_overlay.add(self._scrolled_view)
 
         self._groupchat_state = GroupchatState()
         self._ui.conv_view_overlay.add_overlay(self._groupchat_state)
 
+        self._message_selection = MessageSelection()
+        self._message_selection.connect('copy', self._on_copy_selection)
+        self._message_selection.connect('cancel', self._on_cancel_selection)
+        self._ui.conv_view_overlay.add_overlay(self._message_selection)
+
         self._jump_to_end_button = JumpToEndButton()
         self._jump_to_end_button.connect('clicked', self._on_jump_to_end)
         self._ui.conv_view_overlay.add_overlay(self._jump_to_end_button)
 
         self._roster = GroupchatRoster()
-
-        self._ui.conv_view_box.add(self._roster)
+        self._ui.conv_view_paned.pack2(self._roster, False, False)
 
         # Used with encryption plugins
         self.sendmessage = False
 
+        app.window.get_action('activate-message-selection').connect(
+            'activate', self._on_activate_message_selection)
+
         self.widget = cast(Gtk.Box, self._ui.get_object('control_box'))
         self.widget.show_all()
 
@@ -153,9 +161,13 @@ class ChatControl(EventHelper):
 
         self._jump_to_end_button.switch_contact(contact)
         self._scrolled_view.switch_contact(contact)
+        self._request_history(None, True)
         self._groupchat_state.switch_contact(contact)
         self._roster.switch_contact(contact)
 
+        self._message_selection.set_no_show_all(True)
+        self._message_selection.hide()
+
         self._register_events()
 
         if isinstance(contact, GroupchatParticipant):
@@ -219,6 +231,9 @@ class ChatControl(EventHelper):
              ged.GUI2, self._on_jingle_request_received),
             ('file-request-received', ged.GUI2, self._on_file_request_event),
             ('file-request-sent', ged.GUI2, self._on_file_request_event),
+            ('http-upload-started', ged.GUI2, self._on_http_upload_started),
+            ('http-upload-error', ged.GUI2, self._on_http_upload_error),
+            ('encryption-check', ged.GUI2, self._on_encryption_info),
         ])
 
     def _is_event_processable(self, event: Any) -> bool:
@@ -273,6 +288,7 @@ class ChatControl(EventHelper):
                          tim=event.timestamp,
                          displaymarking=displaymarking,
                          message_id=message_id,
+                         msg_log_id=event.msg_log_id,
                          additional_data=event.additional_data)
 
     def _on_message_received(self, event: events.MessageReceived) -> None:
@@ -354,6 +370,7 @@ class ChatControl(EventHelper):
                                  displaymarking=event.displaymarking,
                                  message_id=event.properties.id,
                                  stanza_id=event.stanza_id,
+                                 msg_log_id=event.msg_log_id,
                                  additional_data=event.additional_data)
 
     def _on_message_updated(self, event: events.MessageUpdated) -> None:
@@ -424,6 +441,19 @@ class ChatControl(EventHelper):
 
         self.add_jingle_file_transfer(event=event)
 
+    def _on_http_upload_started(self, event: events.HTTPUploadStarted) -> None:
+        self.add_file_transfer(event.transfer)
+
+    def _on_http_upload_error(self, event: events.HTTPUploadError) -> None:
+        self.add_info_message(event.error_msg)
+
+    def _on_encryption_info(self, event: events.EncryptionInfo) -> None:
+        if not self._is_event_processable(event):
+            return
+
+        if self._allow_add_message():
+            self._scrolled_view.add_encryption_info(event)
+
     @property
     def is_chat(self) -> bool:
         return isinstance(self.contact, BareContact)
@@ -436,7 +466,7 @@ class ChatControl(EventHelper):
     def is_groupchat(self) -> bool:
         return isinstance(self.contact, GroupchatContact)
 
-    def mark_as_read(self, send_marker: bool = True) -> None:
+    def mark_as_read(self) -> None:
         self._jump_to_end_button.reset_unread_count()
 
     def _on_autoscroll_changed(self,
@@ -452,21 +482,28 @@ class ChatControl(EventHelper):
         if app.window.is_chat_active(self.contact.account, self.contact.jid):
             app.window.mark_as_read(self.contact.account, self.contact.jid)
 
+    def _on_activate_message_selection(self,
+                                       _action: Gio.SimpleAction,
+                                       param: GLib.Variant
+                                       ) -> None:
+
+        log_line_id = param.get_uint32()
+        self._scrolled_view.enable_row_selection(log_line_id)
+        self._message_selection.set_no_show_all(False)
+        self._message_selection.show_all()
+
+    def _on_copy_selection(self, _widget: MessageSelection) -> None:
+        self._scrolled_view.copy_selected_messages()
+
+    def _on_cancel_selection(self, _widget: MessageSelection) -> None:
+        self._scrolled_view.disable_row_selection()
+
     def _on_jump_to_end(self, _button: Gtk.Button) -> None:
         self.reset_view()
 
     def drag_data_file_transfer(self, selection: Gtk.SelectionData) -> None:
-        # we may have more than one file dropped
-        uri_splitted = selection.get_uris()
-        for uri in uri_splitted:
-            path = get_file_path_from_dnd_dropped_uri(uri)
-            if not os.path.isfile(path):  # is it a file?
-                self.add_info_message(
-                    _('The following file could not be accessed and was '
-                      'not uploaded: %s') % path)
-                continue
-
-            app.interface.start_file_transfer(self.contact, path)
+        app.window.activate_action('send-file',
+                                   GLib.Variant('as', selection.get_uris()))
 
     def get_our_nick(self) -> str:
         if isinstance(self.contact, GroupchatParticipant):
@@ -541,6 +578,9 @@ class ChatControl(EventHelper):
         else:
             self._jump_to_end_button.add_unread_count()
 
+    def remove_message(self, log_line_id: int) -> None:
+        self._scrolled_view.remove_message(log_line_id)
+
     def reset_view(self) -> None:
         self._scrolled_view.reset()
 
@@ -551,26 +591,20 @@ class ChatControl(EventHelper):
         row = self._scrolled_view.get_row_by_log_line_id(log_line_id)
         if row is None:
             # Clear view and reload conversation around timestamp
+            self._scrolled_view.reset()
             self._scrolled_view.block_signals(True)
-            self.reset_view()
             before, at_after = app.storage.archive.get_conversation_around(
                 self.contact.account, self.contact.jid, timestamp)
             self.add_messages(before)
             self.add_messages(at_after)
+            self._scrolled_view.set_history_complete(False, False)
 
+        GLib.idle_add(self._scrolled_view.block_signals, False)
         GLib.idle_add(
             self._scrolled_view.scroll_to_message_and_highlight,
             log_line_id)
-        GLib.idle_add(self._scrolled_view.block_signals, False)
-
-    def _fetch_n_lines_history(self,
-                               _scrolled: Gtk.ScrolledWindow,
-                               before: bool,
-                               n_lines: int
-                               ) -> None:
-
-        self._scrolled_view.block_signals(True)
 
+    def _request_messages(self, before: bool) -> list[ConversationRow]:
         if before:
             row = self._scrolled_view.get_first_message_row()
         else:
@@ -581,66 +615,107 @@ class ChatControl(EventHelper):
         else:
             timestamp = row.db_timestamp
 
-        assert self._contact is not None
-        for event in app.storage.events.load(self._contact, before, timestamp):
-            if isinstance(event, events.MUCUserJoined):
-                self._process_muc_user_joined(event)
-
-            elif isinstance(event, events.MUCUserLeft):
-                self._process_muc_user_left(event)
-
-            elif isinstance(event, events.MUCNicknameChanged):
-                self._process_muc_nickname_changed(event)
-
-            elif isinstance(event, events.MUCRoomKicked):
-                self._process_muc_room_kicked(event)
-
-            elif isinstance(event, events.MUCUserAffiliationChanged):
-                self._process_muc_user_affiliation_changed(event)
-
-            elif isinstance(event, events.MUCUserRoleChanged):
-                self._process_muc_user_role_changed(event)
-
-            elif isinstance(event, events.MUCUserStatusShowChanged):
-                self._process_muc_user_status_show_changed(event)
-
-            elif isinstance(event, events.MUCRoomConfigChanged):
-                self._process_muc_room_config_changed(event)
-
-            elif isinstance(event, events.MUCRoomConfigFinished):
-                self._process_muc_room_config_finished(event)
-
-            elif isinstance(event, events.MUCRoomPresenceError):
-                self._process_muc_room_presence_error(event)
-
-            elif isinstance(event, events.MUCRoomDestroyed):
-                self._process_muc_room_destroyed(event)
-            else:
-                raise ValueError('Unknown event: %s' % event)
-
-        messages = app.storage.archive.get_conversation_before_after(
+        return app.storage.archive.get_conversation_before_after(
             self.contact.account,
             self.contact.jid,
             before,
             timestamp,
-            n_lines)
+            REQUEST_LINES_COUNT)
 
-        if not messages:
-            self._scrolled_view.set_history_complete(before, True)
-            self._scrolled_view.block_signals(False)
-            return
+    def _request_events(self, before: bool) -> list[events.ApplicationEvent]:
+        if before:
+            row = self._scrolled_view.get_first_event_row()
+        else:
+            row = self._scrolled_view.get_last_event_row()
 
-        self.add_messages(messages)
+        if row is None:
+            timestamp = time.time()
+        else:
+            timestamp = row.db_timestamp
 
-        if len(messages) < n_lines:
+        assert self._contact is not None
+        return app.storage.events.load(self._contact,
+                                       before,
+                                       timestamp,
+                                       REQUEST_LINES_COUNT)
+
+    def _request_history(self,
+                         _widget: Any,
+                         before: bool
+                         ) -> None:
+
+        self._scrolled_view.block_signals(True)
+
+        messages = self._request_messages(before)
+        event_rows = self._request_events(before)
+        rows = self._sort_request_rows(messages, event_rows, before)
+
+        assert self._contact is not None
+        for row in rows:
+            if not isinstance(row, events.ApplicationEvent):
+                self.add_messages([row])
+
+            elif isinstance(row, events.MUCUserJoined):
+                self._process_muc_user_joined(row)
+
+            elif isinstance(row, events.MUCUserLeft):
+                self._process_muc_user_left(row)
+
+            elif isinstance(row, events.MUCNicknameChanged):
+                self._process_muc_nickname_changed(row)
+
+            elif isinstance(row, events.MUCRoomKicked):
+                self._process_muc_room_kicked(row)
+
+            elif isinstance(row, events.MUCUserAffiliationChanged):
+                self._process_muc_user_affiliation_changed(row)
+
+            elif isinstance(row, events.MUCUserRoleChanged):
+                self._process_muc_user_role_changed(row)
+
+            elif isinstance(row, events.MUCUserStatusShowChanged):
+                self._process_muc_user_status_show_changed(row)
+
+            elif isinstance(row, events.MUCRoomConfigChanged):
+                self._process_muc_room_config_changed(row)
+
+            elif isinstance(row, events.MUCRoomConfigFinished):
+                self._process_muc_room_config_finished(row)
+
+            elif isinstance(row, events.MUCRoomPresenceError):
+                self._process_muc_room_presence_error(row)
+
+            elif isinstance(row, events.MUCRoomDestroyed):
+                self._process_muc_room_destroyed(row)
+
+            else:
+                raise ValueError('Unknown event: %s' % type(row))
+
+        if len(rows) < REQUEST_LINES_COUNT:
             self._scrolled_view.set_history_complete(before, True)
 
         self._scrolled_view.block_signals(False)
 
+    @staticmethod
+    def _sort_request_rows(messages: list[ConversationRow],
+                           event_rows: list[events.ApplicationEvent],
+                           before: bool
+                           ) -> list[HistoryRowT]:
+
+        def sort_func(obj: HistoryRowT) -> float:
+            if isinstance(obj, events.ApplicationEvent):
+                return obj.timestamp  # pyright: ignore
+            return obj.time
+
+        rows = messages + event_rows
+        rows.sort(key=sort_func, reverse=before)
+        return rows
+
     def add_messages(self, messages: list[ConversationRow]):
         for msg in messages:
             if msg.kind in (KindConstant.FILE_TRANSFER_INCOMING,
                             KindConstant.FILE_TRANSFER_OUTGOING):
+                assert msg.additional_data is not None
                 if msg.additional_data.get_value('gajim', 'type') == 'jingle':
                     self._scrolled_view.add_jingle_file_transfer(
                         db_message=msg)
@@ -664,6 +739,10 @@ class ChatControl(EventHelper):
                 contact_name = self.contact.name
             elif msg.kind == KindConstant.GC_MSG:
                 kind = 'incoming'
+                if contact_name is None:
+                    # Fall back to MUC name if contact name is None
+                    # (may be the case for service messages from the MUC)
+                    contact_name = self.contact.name
             elif msg.kind in (
                     KindConstant.SINGLE_MSG_SENT, KindConstant.CHAT_MSG_SENT):
                 kind = 'outgoing'
@@ -672,6 +751,8 @@ class ChatControl(EventHelper):
                 log.warning('kind attribute could not be processed'
                             'while adding message')
 
+            assert contact_name is not None
+
             if msg.additional_data is not None:
                 retracted_by = msg.additional_data.get_value('retracted', 'by')
                 if retracted_by is not None:
@@ -998,8 +1079,7 @@ class ChatControl(EventHelper):
                                          event: events.MUCRoomPresenceError
                                          ) -> None:
 
-        error_message = to_user_string(event.error)
-        self.add_info_message(_('Error: %s') % error_message)
+        self.add_info_message(_('Error: %s') % event.error)
 
     def _on_room_destroyed(self,
                            _contact: GroupchatContact,
@@ -1016,11 +1096,11 @@ class ChatControl(EventHelper):
         reason = event.reason
         reason = '' if reason is None else f': {reason}'
 
-        message = _('Group chat has been destroyed')
+        message = _('Group chat has been destroyed%s') % reason
 
         if event.alternate is not None:
-            message += '\n' + _('You can join this group chat instead: '
-                                'xmpp:%s?join') % str(event.alternate)
+            message += '\n' + _('You can join this group chat instead: %s') % (
+                event.alternate.to_iri(XmppUriQuery.JOIN.value))
 
         self.add_info_message(message, event.timestamp)
 
@@ -1127,6 +1207,7 @@ class ChatControl(EventHelper):
                         displaymarking: Optional[Displaymarking] = None,
                         message_id: Optional[str] = None,
                         stanza_id: Optional[str] = None,
+                        msg_log_id: Optional[int] = None,
                         additional_data: Optional[AdditionalDataDict] = None,
                         ) -> None:
 
@@ -1145,6 +1226,7 @@ class ChatControl(EventHelper):
                           displaymarking=displaymarking,
                           message_id=message_id,
                           stanza_id=stanza_id,
+                          msg_log_id=msg_log_id,
                           additional_data=additional_data)
 
     def _on_room_subject(self,
