@@ -25,8 +25,9 @@ from nbxmpp.structs import MucSubject
 from gajim.common.const import AvatarSize
 from gajim.common.i18n import _
 
-from .base import BaseRow
-from ..message_widget import MessageWidget
+from gajim.gtk.conversation.message_widget import MessageWidget
+from gajim.gtk.conversation.rows.base import BaseRow
+from gajim.gtk.conversation.rows.widgets import DateTimeLabel
 
 
 class MUCSubject(BaseRow):
@@ -71,6 +72,7 @@ class MUCSubject(BaseRow):
         meta_str = f'{author}{date}'
         meta = Gtk.Label(label=meta_str)
         meta.set_halign(Gtk.Align.START)
+        meta.set_selectable(True)
         meta.get_style_context().add_class('small-label')
         subject_box.add(meta)
 
@@ -79,7 +81,7 @@ class MUCSubject(BaseRow):
         subject_box.add(message_widget)
         self.grid.attach(subject_box, 1, 0, 1, 1)
 
-        timestamp_widget = self.create_timestamp_widget(self.timestamp)
+        timestamp_widget = DateTimeLabel(self.timestamp)
         timestamp_widget.set_valign(Gtk.Align.START)
         self.grid.attach(timestamp_widget, 2, 0, 1, 1)
 
