@@ -15,55 +15,56 @@
 from __future__ import annotations
 
 from typing import Any
+from typing import cast
+from typing import Generator
 from typing import Literal
 from typing import Optional
 from typing import Union
-from typing import Generator
-from typing import cast
 
 import logging
 import time
-
 from datetime import datetime
 from datetime import timedelta
 
-from gi.repository import Gtk
-from gi.repository import GObject
+from gi.repository import Gdk
 from gi.repository import Gio
-
+from gi.repository import GObject
+from gi.repository import Gtk
 from nbxmpp.errors import StanzaError
 from nbxmpp.modules.security_labels import Displaymarking
+from nbxmpp.protocol import JID
 from nbxmpp.structs import CommonError
 from nbxmpp.structs import MucSubject
-from nbxmpp.protocol import JID
 
 from gajim.common import app
-from gajim.common import types
 from gajim.common import events
+from gajim.common import types
 from gajim.common.helpers import AdditionalDataDict
-from gajim.common.helpers import to_user_string
 from gajim.common.helpers import get_start_of_day
+from gajim.common.helpers import to_user_string
+from gajim.common.modules.contacts import BareContact
 from gajim.common.modules.contacts import GroupchatContact
 from gajim.common.modules.httpupload import HTTPFileTransfer
 from gajim.common.storage.archive import ConversationRow
-from gajim.common.modules.contacts import BareContact
 from gajim.common.types import ChatContactT
 
-from .rows.base import BaseRow
-from .rows.read_marker import ReadMarkerRow
-from .rows.scroll_hint import ScrollHintRow
-from .rows.message import MessageRow
-from .rows.info import InfoMessage
-from .rows.call import CallRow
-from .rows.command_output import CommandOutputRow
-from .rows.date import DateRow
-from .rows.file_transfer import FileTransferRow
-from .rows.file_transfer_jingle import FileTransferJingleRow
-from .rows.muc_subject import MUCSubject
-from .rows.muc_join_left import MUCJoinLeft
-from .rows.user_status import UserStatus
+from gajim.gtk.conversation.rows.base import BaseRow
+from gajim.gtk.conversation.rows.call import CallRow
+from gajim.gtk.conversation.rows.command_output import CommandOutputRow
+from gajim.gtk.conversation.rows.date import DateRow
+from gajim.gtk.conversation.rows.encryption_info import EncryptionInfoRow
+from gajim.gtk.conversation.rows.file_transfer import FileTransferRow
+from gajim.gtk.conversation.rows.file_transfer_jingle import \
+    FileTransferJingleRow
+from gajim.gtk.conversation.rows.info import InfoMessage
+from gajim.gtk.conversation.rows.message import MessageRow
+from gajim.gtk.conversation.rows.muc_join_left import MUCJoinLeft
+from gajim.gtk.conversation.rows.muc_subject import MUCSubject
+from gajim.gtk.conversation.rows.read_marker import ReadMarkerRow
+from gajim.gtk.conversation.rows.scroll_hint import ScrollHintRow
+from gajim.gtk.conversation.rows.user_status import UserStatus
 
-log = logging.getLogger('gajim.gui.conversation_view')
+log = logging.getLogger('gajim.gtk.conversation_view')
 
 
 class ConversationView(Gtk.ScrolledWindow):
@@ -141,6 +142,40 @@ class ConversationView(Gtk.ScrolledWindow):
         app.window.get_action('scroll-view-down').connect(
             'activate', self._on_scroll_view)
 
+    def copy_selected_messages(self) -> None:
+        format_string = app.settings.get('date_time_format')
+        selection_text = ''
+        for row in cast(list[BaseRow], self._list_box.get_selected_rows()):
+            if isinstance(row, MessageRow):
+                timestamp_formatted = row.timestamp.strftime(format_string)
+                selection_text += (f'{timestamp_formatted} - {row.name}: '
+                                   f'{row.get_text()}\n')
+
+        clip = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
+        clip.set_text(selection_text, -1)
+
+        self.disable_row_selection()
+
+    def enable_row_selection(self,
+                             log_line_id: Optional[int]
+                             ) -> None:
+
+        self._list_box.set_selection_mode(Gtk.SelectionMode.MULTIPLE)
+
+        if log_line_id is not None:
+            row = self.get_row_by_log_line_id(log_line_id)
+            self._list_box.select_row(row)
+
+        for row in self.iter_rows():
+            row.enable_selection_mode()
+
+    def disable_row_selection(self) -> None:
+        self._list_box.unselect_all()
+        self._list_box.set_selection_mode(Gtk.SelectionMode.NONE)
+
+        for row in self.iter_rows():
+            row.disable_selection_mode()
+
     def _on_scroll_view(self,
                         action: Gio.SimpleAction,
                         _param: Literal[None]) -> None:
@@ -187,6 +222,8 @@ class ConversationView(Gtk.ScrolledWindow):
         self._block_signals = True
         self._reset()
 
+        self.disable_row_selection()
+
         self._read_marker_row = ReadMarkerRow(self._contact)
         self._list_box.add(self._read_marker_row)
 
@@ -207,7 +244,6 @@ class ConversationView(Gtk.ScrolledWindow):
 
         self._block_signals = False
         self._enable_signal_handlers(True)
-        self._emit('request-history', True)
 
     def get_autoscroll(self) -> bool:
         return self._autoscroll
@@ -336,7 +372,9 @@ class ConversationView(Gtk.ScrolledWindow):
         return self._contact
 
     def _get_row_at_index(self, index: int) -> BaseRow:
-        return cast(BaseRow, self._list_box.get_row_at_index(index))
+        row = self._list_box.get_row_at_index(index)
+        assert row is not None
+        return cast(BaseRow, row)
 
     def get_first_message_row(self) -> Optional[MessageRow]:
         for row in self._list_box.get_children():
@@ -352,6 +390,20 @@ class ConversationView(Gtk.ScrolledWindow):
                 return row
         return None
 
+    def get_first_event_row(self) -> Optional[Union[InfoMessage, MUCJoinLeft]]:
+        for row in self._list_box.get_children():
+            if isinstance(row, (InfoMessage, MUCJoinLeft)):
+                return row
+        return None
+
+    def get_last_event_row(self) -> Optional[Union[InfoMessage, MUCJoinLeft]]:
+        children = self._list_box.get_children()
+        children.reverse()
+        for row in children:
+            if isinstance(row, (InfoMessage, MUCJoinLeft)):
+                return row
+        return None
+
     @staticmethod
     def _sort_func(row1: BaseRow, row2: BaseRow) -> int:
         if row1.timestamp == row2.timestamp:
@@ -424,6 +476,10 @@ class ConversationView(Gtk.ScrolledWindow):
             db_message=db_message)
         self._insert_message(jingle_transfer_row)
 
+    def add_encryption_info(self, event: events.EncryptionInfo) -> None:
+        assert self._contact is not None
+        self._insert_message(EncryptionInfoRow(event))
+
     def add_call_message(self,
                          event: Optional[events.JingleRequestReceived] = None,
                          db_message: Optional[ConversationRow] = None
@@ -505,7 +561,7 @@ class ConversationView(Gtk.ScrolledWindow):
         self._active_date_rows.add(start_of_day)
         self._list_box.add(date_row)
 
-        row = self._get_row_at_index(date_row.get_index() + 1)
+        row = self._list_box.get_row_at_index(date_row.get_index() + 1)
         if row is None:
             return
 
@@ -532,7 +588,7 @@ class ConversationView(Gtk.ScrolledWindow):
         index = message.get_index()
         while index != 0:
             index -= 1
-            row = self._get_row_at_index(index)
+            row = self._list_box.get_row_at_index(index)
             if row is None:
                 return None
 
@@ -553,7 +609,7 @@ class ConversationView(Gtk.ScrolledWindow):
         index = message.get_index()
         while True:
             index += 1
-            row = self._get_row_at_index(index)
+            row = self._list_box.get_row_at_index(index)
             if row is None:
                 return
 
@@ -563,13 +619,8 @@ class ConversationView(Gtk.ScrolledWindow):
             if not isinstance(row, MessageRow):
                 return
 
-            if message.is_mergeable(row):
-                row.set_merged(True)
-                continue
-
-            if message.is_same_sender(row):
-                row.set_merged(False)
-                self._update_descendants(row)
+            merge = message.is_mergeable(row)
+            row.set_merged(merge)
             return
 
     def reduce_message_count(self, before: bool) -> bool:
@@ -617,6 +668,20 @@ class ConversationView(Gtk.ScrolledWindow):
         row = self._get_row_at_index(len(self._list_box.get_children()) - 1)
         row.destroy()
 
+    def remove_message(self, log_line_id: int) -> None:
+        row = self.get_row_by_log_line_id(log_line_id)
+        if row is None:
+            return
+
+        index = row.get_index()
+        row.destroy()
+        decendant_row = self._list_box.get_row_at_index(index)
+        if isinstance(decendant_row, MessageRow):
+            # Unset possible merged state if we delete a 'top level' message.
+            # Checks for same sender etc. are not necessary, since we simply
+            # unset merged state.
+            decendant_row.set_merged(False)
+
     def scroll_to_message_and_highlight(self, log_line_id: int) -> None:
         highlight_row = None
         for row in cast(list[BaseRow], self._list_box.get_children()):
@@ -655,8 +720,7 @@ class ConversationView(Gtk.ScrolledWindow):
         return None
 
     def iter_rows(self) -> Generator[BaseRow, None, None]:
-        for row in cast(list[BaseRow], self._list_box.get_children()):
-            yield row
+        yield from cast(list[BaseRow], self._list_box.get_children())
 
     def remove_rows_by_type(self, row_type: str) -> None:
         for row in self.iter_rows():
@@ -669,9 +733,6 @@ class ConversationView(Gtk.ScrolledWindow):
                 row.update()
 
     def set_read_marker(self, id_: str) -> None:
-        if id_ is None:
-            return
-
         row = self._get_row_by_message_id(id_)
         if row is None:
             return
