@@ -22,30 +22,30 @@ import logging
 from enum import IntEnum
 
 from gi.repository import Gdk
-from gi.repository import Gtk
 from gi.repository import GLib
+from gi.repository import Gtk
 from nbxmpp.const import Affiliation
 
 from gajim.common import app
 from gajim.common import ged
 from gajim.common import types
-from gajim.common.helpers import get_uf_role
-from gajim.common.helpers import get_uf_affiliation
-from gajim.common.helpers import jid_is_blocked
 from gajim.common.const import AvatarSize
 from gajim.common.const import StyleAttr
 from gajim.common.events import ApplicationEvent
 from gajim.common.events import MUCNicknameChanged
+from gajim.common.helpers import get_uf_affiliation
+from gajim.common.helpers import get_uf_role
+from gajim.common.helpers import jid_is_blocked
+from gajim.common.i18n import p_
 from gajim.common.modules.contacts import GroupchatContact
 
-from .menus import get_groupchat_participant_menu
-from .tooltips import GCTooltip
-from .builder import get_builder
-from .util import EventHelper
-from .util import GajimPopover
+from gajim.gtk.builder import get_builder
+from gajim.gtk.menus import get_groupchat_participant_menu
+from gajim.gtk.tooltips import GCTooltip
+from gajim.gtk.util import EventHelper
+from gajim.gtk.util import GajimPopover
 
-
-log = logging.getLogger('gajim.gui.groupchat_roster')
+log = logging.getLogger('gajim.gtk.groupchat_roster')
 
 
 AffiliationRoleSortOrder = {
@@ -85,7 +85,7 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
         self._tooltip = GCTooltip()
 
         self._ui = get_builder('groupchat_roster.ui')
-        self.add(self._ui.scrolled)
+        self.add(self._ui.box)
 
         self._contact_refs: dict[str, Gtk.TreeRowReference] = {}
         self._group_refs: dict[str, Gtk.TreeRowReference] = {}
@@ -94,7 +94,10 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
         self._store.set_sort_func(Column.TEXT, self._tree_compare_iters)
 
         self._roster = self._ui.roster_treeview
-        self._roster.set_search_equal_func(self._search_func)
+
+        self._filter_string = ''
+        self._modelfilter = self._store.filter_new()
+        self._modelfilter.set_visible_func(self._visible_func)
         self._roster.set_has_tooltip(True)
 
         self._ui.contact_column.set_fixed_width(
@@ -114,6 +117,8 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
         self.set_reveal_child(
             not app.settings.get('hide_groupchat_occupants_list'))
 
+        self.connect('notify::reveal-child', self._on_reveal)
+
     def _hide_roster(self, hide_roster: bool, *args: Any) -> None:
         transition = Gtk.RevealerTransitionType.SLIDE_RIGHT
         if not hide_roster:
@@ -122,11 +127,19 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
 
         self.set_transition_type(transition)
         self.set_reveal_child(not hide_roster)
+        self.set_visible(not hide_roster)
+
+    def _on_reveal(self, revealer: Gtk.Revealer, param: Any) -> None:
+        if revealer.get_reveal_child():
+            self._load_roster()
+        else:
+            self._unload_roster()
 
     def clear(self) -> None:
-        log.info('Clear')
         if self._contact is None:
             return
+
+        log.info('Clear')
         self._unload_roster()
         self._contact.disconnect_signal(self, 'state-changed')
         self._contact = None
@@ -172,6 +185,11 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
             self._tooltip.clear_tooltip()
             return False
 
+        path = self._modelfilter.convert_path_to_child_path(path)
+        if path is None:
+            self._tooltip.clear_tooltip()
+            return False
+
         iter_ = None
         try:
             iter_ = self._store.get_iter(path)
@@ -192,14 +210,24 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
         tooltip.set_custom(widget)
         return value
 
-    @staticmethod
-    def _search_func(model: Gtk.TreeModel,
-                     _column: int,
-                     search_text: str,
-                     iter_: Gtk.TreeIter
-                     ) -> bool:
+    def _on_search_changed(self, widget: Gtk.SearchEntry) -> None:
+        self._filter_string = widget.get_text().lower()
+        self._modelfilter.refilter()
+        self._roster.expand_all()
 
-        return search_text.lower() not in model[iter_][1].lower()
+    def _visible_func(self,
+                      model: Gtk.TreeModelFilter,
+                      iter_: Gtk.TreeIter,
+                      *_data: Any
+                      ) -> bool:
+
+        if not self._filter_string:
+            return True
+
+        if not model[iter_][Column.IS_CONTACT]:
+            return True
+
+        return self._filter_string in model[iter_][Column.TEXT].lower()
 
     def _get_group_iter(self, group_name: str) -> Optional[Gtk.TreeIter]:
         try:
@@ -358,7 +386,10 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
                                  _column: Gtk.TreeViewColumn
                                  ) -> None:
 
-        iter_ = self._store.get_iter(path)
+        child_path = self._modelfilter.convert_path_to_child_path(path)
+        assert child_path is not None
+
+        iter_ = self._store.get_iter(child_path)
         if self._store.iter_parent(iter_) is None:
             # This is a group row
             return
@@ -379,8 +410,7 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
             app.window.add_chat(self._contact.account,
                                 participant.real_jid,
                                 'contact',
-                                select=True,
-                                workspace='current')
+                                select=True)
         else:
             contact = self._contact.get_resource(nick)
             app.window.add_private_chat(self._contact.account,
@@ -392,7 +422,7 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
                                       event: Gdk.EventButton
                                       ) -> None:
 
-        if event.button not in (2, 3):
+        if event.button == Gdk.BUTTON_PRIMARY:
             return
 
         pos = treeview.get_path_at_pos(int(event.x), int(event.y))
@@ -403,6 +433,10 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
         if path is None:
             return
 
+        path = self._modelfilter.convert_path_to_child_path(path)
+        if path is None:
+            return
+
         iter_ = self._store.get_iter(path)
         if self._store.iter_parent(iter_) is None:
             # Group row
@@ -414,10 +448,10 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
         if self._contact.nickname == nick:
             return
 
-        if event.button == 3:  # right click
+        if event.button == Gdk.BUTTON_SECONDARY:
             self._show_contact_menu(nick, event)
 
-        # if event.button == 2:  # middle click
+        # if event.button == Gdk.BUTTON_MIDDLE:
             # self.roster.emit('row-activated', nick)
 
     def _show_contact_menu(self, nick: str, event: Gdk.EventButton) -> None:
@@ -458,10 +492,15 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
             nick1 = model[iter1][Column.NICK_OR_GROUP]
             nick2 = model[iter2][Column.NICK_OR_GROUP]
 
+            assert self._contact is not None
+            our_nick = self._contact.nickname
+            if our_nick in (nick1, nick2):
+                # Always show our nickname at the top
+                return -1 if our_nick == nick1 else 1
+
             if not app.settings.get('sort_by_show_in_muc'):
                 return locale.strcoll(nick1.lower(), nick2.lower())
 
-            assert self._contact is not None
             contact1 = self._contact.get_resource(nick1)
             contact2 = self._contact.get_resource(nick2)
 
@@ -485,6 +524,9 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
         self._store.set_sort_column_id(column, Gtk.SortType.ASCENDING)
 
     def _load_roster(self) -> None:
+        if not self.get_reveal_child():
+            return
+
         log.info('Load Roster')
         assert self._contact is not None
         self._contact.multi_connect({
@@ -501,10 +543,15 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
             self._add_contact(participant)
 
         self.enable_sort(True)
-        self._roster.set_model(self._store)
+        self._roster.set_model(self._modelfilter)
+
+        self._ui.search_entry.set_text('')
         self._roster.expand_all()
 
     def _unload_roster(self) -> None:
+        if self._roster.get_model() is None:
+            return
+
         log.info('Unload Roster')
         assert self._contact is not None
         self._contact.multi_disconnect(self, CONTACT_SIGNALS)
@@ -536,6 +583,9 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
         self._draw_avatar(contact)
 
         name = GLib.markup_escape_text(contact.name)
+        self_contact = self._contact.get_self()
+        if self_contact is not None and self_contact.name == nick:
+            name = p_('own nickname in group chat', '%s (You)' % nick)
 
         # Strike name if blocked
         fjid = f'{self._contact.jid}/{nick}'
@@ -543,10 +593,7 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
             name = f'<span strikethrough="true">{name}</span>'
 
         # add status msg, if not empty, under contact name
-        status = contact.status
-        if status is not None:
-            status = status.strip()
-
+        status = contact.status.strip()
         if status and app.settings.get('show_status_msgs_in_roster'):
             # Display only first line
             status = status.split('\n', 1)[0]
