@@ -19,55 +19,67 @@ from typing import Optional
 from typing import TYPE_CHECKING
 
 import logging
+import os
+import shutil
+from pathlib import Path
 
-from gi.repository import Gtk
 from gi.repository import Gdk
-from gi.repository import GLib
 from gi.repository import Gio
-
+from gi.repository import GLib
+from gi.repository import Gtk
 from nbxmpp import JID
 
 from gajim.common import app
 from gajim.common import events
 from gajim.common import ged
+from gajim.common import types
 from gajim.common.client import Client
 from gajim.common.const import Direction
 from gajim.common.const import Display
 from gajim.common.const import SimpleClientState
 from gajim.common.ged import EventHelper
+from gajim.common.helpers import open_file
+from gajim.common.helpers import open_uri
+from gajim.common.helpers import play_sound
+from gajim.common.helpers import show_in_folder
 from gajim.common.i18n import _
 from gajim.common.modules.bytestream import is_transfer_active
-from gajim.plugins.pluginmanager import PluginManifest
+from gajim.common.modules.contacts import GroupchatContact
+from gajim.plugins.manifest import PluginManifest
 from gajim.plugins.repository import PluginRepository
 
-from .account_side_bar import AccountSideBar
-from .app_side_bar import AppSideBar
-from .workspace_side_bar import WorkspaceSideBar
-from .main_stack import MainStack
-from .call_window import CallWindow
-from .chat_list import ChatList
-from .chat_list_row import ChatListRow
-from .chat_stack import ChatStack
-from .const import MAIN_WIN_ACTIONS
-from .dialogs import DialogButton
-from .dialogs import ConfirmationDialog
-from .dialogs import ConfirmationCheckDialog
-from .builder import get_builder
-from .util import get_app_window
-from .util import resize_window
-from .util import restore_main_window_position
-from .util import save_main_window_position
-from .util import open_window
-from .util import set_urgency_hint
-from .structs import AccountJidParam
-from .structs import AddChatActionParams
-from .structs import actionmethod
-from .structs import ChatListEntryParam
+from gajim.gtk.account_side_bar import AccountSideBar
+from gajim.gtk.app_side_bar import AppSideBar
+from gajim.gtk.builder import get_builder
+from gajim.gtk.call_window import CallWindow
+from gajim.gtk.chat_list import ChatList
+from gajim.gtk.chat_list_row import ChatListRow
+from gajim.gtk.chat_stack import ChatStack
+from gajim.gtk.const import MAIN_WIN_ACTIONS
+from gajim.gtk.dialogs import ConfirmationCheckDialog
+from gajim.gtk.dialogs import ConfirmationDialog
+from gajim.gtk.dialogs import DialogButton
+from gajim.gtk.dialogs import ErrorDialog
+from gajim.gtk.dialogs import InputDialog
+from gajim.gtk.filechoosers import FileSaveDialog
+from gajim.gtk.main_stack import MainStack
+from gajim.gtk.structs import AccountJidParam
+from gajim.gtk.structs import actionmethod
+from gajim.gtk.structs import AddChatActionParams
+from gajim.gtk.structs import ChatListEntryParam
+from gajim.gtk.structs import RetractMessageParam
+from gajim.gtk.util import get_app_window
+from gajim.gtk.util import open_window
+from gajim.gtk.util import resize_window
+from gajim.gtk.util import restore_main_window_position
+from gajim.gtk.util import save_main_window_position
+from gajim.gtk.util import set_urgency_hint
+from gajim.gtk.workspace_side_bar import WorkspaceSideBar
 
 if TYPE_CHECKING:
-    from .control import ChatControl
+    from gajim.gtk.control import ChatControl
 
-log = logging.getLogger('gajim.gui.main')
+log = logging.getLogger('gajim.gtk.main')
 
 
 class MainWindow(Gtk.ApplicationWindow, EventHelper):
@@ -128,6 +140,8 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
             ('password-required', ged.GUI1, self._on_password_required),
             ('http-auth', ged.GUI1, self._on_http_auth),
             ('muc-added', ged.GUI1, self._on_muc_added),
+            ('message-sent', ged.GUI1, self._on_message_sent),
+            ('signed-in', ged.GUI1, self._on_signed_in),
         ])
 
         app.plugin_repository.connect('plugin-updates-available',
@@ -151,7 +165,7 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
 
     def get_action(self, name: str) -> Gio.SimpleAction:
         action = self.lookup_action(name)
-        assert action is not None
+        assert isinstance(action, Gio.SimpleAction)
         return action
 
     def get_chat_stack(self) -> ChatStack:
@@ -186,6 +200,22 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
         self.deiconify()
         self.present_with_time(Gtk.get_current_event_time())
 
+    def mark_workspace_as_read(self, workspace: str) -> None:
+        chat_list_stack = self._chat_page.get_chat_list_stack()
+        chat_list = chat_list_stack.get_chatlist(workspace)
+        open_chats = chat_list.get_open_chats()
+        for chat in open_chats:
+            self.mark_as_read(chat['account'], chat['jid'])
+
+    def _mark_workspace_as_read(self,
+                                _action: Gio.SimpleAction,
+                                param: GLib.Variant
+                                ) -> None:
+
+        workspace_id = param.get_string() or None
+        if workspace_id is not None:
+            self.mark_workspace_as_read(workspace_id)
+
     def _prepare_window(self) -> None:
         window_width = app.settings.get('mainwin_width')
         window_height = app.settings.get('mainwin_height')
@@ -232,8 +262,13 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
         self.add_app_message(event.name)
 
     def _on_gajim_update_available(self,
-                                   event: events.GajimUpdateAvailable) -> None:
-        self.add_app_message(event.name, event.version)
+                                   event: events.GajimUpdateAvailable
+                                   ) -> None:
+
+        self.add_app_message(
+            event.name,
+            new_version=event.version,
+            new_setup_url=event.setup_url)
 
     @staticmethod
     def _on_roster_item_exchange(event: events.RosterItemExchangeEvent) -> None:
@@ -300,7 +335,22 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
         if self.chat_exists(event.account, event.jid):
             return
 
-        self.add_group_chat(event.account, event.jid)
+        self.add_group_chat(event.account, event.jid, select=event.select_chat)
+
+    def _on_message_sent(self, event: events.MessageSent) -> None:
+        if not event.play_sound:
+            return
+
+        enabled = app.settings.get_soundevent_settings(
+            'message_sent')['enabled']
+        if enabled:
+            if isinstance(event.jid, list) and len(event.jid) > 1:
+                return
+            play_sound('message_sent', event.account)
+
+    def _on_signed_in(self, event: events.SignedIn) -> None:
+        if app.settings.get('ask_online_status'):
+            self.show_account_page(event.account)
 
     def _add_actions(self) -> None:
         for action, variant_type, enabled in MAIN_WIN_ACTIONS:
@@ -366,10 +416,20 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
             ('switch-workspace-8', self._on_action),
             ('switch-workspace-9', self._on_action),
             ('toggle-chat-list', self._on_action),
+            ('preview-download', self._on_preview_action),
+            ('preview-open', self._on_preview_action),
+            ('preview-save-as', self._on_preview_action),
+            ('preview-open-folder', self._on_preview_action),
+            ('preview-copy-link', self._on_preview_action),
+            ('preview-open-link', self._on_preview_action),
+            ('copy-message', self._on_copy_message),
+            ('retract-message', self._on_retract_message),
+            ('delete-message-locally', self._on_delete_message_locally),
             ('add-workspace', self._add_workspace),
             ('edit-workspace', self._edit_workspace),
             ('remove-workspace', self._remove_workspace),
             ('activate-workspace', self._activate_workspace),
+            ('mark-workspace-as-read', self._mark_workspace_as_read),
             ('add-chat', self._add_chat),
             ('add-group-chat', self._add_group_chat),
             ('add-to-roster', self._add_to_roster),
@@ -444,6 +504,92 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
 
         return None
 
+    def _on_preview_action(self,
+                           action: Gio.SimpleAction,
+                           param: GLib.Variant
+                           ) -> None:
+
+        action_name = action.get_name()
+        preview = app.preview_manager.get_preview(param.get_string())
+        if preview is None:
+            return
+
+        if action_name == 'preview-download':
+            if not preview.orig_exists:
+                app.preview_manager.download_content(preview, force=True)
+
+        elif action_name == 'preview-open':
+            if preview.is_geo_uri:
+                open_uri(preview.uri)
+                return
+
+            if not preview.orig_exists:
+                app.preview_manager.download_content(preview, force=True)
+                return
+
+            assert preview.orig_path is not None
+            open_file(preview.orig_path)
+
+        elif action_name == 'preview-save-as':
+            def _on_ok(paths: list[str]) -> None:
+                target = paths[0]
+                assert preview is not None
+                assert preview.orig_path is not None
+
+                target_path = Path(target)
+                orig_ext = preview.orig_path.suffix
+                new_ext = target_path.suffix
+                if orig_ext != new_ext:
+                    # Windows file chooser selects the full file name including
+                    # extension. Starting to type will overwrite the extension
+                    # as well. Restore the extension if it's lost.
+                    target_path = target_path.with_suffix(orig_ext)
+                dirname = target_path.parent
+                if not os.access(dirname, os.W_OK):
+                    ErrorDialog(
+                        _('Directory "%s" is not writable') % dirname,
+                        _('You do not have the proper permissions to '
+                          'create files in this directory.'),
+                        transient_for=self)
+                    return
+
+                shutil.copyfile(preview.orig_path, target_path)
+                app.settings.set('last_save_dir', str(target_path.parent))
+
+            if not preview.orig_exists:
+                app.preview_manager.download_content(preview, force=True)
+                return
+
+            FileSaveDialog(_on_ok,
+                           path=app.settings.get('last_save_dir'),
+                           file_name=preview.filename,
+                           transient_for=self)
+
+        elif action_name == 'preview-open-folder':
+            if not preview.orig_exists:
+                app.preview_manager.download_content(preview, force=True)
+                return
+
+            assert preview.orig_path is not None
+            show_in_folder(preview.orig_path)
+
+        elif action_name == 'preview-copy-link':
+            display = Gdk.Display.get_default()
+            assert display is not None
+            clipboard = Gtk.Clipboard.get_default(display)
+            clipboard.set_text(preview.uri, -1)
+
+        elif action_name == 'preview-open-link':
+            if preview.is_aes_encrypted:
+                if preview.is_geo_uri:
+                    open_uri(preview.uri)
+                    return
+
+                assert preview.orig_path
+                open_file(preview.orig_path)
+            else:
+                open_uri(preview.uri)
+
     def _on_show_offline(self,
                          action: Gio.SimpleAction,
                          value: GLib.Variant) -> None:
@@ -474,6 +620,56 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
                     'go-previous-symbolic', Gtk.IconSize.BUTTON)
         self._chat_page.toggle_chat_list()
 
+    def _on_copy_message(self,
+                         _action: Gio.SimpleAction,
+                         param: GLib.Variant
+                         ) -> None:
+
+        clip = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
+        clip.set_text(param.get_string(), -1)
+
+    @actionmethod
+    def _on_retract_message(self,
+                            _action: Gio.SimpleAction,
+                            params: RetractMessageParam
+                            ) -> None:
+
+        def _on_retract(reason: str) -> None:
+            client = app.get_client(params.account)
+            client.get_module('MUC').retract_message(
+                params.jid, params.stanza_id, reason or None)
+
+        InputDialog(
+            _('Retract Message'),
+            _('Retract message?'),
+            _('Why do you want to retract this message?'),
+            [DialogButton.make('Cancel'),
+             DialogButton.make('Remove',
+                               text=_('_Retract'),
+                               callback=_on_retract)],
+            input_str=_('Spam'),
+            transient_for=app.window).show()
+
+    def _on_delete_message_locally(self,
+                                   _action: Gio.SimpleAction,
+                                   param: GLib.Variant
+                                   ) -> None:
+
+        def _on_delete() -> None:
+            log_line_id = param.get_uint32()
+            app.storage.archive.delete_message_from_logs(log_line_id)
+            control = self.get_control()
+            control.remove_message(log_line_id)
+
+        ConfirmationDialog(
+            _('Delete Message'),
+            _('Delete message locally?'),
+            _('This message will be deleted from your local chat history'),
+            [DialogButton.make('Cancel'),
+             DialogButton.make('Delete',
+                               callback=_on_delete)],
+            transient_for=app.window).show()
+
     def _on_window_motion_notify(self,
                                  _widget: Gtk.ApplicationWindow,
                                  _event: Gdk.EventMotion
@@ -565,11 +761,11 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
     def is_chat_active(self, account: str, jid: JID) -> bool:
         if not self.has_toplevel_focus():
             return False
+        if self._main_stack.get_visible_page_name() != 'chats':
+            return False
         return self._chat_page.is_chat_selected(account, jid)
 
     def highlight_dnd_targets(self, drag_row: Any, highlight: bool) -> None:
-        css_class = 'dnd-target'
-
         if isinstance(drag_row, ChatListRow):
             chat_list_stack = self._chat_page.get_chat_list_stack()
             workspace = self.get_active_workspace()
@@ -583,22 +779,25 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
                         continue
 
                     if highlight:
-                        row.get_style_context().add_class(css_class)
+                        row.get_style_context().add_class(
+                            'dnd-target-chatlist')
                     else:
-                        row.get_style_context().remove_class(css_class)
+                        row.get_style_context().remove_class(
+                            'dnd-target-chatlist')
 
         if highlight:
-            self._workspace_side_bar.get_style_context().add_class(css_class)
+            self._workspace_side_bar.get_style_context().add_class(
+                'dnd-target')
         else:
             self._workspace_side_bar.get_style_context().remove_class(
-                css_class)
+                'dnd-target')
 
     def _add_workspace(self,
                        _action: Gio.SimpleAction,
                        param: GLib.Variant) -> None:
 
         workspace_id = param.get_string()
-        if workspace_id is not None:
+        if workspace_id:
             self.add_workspace(workspace_id)
 
     def add_workspace(self,
@@ -637,7 +836,7 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
             self.remove_workspace(workspace_id)
 
     def remove_workspace(self, workspace_id: str) -> None:
-        if len(app.settings.get_workspaces()) == 1:
+        if app.settings.get_workspace_count() == 1:
             log.warning('Tried to remove the only workspace')
             return
 
@@ -689,7 +888,7 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
                             param: GLib.Variant) -> None:
 
         workspace_id = param.get_string()
-        if workspace_id is not None:
+        if workspace_id:
             self.activate_workspace(workspace_id)
 
     def activate_workspace(self, workspace_id: str) -> None:
@@ -713,6 +912,29 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
         chat_list_stack = self._chat_page.get_chat_list_stack()
         return chat_list_stack.get_chatlist(workspace_id)
 
+    def _get_suitable_workspace(self,
+                                account: str,
+                                jid: JID,
+                                private_chat: bool = False
+                                ) -> str:
+
+        if private_chat:
+            # Try to add private chat to the same workspace the MUC resides in
+            chat_list_stack = self._chat_page.get_chat_list_stack()
+            chat_list = chat_list_stack.find_chat(account, jid.new_as_bare())
+            if chat_list is not None:
+                return chat_list.workspace_id
+
+        default = app.settings.get_account_setting(account, 'default_workspace')
+        workspaces = app.settings.get_workspaces()
+        if default in workspaces:
+            return default
+
+        workspace_id = self.get_active_workspace()
+        if workspace_id is not None:
+            return workspace_id
+        return self._workspace_side_bar.get_first_workspace()
+
     def _add_group_chat(self,
                         _action: Gio.SimpleAction,
                         param: GLib.Variant) -> None:
@@ -720,11 +942,13 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
         account, jid, select = param.unpack()
         self.add_group_chat(account, JID.from_string(jid), select)
 
-    def add_group_chat(self, account: str, jid: JID,
-                       select: bool = False) -> None:
-        workspace_id = self.get_active_workspace()
-        if workspace_id is None:
-            workspace_id = self._workspace_side_bar.get_first_workspace()
+    def add_group_chat(self,
+                       account: str,
+                       jid: JID,
+                       select: bool = False
+                       ) -> None:
+
+        workspace_id = self._get_suitable_workspace(account, jid)
         self._chat_page.add_chat_for_workspace(workspace_id,
                                                account,
                                                jid,
@@ -743,17 +967,10 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
                  jid: JID,
                  type_: str,
                  select: bool = False,
-                 workspace: str = 'default',
                  message: Optional[str] = None
                  ) -> None:
 
-        if workspace == 'current':
-            workspace_id = self.get_active_workspace()
-            if workspace_id is None:
-                workspace_id = self._workspace_side_bar.get_first_workspace()
-        else:
-            workspace_id = self._workspace_side_bar.get_first_workspace()
-
+        workspace_id = self._get_suitable_workspace(account, jid)
         self._chat_page.add_chat_for_workspace(workspace_id,
                                                account,
                                                jid,
@@ -764,21 +981,24 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
     def add_private_chat(self,
                          account: str,
                          jid: JID,
-                         select: bool = False) -> None:
-        # Try to add private chat to the same workspace the MUC resides in
-        chat_list_stack = self._chat_page.get_chat_list_stack()
-        chat_list = chat_list_stack.find_chat(account, jid.new_as_bare())
-        if chat_list is not None:
-            workspace_id = chat_list.workspace_id
-        else:
-            workspace_id = self._workspace_side_bar.get_first_workspace()
+                         select: bool = False
+                         ) -> None:
 
+        workspace_id = self._get_suitable_workspace(account,
+                                                    jid,
+                                                    private_chat=True)
         self._chat_page.add_chat_for_workspace(workspace_id,
                                                account,
                                                jid,
                                                'pm',
                                                select=select)
 
+    def clear_chat_list_row(self, account: str, jid: JID) -> None:
+        chat_list_stack = self._chat_page.get_chat_list_stack()
+        chat_list = chat_list_stack.find_chat(account, jid)
+        if chat_list is not None:
+            chat_list.clear_chat_list_row(account, jid)
+
     def select_chat(self, account: str, jid: JID) -> None:
         self._app_side_bar.unselect_all()
         self._account_side_bar.unselect_all()
@@ -812,8 +1032,11 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
 
     def add_app_message(self,
                         category: str,
-                        message: Optional[str] = None) -> None:
-        self._app_page.add_app_message(category, message)
+                        new_version: Optional[str] = None,
+                        new_setup_url: Optional[str] = None
+                        ) -> None:
+
+        self._app_page.add_app_message(category, new_version, new_setup_url)
 
     def get_control(self) -> ChatControl:
         return self._chat_page.get_control()
@@ -822,17 +1045,17 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
         return self._chat_page.chat_exists(account, jid)
 
     def is_message_correctable(self,
-                               account: str,
-                               jid: JID,
+                               contact: types.ChatContactT,
                                message_id: str
                                ) -> bool:
+
         chat_stack = self._chat_page.get_chat_stack()
-        last_message_id = chat_stack.get_last_message_id(account, jid)
+        last_message_id = chat_stack.get_last_message_id(contact)
         if last_message_id is None or last_message_id != message_id:
             return False
 
         message_row = app.storage.archive.get_last_correctable_message(
-            account, jid, last_message_id)
+            contact.account, contact.jid, last_message_id)
         return message_row is not None
 
     def get_total_unread_count(self) -> int:
@@ -855,13 +1078,15 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
                      send_marker: bool = True
                      ) -> None:
 
-        unread_count = self.get_chat_unread_count(account, jid)
+        unread_count = self.get_chat_unread_count(account,
+                                                  jid,
+                                                  include_silent=True)
 
         set_urgency_hint(self, False)
         control = self.get_control()
         if control.has_active_chat():
             # Reset jump to bottom button unread counter
-            control.mark_as_read(send_marker=send_marker)
+            control.mark_as_read()
 
         # Reset chat list unread counter (emits unread-count-changed)
         chat_list_stack = self._chat_page.get_chat_list_stack()
@@ -873,11 +1098,15 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
 
         last_message = app.storage.archive.get_last_conversation_line(
             account, jid)
+        if last_message is None:
+            return
+
         client = app.get_client(account)
         contact = client.get_module('Contacts').get_contact(jid)
         client.get_module('ChatMarkers').send_displayed_marker(
             contact,
-            last_message.message_id)
+            last_message.message_id,
+            last_message.stanza_id)
 
     def _on_window_active(self,
                           window: Gtk.ApplicationWindow,
@@ -895,6 +1124,64 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
         if control.get_autoscroll():
             self.mark_as_read(control.contact.account, control.contact.jid)
 
+    def get_preferred_ft_method(self,
+                                contact: types.ChatContactT
+                                ) -> Optional[str]:
+
+        ft_pref = app.settings.get_account_setting(
+            contact.account,
+            'filetransfer_preference')
+        httpupload_enabled = app.window.get_action_enabled(
+            'send-file-httpupload')
+        jingle_enabled = app.window.get_action_enabled('send-file-jingle')
+
+        if isinstance(contact, GroupchatContact):
+            if httpupload_enabled:
+                return 'httpupload'
+            return None
+
+        if httpupload_enabled and jingle_enabled:
+            return ft_pref
+
+        if httpupload_enabled:
+            return 'httpupload'
+
+        if jingle_enabled:
+            return 'jingle'
+
+        return None
+
+    def show_add_join_groupchat(self,
+                                account: str,
+                                jid: str,
+                                nickname: Optional[str] = None,
+                                password: Optional[str] = None
+                                ) -> None:
+
+        if not self.chat_exists(account, JID.from_string(jid)):
+            client = app.get_client(account)
+            client.get_module('MUC').join(
+                jid, nick=nickname, password=password)
+
+        self.add_group_chat(account, JID.from_string(jid), select=True)
+
+    def start_chat_from_jid(self,
+                            account: str,
+                            jid: str,
+                            message: Optional[str] = None
+                            ) -> None:
+
+        jid_ = JID.from_string(jid)
+        if self.chat_exists(account, jid_):
+            self.select_chat(account, jid_)
+            if message is not None:
+                message_input = self.get_chat_stack().get_message_input()
+                message_input.insert_text(message)
+            return
+
+        app.app.activate_action(
+            'start-chat', GLib.Variant('as', [str(jid), message or '']))
+
     @staticmethod
     def contact_info(account: str, jid: str) -> None:
         client = app.get_client(account)
@@ -989,21 +1276,17 @@ class MainWindow(Gtk.ApplicationWindow, EventHelper):
                 self.add_chat(event.account, jid, 'contact')
 
     def _on_read_state_sync(self, event: events.ReadStateSync) -> None:
-        if event.is_muc_pm:
-            jid = JID.from_string(event.jid.bare)
-        else:
-            jid = event.jid
-
         last_message = app.storage.archive.get_last_conversation_line(
-            event.account, jid)
+            event.account, event.jid)
 
         if last_message is None:
             return
 
-        if event.marker_id != last_message.message_id:
+        if event.marker_id not in (last_message.message_id,
+                                   last_message.stanza_id):
             return
 
-        self.mark_as_read(event.account, jid, send_marker=False)
+        self.mark_as_read(event.account, event.jid, send_marker=False)
 
     def _on_call_started(self, event: events.CallStarted) -> None:
         # Make sure there is only one window
