@@ -17,73 +17,56 @@ from __future__ import annotations
 from typing import Any
 from typing import Optional
 
-import os
-import sys
-import tempfile
 import logging
+import tempfile
 import uuid
+from pathlib import Path
 
-from gi.repository import GLib
 from gi.repository import Gdk
-from gi.repository import GdkPixbuf
 from gi.repository import Gio
-from gi.repository import Gtk
+from gi.repository import GLib
 from gi.repository import GObject
-
+from gi.repository import Gtk
 from nbxmpp.const import Chatstate
 from nbxmpp.modules.security_labels import SecurityLabel
-from nbxmpp.protocol import JID
 
 from gajim.common import app
-from gajim.common import events
-from gajim.common import ged
-from gajim.common.commands import CommandFailed
-from gajim.common.i18n import _
 from gajim.common.client import Client
-from gajim.common.const import SimpleClientState
+from gajim.common.commands import CommandFailed
 from gajim.common.const import Direction
+from gajim.common.const import SimpleClientState
+from gajim.common.i18n import _
+from gajim.common.modules.contacts import BareContact
 from gajim.common.modules.contacts import GroupchatContact
 from gajim.common.modules.contacts import GroupchatParticipant
 from gajim.common.types import ChatContactT
-from gajim.gui.security_label_selector import SecurityLabelSelector
 
-from .builder import get_builder
-from .dialogs import DialogButton
-from .dialogs import ErrorDialog
-from .dialogs import PastePreviewDialog
-from .emoji_chooser import emoji_chooser
-from .menus import get_encryption_menu
-from .menus import get_format_menu
-from .menus import get_groupchat_menu
-from .menus import get_private_chat_menu
-from .menus import get_self_contact_menu
-from .menus import get_singlechat_menu
-from .message_input import MessageInputTextView
+from gajim.gtk.builder import get_builder
+from gajim.gtk.dialogs import ErrorDialog
+from gajim.gtk.menus import get_encryption_menu
+from gajim.gtk.menus import get_format_menu
+from gajim.gtk.message_input import MessageInputTextView
+from gajim.gtk.security_label_selector import SecurityLabelSelector
+from gajim.gtk.util import open_window
+
+log = logging.getLogger('gajim.gtk.messageactionsbox')
 
 
-log = logging.getLogger('gajim.gui.messageactionsbox')
-
-
-class MessageActionsBox(Gtk.Grid, ged.EventHelper):
+class MessageActionsBox(Gtk.Grid):
     def __init__(self) -> None:
         Gtk.Grid.__init__(self)
-        ged.EventHelper.__init__(self)
 
         self._client: Optional[Client] = None
         self._contact: Optional[ChatContactT] = None
 
         self._ui = get_builder('message_actions_box.ui')
-        self.get_style_context().add_class('message-action-box')
+        self.get_style_context().add_class('message-actions-box')
 
         self.attach(self._ui.box, 0, 0, 1, 1)
 
         # For undo
         self.space_pressed = False
 
-        # XEP-0308 Message Correction
-        self.is_correcting = False
-        self.last_message_id: dict[tuple[str, JID], Optional[str]] = {}
-
         self._ui.send_message_button.set_visible(
             app.settings.get('show_send_message_button'))
         app.settings.bind_signal('show_send_message_button',
@@ -108,19 +91,11 @@ class MessageActionsBox(Gtk.Grid, ged.EventHelper):
         self._ui.formattings_button.set_menu_model(get_format_menu())
         self._ui.encryption_menu_button.set_menu_model(get_encryption_menu())
 
-        self._ui.quick_invite_button.set_action_name('win.muc-invite')
-
-        self._init_emoticon_popover()
-
         self.show_all()
         self._ui.connect_signals(self)
 
         self._connect_actions()
 
-        self.register_events([
-            ('message-sent', ged.GUI2, self._on_message_sent)
-        ])
-
         app.plugin_manager.gui_extension_point(
             'message_actions_box', self, self._ui.box)
 
@@ -173,7 +148,8 @@ class MessageActionsBox(Gtk.Grid, ged.EventHelper):
             self._on_format(action_name)
 
         elif action_name == 'show-emoji-chooser':
-            self._on_show_emoji_chooser()
+            self.msg_textview.emit('insert-emoji')
+            self._ui.emoticons_button.set_active(False)
 
         elif action_name == 'quote':
             assert param
@@ -201,26 +177,21 @@ class MessageActionsBox(Gtk.Grid, ged.EventHelper):
         self._contact = contact
 
         if isinstance(self._contact, GroupchatContact):
-            self._ui.quick_invite_button.show()
             self._contact.multi_connect({
                 'state-changed': self._on_muc_state_changed,
                 'user-role-changed': self._on_muc_state_changed,
             })
         elif isinstance(self._contact, GroupchatParticipant):
-            self._ui.quick_invite_button.hide()
             self._contact.multi_connect({
                 'user-joined': self._on_user_state_changed,
                 'user-left': self._on_user_state_changed,
                 'room-joined': self._on_user_state_changed,
                 'room-left': self._on_user_state_changed,
             })
-        else:
-            self._ui.quick_invite_button.hide()
-
-        self._set_settings_menu(contact)
 
         self._update_encryption_button()
         self._update_encryption_details_button()
+        self._update_send_file_button_tooltip()
 
         self._set_chatstate(True)
 
@@ -240,6 +211,19 @@ class MessageActionsBox(Gtk.Grid, ged.EventHelper):
         self._contact = None
         self._client = None
 
+    @property
+    def is_correcting(self) -> bool:
+        return self.msg_textview.is_correcting
+
+    def toggle_message_correction(self) -> None:
+        self.msg_textview.toggle_message_correction()
+
+    def try_message_correction(self, message: str) -> Optional[str]:
+        return self.msg_textview.try_message_correction(message)
+
+    def get_last_message_id(self, contact: ChatContactT) -> Optional[str]:
+        return self.msg_textview.get_last_message_id(contact)
+
     def _on_client_state_changed(self,
                                  _client: Client,
                                  _signal_name: str,
@@ -297,7 +281,7 @@ class MessageActionsBox(Gtk.Grid, ged.EventHelper):
         if current_state == new_state:
             return
 
-        if new_state:
+        if new_state and new_state != 'OMEMO':
             plugin = app.plugin_manager.encryption_plugins.get(new_state)
             if plugin is None:
                 # TODO: Add GUI error here
@@ -334,6 +318,12 @@ class MessageActionsBox(Gtk.Grid, ged.EventHelper):
                             'Encryption is not available.')
                 icon_name = 'channel-insecure-symbolic'
 
+                # Disable encryption if chat was encrypted before, but due to
+                # changed circumstances, encryption is not applicable anymore
+                # (i.e. group chat configuration changed).
+                contact.settings.set('encryption', '')
+                action.set_state(GLib.Variant('s', ''))
+
         self._ui.encryption_menu_button.set_sensitive(sensitive)
         self._ui.encryption_menu_button.set_tooltip_text(tooltip)
         self._ui.encryption_image.set_from_icon_name(
@@ -341,17 +331,21 @@ class MessageActionsBox(Gtk.Grid, ged.EventHelper):
 
     def _update_encryption_details_button(self) -> None:
         contact = self.get_current_contact()
-        state = contact.settings.get('encryption')
+        encryption = contact.settings.get('encryption')
 
-        encryption_state = {'visible': bool(state),
-                            'enc_type': state,
+        encryption_state = {'visible': bool(encryption),
+                            'enc_type': encryption,
                             'authenticated': False}
 
-        if state:
-            app.plugin_manager.extension_point(
-                f'encryption_state{state}',
-                app.window.get_control(),
-                encryption_state)
+        if encryption:
+            if encryption == 'OMEMO':
+                encryption_state['authenticated'] = True
+            else:
+                # Only fire extension_point for plugins (i.e. not OMEMO)
+                app.plugin_manager.extension_point(
+                    f'encryption_state{encryption}',
+                    app.window.get_control(),
+                    encryption_state)
 
         visible, enc_type, authenticated = encryption_state.values()
         assert isinstance(visible, bool)
@@ -378,45 +372,27 @@ class MessageActionsBox(Gtk.Grid, ged.EventHelper):
     def _on_encryption_details_clicked(self, _button: Gtk.Button) -> None:
         contact = self.get_current_contact()
         encryption = contact.settings.get('encryption')
+        if encryption == 'OMEMO':
+            if contact.is_groupchat:
+                open_window('GroupchatDetails',
+                            contact=contact,
+                            page='encryption-omemo')
+                return
+
+            if isinstance(contact, BareContact) and contact.is_self:
+                window = open_window('AccountsWindow')
+                window.select_account(contact.account, page='encryption-omemo')
+                return
+
+            open_window('ContactInfo',
+                        account=contact.account,
+                        contact=contact,
+                        page='encryption-omemo')
+            return
+
         app.plugin_manager.extension_point(
             f'encryption_dialog{encryption}', app.window.get_control())
 
-    def _set_settings_menu(self, contact: ChatContactT) -> None:
-        if isinstance(contact, GroupchatContact):
-            menu = get_groupchat_menu(contact)
-        elif isinstance(contact, GroupchatParticipant):
-            menu = get_private_chat_menu(contact)
-        elif contact.is_self:
-            menu = get_self_contact_menu(contact)
-        else:
-            menu = get_singlechat_menu(contact)
-        self._ui.settings_menu.set_menu_model(menu)
-
-    def _init_emoticon_popover(self) -> None:
-        if not app.settings.get('emoticons_theme'):
-            return
-
-        if sys.platform == 'darwin':
-            emoji_chooser.text_widget = self.msg_textview
-            self._ui.emoticons_button.set_popover(emoji_chooser)
-            return
-
-        self._ui.emoticons_button.set_sensitive(True)
-        self._ui.emoticons_button.connect('clicked',
-                                          self._on_emoticon_button_clicked)
-
-    def toggle_emoticons(self) -> None:
-        if app.settings.get('emoticons_theme'):
-            self._ui.emoticons_button.set_no_show_all(False)
-            self._ui.emoticons_button.show()
-        else:
-            self._ui.emoticons_button.set_no_show_all(True)
-            self._ui.emoticons_button.hide()
-
-    def _on_emoticon_button_clicked(self, _widget: Gtk.Button) -> None:
-        self.msg_textview.emit('insert-emoji')
-        self._ui.emoticons_button.set_active(False)
-
     def _on_format(self, name: str) -> None:
         name = name.removeprefix('input-')
         self.msg_textview.apply_formatting(name)
@@ -424,14 +400,6 @@ class MessageActionsBox(Gtk.Grid, ged.EventHelper):
     def _on_clear(self) -> None:
         self.msg_textview.clear()
 
-    def _on_show_emoji_chooser(self) -> None:
-        if sys.platform == 'darwin':
-            popover = self._ui.emoticons_button.get_popover()
-            assert popover
-            popover.show()
-        else:
-            self.msg_textview.emit('insert-emoji')
-
     def _on_send_file_enabled_changed(self,
                                       action: Gio.SimpleAction,
                                       _param: GObject.ParamSpec) -> None:
@@ -547,15 +515,14 @@ class MessageActionsBox(Gtk.Grid, ged.EventHelper):
             assert self._contact is not None
 
             message = self.msg_textview.get_text()
-            if (message.startswith('/') and
-                    not message.startswith('//') and
-                    not message.startswith('/me ')):
-                try:
-                    app.commands.parse(self._contact.type_string, message)
-                except CommandFailed:
-                    pass
-                else:
-                    self.msg_textview.clear()
+
+            try:
+                handled = app.commands.parse(self._contact.type_string, message)
+            except CommandFailed:
+                return True
+
+            if handled:
+                self.msg_textview.clear()
                 return True
 
             if not app.account_is_available(self._contact.account):
@@ -571,87 +538,30 @@ class MessageActionsBox(Gtk.Grid, ged.EventHelper):
         return False
 
     def _on_paste_clipboard(self,
-                            _texview: MessageInputTextView
+                            texview: MessageInputTextView
                             ) -> None:
 
         clipboard = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
+
+        uris = clipboard.wait_for_uris()
+        if uris:
+            app.window.activate_action('send-file', GLib.Variant('as', uris))
+            # prevent TextView from pasting the URIs as text:
+            texview.stop_emission_by_name('paste-clipboard')
+            return
+        log.info('No URIs pasted')
+
         image = clipboard.wait_for_image()
         if image is None:
+            log.info('No image pasted')
             return
 
-        if not app.settings.get('confirm_paste_image'):
-            self._paste_event_confirmed(True, image)
-            return
+        temp_dir = Path(tempfile.gettempdir())
+        image_path = temp_dir / f'{uuid.uuid4()}.png'
 
-        PastePreviewDialog(
-            _('Paste Image'),
-            _('You are trying to paste an image'),
-            _('Are you sure you want to paste your '
-              "clipboard's image into the chat window?"),
-            _('_Do not ask me again'),
-            image,
-            [DialogButton.make('Cancel'),
-                DialogButton.make('Accept',
-                                  text=_('_Paste'),
-                                  callback=self._paste_event_confirmed,
-                                  args=[image])]).show()
-
-    def _paste_event_confirmed(self,
-                               is_checked: bool,
-                               image: GdkPixbuf.Pixbuf
-                               ) -> None:
-        if is_checked:
-            app.settings.set('confirm_paste_image', False)
-
-        dir_ = tempfile.gettempdir()
-        path = os.path.join(dir_, f'{uuid.uuid4()}.png')
-        if image is None:
+        if not image.savev(str(image_path), 'png', [], []):
             log.error('Could not process pasted image')
             return
 
-        image.savev(path, 'png', [], [])
-
-        assert self._contact is not None
-        app.interface.start_file_transfer(self._contact, path)
-
-    def toggle_message_correction(self) -> None:
-        if self.is_correcting:
-            self.is_correcting = False
-            self.msg_textview.clear()
-            self.msg_textview.get_style_context().remove_class(
-                'gajim-msg-correcting')
-            self.msg_textview.grab_focus()
-            return
-
-        assert self._contact is not None
-        last_message_id = self.last_message_id.get(
-            (self._contact.account, self._contact.jid))
-        if last_message_id is None:
-            return
-
-        message_row = app.storage.archive.get_last_correctable_message(
-            self._contact.account, self._contact.jid, last_message_id)
-        if message_row is None:
-            return
-
-        self.is_correcting = True
-        self.msg_textview.clear()
-        self.msg_textview.insert_text(message_row.message)
-        self.msg_textview.get_style_context().add_class('gajim-msg-correcting')
-        self.msg_textview.grab_focus()
-
-    def _on_message_sent(self, event: events.MessageSent) -> None:
-        if not event.message:
-            return
-
-        if event.correct_id is None:
-            # This wasn't a corrected message
-            assert self._contact is not None
-            oob_url = event.additional_data.get_value('gajim', 'oob_url')
-            account = self._contact.account
-            jid = self._contact.jid
-            if oob_url == event.message:
-                # Don't allow to correct HTTP Upload file transfer URLs
-                self.last_message_id[(account, jid)] = None
-            else:
-                self.last_message_id[(account, jid)] = event.message_id
+        app.window.activate_action(
+            'send-file', GLib.Variant('as', [image_path.as_uri()]))
