@@ -22,25 +22,27 @@ from __future__ import annotations
 from typing import Any
 from typing import Optional
 
-import sys
+import logging
+from collections import defaultdict
 
-from gi.repository import Gtk
 from gi.repository import Gdk
-from gi.repository import GdkPixbuf
 from gi.repository import GLib
+from gi.repository import Gtk
 from gi.repository import Pango
 
-from nbxmpp.protocol import JID
-
 from gajim.common import app
-from gajim.common.i18n import LANG
+from gajim.common import ged
+from gajim.common.events import MessageSent
+from gajim.common.ged import EventHelper
 from gajim.common.i18n import _
-from gajim.common.styling import process
+from gajim.common.i18n import LANG
 from gajim.common.styling import PlainBlock
+from gajim.common.styling import process
 from gajim.common.types import ChatContactT
 
-from .chat_action_processor import ChatActionProcessor
-from .util import scroll_to_end
+from gajim.gtk.chat_action_processor import ChatActionProcessor
+from gajim.gtk.const import MAX_MESSAGE_LENGTH
+from gajim.gtk.util import scroll_to_end
 
 if app.is_installed('GSPELL'):
     from gi.repository import Gspell  # pylint: disable=ungrouped-imports
@@ -54,13 +56,17 @@ FORMAT_CHARS: dict[str, str] = {
     'pre': '`',
 }
 
+log = logging.getLogger('gajim.gtk.message_input')
 
-class MessageInputTextView(Gtk.TextView):
+
+class MessageInputTextView(Gtk.TextView, EventHelper):
     '''
     A Gtk.Textview for chat message input
     '''
     def __init__(self) -> None:
         Gtk.TextView.__init__(self)
+        EventHelper.__init__(self)
+
         self.set_border_width(3)
         self.set_accepts_tab(True)
         self.set_editable(True)
@@ -78,7 +84,10 @@ class MessageInputTextView(Gtk.TextView):
         self.undo_pressed: bool = False
 
         self._contact: Optional[ChatContactT] = None
-        self._drafts: dict[tuple[str, JID], str] = {}
+
+        # XEP-0308 Message Correction
+        self._correcting: dict[ChatContactT, bool] = defaultdict(lambda: False)
+        self._last_message_id: dict[ChatContactT, Optional[str]] = {}
 
         self._chat_action_processor = ChatActionProcessor(self)
 
@@ -94,11 +103,86 @@ class MessageInputTextView(Gtk.TextView):
         self.connect('destroy', self._on_destroy)
         self.connect('populate-popup', self._on_populate_popup)
 
+        self.register_events([
+            ('message-sent', ged.GUI2, self._on_message_sent)
+        ])
+
         self._language_handler_id = self._init_spell_checker()
         app.settings.connect_signal('use_speller', self._on_toggle_spell_check)
 
         app.plugin_manager.gui_extension_point('message_input', self)
 
+    @property
+    def is_correcting(self) -> bool:
+        if self._contact is None:
+            return False
+
+        return self._correcting[self._contact]
+
+    def get_last_message_id(self, contact: ChatContactT) -> Optional[str]:
+        return self._last_message_id.get(contact)
+
+    def toggle_message_correction(self) -> None:
+        self.clear()
+        self.grab_focus()
+
+        if self.is_correcting:
+            self._set_correcting(False)
+            self.get_style_context().remove_class('gajim-msg-correcting')
+            return
+
+        assert self._contact is not None
+        last_message_id = self._last_message_id.get(self._contact)
+        if last_message_id is None:
+            return
+
+        message_row = app.storage.archive.get_last_correctable_message(
+            self._contact.account, self._contact.jid, last_message_id)
+        if message_row is None or message_row.message is None:
+            return
+
+        self._set_correcting(True)
+        self.get_style_context().add_class('gajim-msg-correcting')
+        self.insert_text(message_row.message)
+
+    def try_message_correction(self, message: str) -> Optional[str]:
+        assert self._contact is not None
+        if not self.is_correcting:
+            return None
+
+        self.toggle_message_correction()
+
+        correct_id = self._last_message_id.get(self._contact)
+        message_row = app.storage.archive.get_last_correctable_message(
+            self._contact.account, self._contact.jid, correct_id)
+        if message_row is None:
+            log.info('Trying to correct message older than threshold')
+            return None
+
+        if message_row.message == message:
+            log.info('Trying to correct message with original text')
+            return None
+
+        return correct_id
+
+    def _set_correcting(self, state: bool) -> None:
+        assert self._contact is not None
+        self._correcting[self._contact] = state
+
+    def _on_message_sent(self, event: MessageSent) -> None:
+        if not event.message:
+            return
+
+        if event.correct_id is None:
+            # This wasn't a corrected message
+            assert self._contact is not None
+            oob_url = event.additional_data.get_value('gajim', 'oob_url')
+            if oob_url == event.message:
+                # Don't allow to correct HTTP Upload file transfer URLs
+                self._last_message_id[self._contact] = None
+            else:
+                self._last_message_id[self._contact] = event.message_id
+
     def _init_spell_checker(self) -> int:
         if not app.is_installed('GSPELL'):
             return 0
@@ -141,7 +225,7 @@ class MessageInputTextView(Gtk.TextView):
         assert isinstance(lang, str)
         lang = Gspell.language_lookup(lang)
         if lang is None:
-            lang = Gspell.language_get_default()
+            return Gspell.language_get_default()
         return lang
 
     def _on_language_changed(self,
@@ -156,19 +240,19 @@ class MessageInputTextView(Gtk.TextView):
 
     def switch_contact(self, contact: ChatContactT) -> None:
         if self._contact is not None:
-            account = self._contact.account
-            jid = self._contact.jid
-            if self.has_text:
-                self._drafts[(account, jid)] = self.get_text()
-            else:
-                self._drafts.pop((account, jid), None)
+            app.storage.drafts.set(self._contact, self.get_text())
 
         self.clear()
-        draft = self._drafts.get((contact.account, contact.jid))
+        draft = app.storage.drafts.get(contact)
         if draft is not None:
             self.insert_text(draft)
 
         self._contact = contact
+        if self.is_correcting:
+            self.get_style_context().add_class('gajim-msg-correcting')
+        else:
+            self.get_style_context().remove_class('gajim-msg-correcting')
+
         self._chat_action_processor.switch_contact(contact)
         self._set_spell_checker_language(contact)
 
@@ -228,7 +312,7 @@ class MessageInputTextView(Gtk.TextView):
 
         self._clear_tags()
 
-        if len(text) > 10000:
+        if len(text) > MAX_MESSAGE_LENGTH:
             # Limit message styling processing
             return
 
@@ -263,11 +347,7 @@ class MessageInputTextView(Gtk.TextView):
     def get_text(self) -> str:
         buf = self.get_buffer()
         start, end = buf.get_bounds()
-        text = self.get_buffer().get_text(start, end, True)
-        return text
-
-    def get_draft(self, account: str, jid: JID) -> Optional[str]:
-        return self._drafts.get((account, jid))
+        return self.get_buffer().get_text(start, end, True)
 
     def _on_toggle_spell_check(self, *args: Any) -> None:
         if not app.is_installed('GSPELL'):
@@ -337,60 +417,6 @@ class MessageInputTextView(Gtk.TextView):
             buf.get_iter_at_offset(start_offset),
             buf.get_iter_at_offset(end_offset + 2))
 
-    def replace_emojis(self) -> None:
-        if sys.platform != 'darwin':
-            # TODO: Remove if colored emoji rendering works well on MacOS
-            return
-
-        theme = app.settings.get('emoticons_theme')
-        if not theme or theme == 'font':
-            return
-
-        def _replace(anchor: Gtk.TextChildAnchor) -> None:
-            if anchor is None:
-                return
-            image = anchor.get_widgets()[0]
-            if hasattr(image, 'codepoint'):
-                # found emoji
-                codepoint = getattr(image, 'codepoint')
-                self._replace_char_at_iter(iter_, codepoint)
-                image.destroy()
-
-        iter_ = self.get_buffer().get_start_iter()
-        _replace(iter_.get_child_anchor())
-
-        while iter_.forward_char():
-            _replace(iter_.get_child_anchor())
-
-    def _replace_char_at_iter(self,
-                              iter_: Gtk.TextIter,
-                              new_char: str
-                              ) -> None:
-        buf = self.get_buffer()
-        iter_2 = iter_.copy()
-        iter_2.forward_char()
-        buf.delete(iter_, iter_2)
-        buf.insert(iter_, new_char)
-
-    def insert_emoji(self, codepoint: str, pixbuf: GdkPixbuf.Pixbuf) -> None:
-        buf = self.get_buffer()
-        if buf.get_char_count():
-            # buffer contains text
-            buf.insert_at_cursor(' ')
-
-        insert_mark = buf.get_insert()
-        insert_iter = buf.get_iter_at_mark(insert_mark)
-
-        if pixbuf is None:
-            buf.insert(insert_iter, codepoint)
-        else:
-            anchor = buf.create_child_anchor(insert_iter)
-            image = Gtk.Image.new_from_pixbuf(pixbuf)
-            setattr(image, 'codepoint', codepoint)
-            image.show()
-            self.add_child_at_anchor(image, anchor)
-        buf.insert_at_cursor(' ')
-
     def clear(self, *args: Any) -> None:
         buf = self.get_buffer()
         start, end = buf.get_bounds()
