@@ -13,55 +13,55 @@
 # You should have received a copy of the GNU General Public License
 # along with Gajim. If not, see <http://www.gnu.org/licenses/>.
 
-from typing import cast
 from typing import Any
+from typing import cast
 from typing import Optional
 from typing import Union
 
 import logging
 
-from gi.repository import Gio
-from gi.repository import Gdk
-from gi.repository import Gtk
-from gi.repository import GLib
-
 import cairo
-
+from gi.repository import Gdk
+from gi.repository import Gio
+from gi.repository import GLib
+from gi.repository import Gtk
 from nbxmpp.errors import StanzaError
-from nbxmpp.namespaces import Namespace
-from nbxmpp.modules.vcard4 import VCard
 from nbxmpp.modules.user_avatar import Avatar
+from nbxmpp.modules.vcard4 import VCard
+from nbxmpp.namespaces import Namespace
 from nbxmpp.task import Task
 
 from gajim.common import app
+from gajim.common.client import Client
 from gajim.common.const import AvatarSize
+from gajim.common.const import SimpleClientState
 from gajim.common.i18n import _
-from gajim.common.i18n import Q_
+from gajim.common.i18n import p_
 
-from gajim.gui.avatar import clip_circle
-from gajim.gui.avatar_selector import AvatarSelector
-from gajim.gui.dialogs import ErrorDialog
-from gajim.gui.filechoosers import AvatarChooserDialog
-from gajim.gui.builder import get_builder
-from gajim.gui.vcard_grid import VCardGrid
-from gajim.gui.util import scroll_to_end
+from gajim.gtk.avatar import clip_circle
+from gajim.gtk.avatar_selector import AvatarSelector
+from gajim.gtk.builder import get_builder
+from gajim.gtk.dialogs import ErrorDialog
+from gajim.gtk.filechoosers import AvatarChooserDialog
+from gajim.gtk.util import scroll_to_end
+from gajim.gtk.vcard_grid import VCardGrid
 
-log = logging.getLogger('gajim.gui.profile')
+log = logging.getLogger('gajim.gtk.profile')
 
 MENU_DICT = {
-    'fn': Q_('?profile:Full Name'),
+    'fn': p_('Profile', 'Full Name'),
     'bday': _('Birthday'),
-    'gender': Q_('?profile:Gender'),
-    'adr': Q_('?profile:Address'),
+    'gender': p_('Profile', 'Gender'),
+    'adr': p_('Profile', 'Address'),
     'email': _('Email'),
-    'impp': Q_('?profile:IM Address'),
+    'impp': p_('Profile', 'IM Address'),
     'tel': _('Phone No.'),
-    'org': Q_('?profile:Organisation'),
-    'title': Q_('?profile:Title'),
-    'role': Q_('?profile:Role'),
+    'org': p_('Profile', 'Organisation'),
+    'title': p_('Profile', 'Title'),
+    'role': p_('Profile', 'Role'),
     'url': _('URL'),
-    'key': Q_('?profile:Public Encryption Key'),
-    'note': Q_('?profile:Note'),
+    'key': p_('Profile', 'Public Encryption Key'),
+    'note': p_('Profile', 'Note'),
 }
 
 
@@ -81,6 +81,9 @@ class ProfileWindow(Gtk.ApplicationWindow):
         self._jid = app.get_jid_from_account(account)
 
         self._client = app.get_client(self.account)
+        self._client.connect_signal(
+            'state-changed', self._on_client_state_changed)
+
         self._contact = self._client.get_module('Contacts').get_contact(
             self._jid)
 
@@ -147,6 +150,16 @@ class ProfileWindow(Gtk.ApplicationWindow):
         self._ui.privacy_popover.destroy()
         app.check_finalize(self)
 
+    def _on_client_state_changed(self,
+                                 _client: Client,
+                                 _signal_name: str,
+                                 state: SimpleClientState
+                                 ) -> None:
+
+        self._ui.save_button.set_sensitive(state.is_connected)
+        self._ui.save_button.set_tooltip_text(
+            _('Not connected') if not state.is_connected else '')
+
     def _on_access_model_received(self, task: Task) -> None:
         namespace = task.get_user_data()
 
@@ -172,10 +185,10 @@ class ProfileWindow(Gtk.ApplicationWindow):
 
     def _on_vcard_received(self, task: Task):
         try:
-            self._current_vcard = cast(VCard, task.finish())
+            self._current_vcard = cast(VCard | None, task.finish())
         except StanzaError as error:
             log.info('Error loading VCard: %s', error)
-            self._current_vcard = VCard()
+            self._current_vcard = None
 
         if self._current_vcard is None:
             self._current_vcard = VCard()
@@ -237,7 +250,7 @@ class ProfileWindow(Gtk.ApplicationWindow):
         self._ui.nickname_entry.set_sensitive(False)
         self._ui.avatar_image.set_from_surface(self._current_avatar)
         self._ui.nickname_entry.set_text(app.nicks[self.account])
-        self._vcard_grid.set_vcard(self._current_vcard.copy())
+        self._vcard_grid.set_vcard(self._current_vcard.copy())  # pyright: ignore # noqa: E501
         self._new_avatar = False
 
     def _on_save_clicked(self, _button: Gtk.Button) -> None:
@@ -256,7 +269,7 @@ class ProfileWindow(Gtk.ApplicationWindow):
         self._vcard_grid.sort()
 
         vcard = self._vcard_grid.get_vcard()
-        self._current_vcard = vcard.copy()
+        self._current_vcard = vcard.copy()  # pyright: ignore
 
         client = app.get_client(self.account)
         client.get_module('VCard4').set_vcard(
@@ -319,18 +332,19 @@ class ProfileWindow(Gtk.ApplicationWindow):
         self._new_avatar = None
 
     def _on_edit_avatar(self, button: Gtk.Button) -> None:
-        def _on_file_selected(path: str) -> None:
+        def _on_file_selected(paths: list[str]) -> None:
             if self._avatar_selector is None:
                 self._avatar_selector = AvatarSelector()
                 self._ui.avatar_selector_box.add(self._avatar_selector)
 
-            self._avatar_selector.prepare_crop_area(path)
+            self._avatar_selector.prepare_crop_area(paths[0])
             self._ui.avatar_update_button.set_sensitive(
                 self._avatar_selector.get_prepared())
             self._ui.profile_stack.set_visible_child_name('avatar_selector')
 
         AvatarChooserDialog(_on_file_selected,
-                            transient_for=button.get_toplevel())
+                            transient_for=cast(Gtk.Window,
+                                               button.get_toplevel()))
 
     def _on_cancel_update_avatar(self, _button: Gtk.Button) -> None:
         self._ui.profile_stack.set_visible_child_name('profile')
@@ -355,7 +369,7 @@ class ProfileWindow(Gtk.ApplicationWindow):
             return
 
         self._new_avatar = Avatar()
-        self._new_avatar.add_image_source(data, 'image/png', height, width)
+        self._new_avatar.add_image_source(data, 'image/png', height, width)  # pyright: ignore  # noqa: E501
 
         scale = self.get_scale_factor()
         surface = app.app.avatar_storage.surface_from_filename(
