@@ -15,15 +15,14 @@
 from typing import Optional
 
 from gi.repository import Gtk
-
 from nbxmpp.protocol import JID
 
 from gajim.common import app
 from gajim.common import i18n
 from gajim.common.i18n import _
 
-from .dialogs import InformationDialog
-from .builder import get_builder
+from gajim.gtk.builder import get_builder
+from gajim.gtk.dialogs import InformationDialog
 
 
 class RosterItemExchange(Gtk.ApplicationWindow):
@@ -142,7 +141,11 @@ class RosterItemExchange(Gtk.ApplicationWindow):
                 self.show_all()
                 assert isinstance(self._model, Gtk.ListStore)
                 iter_ = self._model.append()
-                self._model.set(iter_, 0, True, 1, jid, 2, name, 3, groups)
+                self._model.set(iter_,
+                                {0: True,
+                                 1: jid,
+                                 2: name,
+                                 3: groups})
 
         self._ui.accept_button.set_label(_('Add'))
 
@@ -169,7 +172,11 @@ class RosterItemExchange(Gtk.ApplicationWindow):
                 self.show_all()
                 assert isinstance(self._model, Gtk.ListStore)
                 iter_ = self._model.append()
-                self._model.set(iter_, 0, True, 1, jid, 2, name, 3, groups)
+                self._model.set(iter_,
+                                {0: True,
+                                 1: jid,
+                                 2: name,
+                                 3: groups})
 
         self._ui.accept_button.set_label(_('Modify'))
 
@@ -190,7 +197,11 @@ class RosterItemExchange(Gtk.ApplicationWindow):
                 self.show_all()
                 assert isinstance(self._model, Gtk.ListStore)
                 iter_ = self._model.append()
-                self._model.set(iter_, 0, True, 1, jid, 2, name, 3, groups)
+                self._model.set(iter_,
+                                {0: True,
+                                 1: jid,
+                                 2: name,
+                                 3: groups})
 
         self._ui.accept_button.set_label(_('Delete'))
 
@@ -226,9 +237,11 @@ class RosterItemExchange(Gtk.ApplicationWindow):
                         groups=groups,
                         auto_auth=True)
                 iter_ = model.iter_next(iter_)
-            InformationDialog(i18n.ngettext('Added %d contact',
-                                            'Added %d contacts',
-                                            count, str(count), str(count)))
+            InformationDialog(i18n.ngettext('Added %s contact',
+                                            'Added %s contacts',
+                                            count,
+                                            str(count),
+                                            str(count)))
         elif self._action == 'modify':
             count = 0
             while iter_:
@@ -254,9 +267,11 @@ class RosterItemExchange(Gtk.ApplicationWindow):
                     self._client.get_module('Presence').unsubscribe(jid)
                     self._client.get_module('Roster').delete_item(jid)
                 iter_ = model.iter_next(iter_)
-            InformationDialog(i18n.ngettext('Removed %d contact',
-                                            'Removed %d contacts',
-                                            count, str(count), str(count)))
+            InformationDialog(i18n.ngettext('Removed %s contact',
+                                            'Removed %s contacts',
+                                            count,
+                                            str(count),
+                                            str(count)))
         self.destroy()
 
     def _on_cancel_button_clicked(self, _button: Gtk.Button) -> None:
