@@ -14,8 +14,8 @@
 
 from __future__ import annotations
 
-from typing import cast
 from typing import Any
+from typing import cast
 from typing import Optional
 from typing import Union
 
@@ -24,40 +24,42 @@ from enum import IntEnum
 
 from gi.repository import Gdk
 from gi.repository import GdkPixbuf
-from gi.repository import Gtk
 from gi.repository import GLib
+from gi.repository import Gtk
 from gi.repository import Pango
-
 from nbxmpp import JID
+from nbxmpp.errors import CancelledError
 from nbxmpp.errors import is_error
 from nbxmpp.errors import StanzaError
 from nbxmpp.errors import TimeoutStanzaError
-from nbxmpp.errors import CancelledError
 from nbxmpp.structs import DiscoInfo
 from nbxmpp.structs import MuclumbusItem
 from nbxmpp.structs import MuclumbusResult
 from nbxmpp.task import Task
 
 from gajim.common import app
+from gajim.common.const import AvatarSize
 from gajim.common.const import Direction
-from gajim.common.helpers import validate_jid
-from gajim.common.helpers import to_user_string
+from gajim.common.const import MUC_DISCO_ERRORS
+from gajim.common.const import URIType
 from gajim.common.helpers import get_group_chat_nick
+from gajim.common.helpers import parse_uri
+from gajim.common.helpers import to_user_string
+from gajim.common.helpers import validate_jid
 from gajim.common.i18n import _
 from gajim.common.i18n import get_rfc5646_lang
-from gajim.common.const import AvatarSize
-from gajim.common.const import MUC_DISCO_ERRORS
-from gajim.common.modules.util import as_task
 from gajim.common.modules.contacts import BareContact
 from gajim.common.modules.contacts import GroupchatContact
+from gajim.common.modules.util import as_task
 
-from .menus import get_start_chat_row_menu
-from .chat_filter import ChatFilter
-from .groupchat_info import GroupChatInfoScrolled
-from .groupchat_nick import NickChooser
-from .builder import get_builder
-from .util import GajimPopover, get_icon_name
-from .util import AccountBadge
+from gajim.gtk.builder import get_builder
+from gajim.gtk.chat_filter import ChatFilter
+from gajim.gtk.groupchat_info import GroupChatInfoScrolled
+from gajim.gtk.groupchat_nick import NickChooser
+from gajim.gtk.menus import get_start_chat_row_menu
+from gajim.gtk.util import AccountBadge
+from gajim.gtk.util import GajimPopover
+from gajim.gtk.util import get_icon_name
 
 ContactT = Union[BareContact, GroupchatContact]
 
@@ -69,7 +71,7 @@ class Search(IntEnum):
 
 class StartChatDialog(Gtk.ApplicationWindow):
     def __init__(self,
-                 jid: Optional[str] = None,
+                 initial_jid: Optional[str] = None,
                  initial_message: Optional[str] = None
                  ) -> None:
 
@@ -143,9 +145,9 @@ class StartChatDialog(Gtk.ApplicationWindow):
             self._load_contacts(rows)
 
         self._initial_message: dict[str, Optional[str]] = {}
-        if jid is not None:
-            self._initial_message[jid] = initial_message
-            self._ui.search_entry.set_text(jid)
+        if initial_jid is not None:
+            self._initial_message[initial_jid] = initial_message
+            self._ui.search_entry.set_text(initial_jid)
 
         self.select_first_row()
         self._ui.connect_signals(self)
@@ -337,6 +339,9 @@ class StartChatDialog(Gtk.ApplicationWindow):
         self._ui.listbox.invalidate_filter()
 
     def _start_new_chat(self, row: ContactRow) -> None:
+        if row.jid is None:
+            return
+
         if row.is_new:
             try:
                 validate_jid(row.jid)
@@ -347,7 +352,6 @@ class StartChatDialog(Gtk.ApplicationWindow):
             self._disco_info(row)
             return
 
-        jid = JID.from_string(row.jid)
         if row.groupchat:
             if not app.account_is_available(row.account):
                 self._show_error_page(_('You can not join a group chat '
@@ -355,23 +359,22 @@ class StartChatDialog(Gtk.ApplicationWindow):
                 return
 
             self.ready_to_destroy = True
-            if app.window.chat_exists(row.account, jid):
-                app.window.select_chat(row.account, jid)
+            if app.window.chat_exists(row.account, row.jid):
+                app.window.select_chat(row.account, row.jid)
                 self.destroy()
                 return
 
             self.ready_to_destroy = False
             self._redirected = False
-            self._disco_muc(row.account, jid, request_vcard=row.is_new)
+            self._disco_muc(row.account, row.jid, request_vcard=row.is_new)
 
         else:
-            initial_message = self._initial_message.get(str(jid))
+            initial_message = self._initial_message.get(str(row.jid))
             app.window.add_chat(
                 row.account,
-                jid,
+                row.jid,
                 'contact',
                 select=True,
-                workspace='current',
                 message=initial_message)
             self.ready_to_destroy = True
             self.destroy()
@@ -479,7 +482,7 @@ class StartChatDialog(Gtk.ApplicationWindow):
         jid = self._muc_info_box.get_jid()
         nickname = self._nick_chooser.get_text()
         assert account
-        app.interface.show_add_join_groupchat(
+        app.window.show_add_join_groupchat(
             account, str(jid), nickname=nickname)
 
         self.ready_to_destroy = True
@@ -550,6 +553,11 @@ class StartChatDialog(Gtk.ApplicationWindow):
             return
 
         search_text = search_entry.get_text()
+        uri = parse_uri(search_text)
+        if uri.type == URIType.XMPP:
+            search_entry.set_text(uri.data['jid'])
+            return
+
         if '@' in search_text:
             try:
                 validate_jid(search_text)
@@ -824,7 +832,7 @@ class ContactRow(Gtk.ListBoxRow):
         if not self.groupchat:
             return
 
-        if event.button != 3:  # right click
+        if event.button != Gdk.BUTTON_SECONDARY:
             return
 
         menu = get_start_chat_row_menu(self.account, self.jid)
@@ -855,8 +863,7 @@ class ContactRow(Gtk.ListBoxRow):
     def get_search_text(self) -> str:
         if self.contact is None and not self.groupchat:
             return str(self.jid)
-        if self.show_account:
-            return f'{self.name} {self.jid} {self.account_label}'
+
         return f'{self.name} {self.jid}'
 
 
