@@ -17,8 +17,11 @@ from typing import Optional
 from pathlib import Path
 
 from gajim.common import configpaths
-from gajim.plugins import plugins_i18n
-from gajim.gui.builder import Builder
+
+from gajim.gtk.builder import Builder
+
+from .plugins_i18n import _
+from .plugins_i18n import DOMAIN
 
 
 class GajimPluginActivateException(Exception):
@@ -28,10 +31,10 @@ class GajimPluginActivateException(Exception):
 
 
 def get_builder(file_name: str, widgets: Optional[list[str]] = None) -> Builder:
-    return Builder(file_name,     # pyright: ignore
-                   widgets,
-                   domain=plugins_i18n.DOMAIN,
-                   gettext_=plugins_i18n._)
+    return Builder(file_name,
+                   widgets,  # pyright: ignore
+                   domain=DOMAIN,
+                   gettext_=_)
 
 
 def is_shipped_plugin(path: Path) -> bool:
